@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -190,7 +190,7 @@ namespace Orthanc
       {
       }
 
-      DicomTag2(const DicomTag& tag) :
+      explicit DicomTag2(const DicomTag& tag) :
         DicomTag(tag)
       {
       }
@@ -212,7 +212,7 @@ namespace Orthanc
     {
       assert(target.find(tags[i].name_) == target.end());
       
-      target[tags[i].name_] = tags[i].tag_;
+      target[tags[i].name_] = DicomTag2(tags[i].tag_);
     }
   }
 
@@ -247,6 +247,51 @@ namespace Orthanc
     content_.clear();
   }
 
+  void DicomMap::SetNullValue(uint16_t group, uint16_t element)
+  {
+    SetValueInternal(group, element, new DicomValue);
+  }
+
+  void DicomMap::SetNullValue(const DicomTag &tag)
+  {
+    SetValueInternal(tag.GetGroup(), tag.GetElement(), new DicomValue);
+  }
+
+  void DicomMap::SetValue(uint16_t group, uint16_t element, const DicomValue &value)
+  {
+    SetValueInternal(group, element, value.Clone());
+  }
+
+  void DicomMap::SetValue(const DicomTag &tag, const DicomValue &value)
+  {
+    SetValueInternal(tag.GetGroup(), tag.GetElement(), value.Clone());
+  }
+
+  void DicomMap::SetValue(const DicomTag &tag, const std::string &str, bool isBinary)
+  {
+    SetValueInternal(tag.GetGroup(), tag.GetElement(), new DicomValue(str, isBinary));
+  }
+
+  void DicomMap::SetValue(uint16_t group, uint16_t element, const std::string &str, bool isBinary)
+  {
+    SetValueInternal(group, element, new DicomValue(str, isBinary));
+  }
+
+  bool DicomMap::HasTag(uint16_t group, uint16_t element) const
+  {
+    return HasTag(DicomTag(group, element));
+  }
+
+  bool DicomMap::HasTag(const DicomTag &tag) const
+  {
+    return content_.find(tag) != content_.end();
+  }
+
+  const DicomValue &DicomMap::GetValue(uint16_t group, uint16_t element) const
+  {
+    return GetValue(DicomTag(group, element));
+  }
+
 
   static void ExtractTags(DicomMap& result,
                           const DicomMap::Content& source,
@@ -287,6 +332,16 @@ namespace Orthanc
   }
 
 
+  DicomMap::~DicomMap()
+  {
+    Clear();
+  }
+
+  size_t DicomMap::GetSize() const
+  {
+    return content_.size();
+  }
+
 
   DicomMap* DicomMap::Clone() const
   {
@@ -326,6 +381,11 @@ namespace Orthanc
     }
   }
 
+  const DicomValue *DicomMap::TestAndGetValue(uint16_t group, uint16_t element) const
+  {
+    return TestAndGetValue(DicomTag(group, element));
+  }
+
 
   const DicomValue* DicomMap::TestAndGetValue(const DicomTag& tag) const
   {
@@ -493,15 +553,23 @@ namespace Orthanc
   }
 
 
-  static uint16_t ReadUnsignedInteger16(const char* dicom)
+  static uint16_t ReadLittleEndianUint16(const char* dicom)
   {
-    return le16toh(*reinterpret_cast<const uint16_t*>(dicom));
+    const uint8_t* p = reinterpret_cast<const uint8_t*>(dicom);
+
+    return (static_cast<uint16_t>(p[0]) |
+            (static_cast<uint16_t>(p[1]) << 8));
   }
 
 
-  static uint32_t ReadUnsignedInteger32(const char* dicom)
+  static uint32_t ReadLittleEndianUint32(const char* dicom)
   {
-    return le32toh(*reinterpret_cast<const uint32_t*>(dicom));
+    const uint8_t* p = reinterpret_cast<const uint8_t*>(dicom);
+
+    return (static_cast<uint32_t>(p[0]) |
+            (static_cast<uint32_t>(p[1]) << 8) |
+            (static_cast<uint32_t>(p[2]) << 16) |
+            (static_cast<uint32_t>(p[3]) << 24));
   }
 
 
@@ -670,8 +738,8 @@ namespace Orthanc
       return false;
     }
 
-    tag = DicomTag(ReadUnsignedInteger16(dicom + position),
-                   ReadUnsignedInteger16(dicom + position + 2));
+    tag = DicomTag(ReadLittleEndianUint16(dicom + position),
+                   ReadLittleEndianUint16(dicom + position + 2));
 
     vr = StringToValueRepresentation(std::string(dicom + position + 4, 2), true);
     if (vr == ValueRepresentation_NotSupported)
@@ -679,41 +747,40 @@ namespace Orthanc
       return false;
     }
 
-    if (vr == ValueRepresentation_OtherByte ||
-        vr == ValueRepresentation_OtherDouble ||
-        vr == ValueRepresentation_OtherFloat ||
-        vr == ValueRepresentation_OtherLong ||
-        vr == ValueRepresentation_OtherWord ||
-        vr == ValueRepresentation_Sequence ||
-        vr == ValueRepresentation_UnlimitedCharacters ||
-        vr == ValueRepresentation_UniversalResource ||
-        vr == ValueRepresentation_UnlimitedText ||
-        vr == ValueRepresentation_Unknown)    // Note that "UN" should never appear in the Meta Information
-    {
-      if (position + 12 > size)
-      {
-        return false;
-      }
-
-      uint32_t length = ReadUnsignedInteger32(dicom + position + 8);
-
-      if (position + 12 + length > size)
-      {
-        return false;
-      }
-
-      value.assign(dicom + position + 12, length);
-      position += (12 + length);
-    }
-    else
+    // http://dicom.nema.org/medical/dicom/current/output/chtml/part05/chapter_7.html#sect_7.1.2
+    if (vr == ValueRepresentation_ApplicationEntity   /* AE */ ||
+        vr == ValueRepresentation_AgeString           /* AS */ ||
+        vr == ValueRepresentation_AttributeTag        /* AT */ ||
+        vr == ValueRepresentation_CodeString          /* CS */ ||
+        vr == ValueRepresentation_Date                /* DA */ ||
+        vr == ValueRepresentation_DecimalString       /* DS */ ||
+        vr == ValueRepresentation_DateTime            /* DT */ ||
+        vr == ValueRepresentation_FloatingPointSingle /* FL */ ||
+        vr == ValueRepresentation_FloatingPointDouble /* FD */ ||
+        vr == ValueRepresentation_IntegerString       /* IS */ ||
+        vr == ValueRepresentation_LongString          /* LO */ ||
+        vr == ValueRepresentation_LongText            /* LT */ ||
+        vr == ValueRepresentation_PersonName          /* PN */ ||
+        vr == ValueRepresentation_ShortString         /* SH */ ||
+        vr == ValueRepresentation_SignedLong          /* SL */ ||
+        vr == ValueRepresentation_SignedShort         /* SS */ ||
+        vr == ValueRepresentation_ShortText           /* ST */ ||
+        vr == ValueRepresentation_Time                /* TM */ ||
+        vr == ValueRepresentation_UniqueIdentifier    /* UI */ ||
+        vr == ValueRepresentation_UnsignedLong        /* UL */ ||
+        vr == ValueRepresentation_UnsignedShort       /* US */)
     {
+      /**
+       * This is Table 7.1-2. "Data Element with Explicit VR of AE,
+       * AS, AT, CS, DA, DS, DT, FL, FD, IS, LO, LT, PN, SH, SL, SS,
+       * ST, TM, UI, UL and US"
+       **/
       if (position + 8 > size)
       {
         return false;
       }
 
-      uint16_t length = ReadUnsignedInteger16(dicom + position + 6);
-
+      uint16_t length = ReadLittleEndianUint16(dicom + position + 6);
       if (position + 8 + length > size)
       {
         return false;
@@ -722,6 +789,32 @@ namespace Orthanc
       value.assign(dicom + position + 8, length);
       position += (8 + length);
     }
+    else
+    {
+      /**
+       * This is Table 7.1-1. "Data Element with Explicit VR other
+       * than as shown in Table 7.1-2"
+       **/
+      if (position + 12 > size)
+      {
+        return false;
+      }
+      
+      uint16_t reserved = ReadLittleEndianUint16(dicom + position + 6);
+      if (reserved != 0)
+      {
+        return false;
+      }
+
+      uint32_t length = ReadLittleEndianUint32(dicom + position + 8);
+      if (position + 12 + length > size)
+      {
+        return false;
+      }
+
+      value.assign(dicom + position + 12, length);
+      position += (12 + length);
+    }
 
     if (!ValidateTag(vr, value))
     {
@@ -788,7 +881,7 @@ namespace Orthanc
       return false;
     }
 
-    size_t stopPosition = position + ReadUnsignedInteger32(value.c_str());
+    size_t stopPosition = position + ReadLittleEndianUint32(value.c_str());
     if (stopPosition > size)
     {
       return false;
@@ -1230,43 +1323,43 @@ namespace Orthanc
           bool supported = true;
           
           std::string s;
-          for (Json::Value::ArrayIndex i = 0; i < value.size() && supported; i++)
+          for (Json::Value::ArrayIndex j = 0; j < value.size() && supported; j++)
           {
             if (!s.empty())
             {
               s += '\\';
             }
 
-            switch (value[i].type())
+            switch (value[j].type())
             {
               case Json::objectValue:
                 if (vr == ValueRepresentation_PersonName &&
-                    value[i].type() == Json::objectValue)
+                    value[j].type() == Json::objectValue)
                 {
-                  if (value[i].isMember(ALPHABETIC) &&
-                      value[i][ALPHABETIC].type() == Json::stringValue)
+                  if (value[j].isMember(ALPHABETIC) &&
+                      value[j][ALPHABETIC].type() == Json::stringValue)
                   {
-                    s += value[i][ALPHABETIC].asString();
+                    s += value[j][ALPHABETIC].asString();
                   }
 
                   bool hasIdeographic = false;
                   
-                  if (value[i].isMember(IDEOGRAPHIC) &&
-                      value[i][IDEOGRAPHIC].type() == Json::stringValue)
+                  if (value[j].isMember(IDEOGRAPHIC) &&
+                      value[j][IDEOGRAPHIC].type() == Json::stringValue)
                   {
-                    s += '=' + value[i][IDEOGRAPHIC].asString();
+                    s += '=' + value[j][IDEOGRAPHIC].asString();
                     hasIdeographic = true;
                   }
                   
-                  if (value[i].isMember(PHONETIC) &&
-                      value[i][PHONETIC].type() == Json::stringValue)
+                  if (value[j].isMember(PHONETIC) &&
+                      value[j][PHONETIC].type() == Json::stringValue)
                   {
                     if (!hasIdeographic)
                     {
                       s += '=';
                     }
                       
-                    s += '=' + value[i][PHONETIC].asString();
+                    s += '=' + value[j][PHONETIC].asString();
                   }
                 }
                 else
@@ -1278,19 +1371,19 @@ namespace Orthanc
                 break;
             
               case Json::stringValue:
-                s += value[i].asString();
+                s += value[j].asString();
                 break;
               
               case Json::intValue:
-                s += boost::lexical_cast<std::string>(value[i].asInt());
+                s += boost::lexical_cast<std::string>(value[j].asInt());
                 break;
               
               case Json::uintValue:
-                s += boost::lexical_cast<std::string>(value[i].asUInt());
+                s += boost::lexical_cast<std::string>(value[j].asUInt());
                 break;
               
               case Json::realValue:
-                s += boost::lexical_cast<std::string>(value[i].asDouble());
+                s += boost::lexical_cast<std::string>(value[j].asDouble());
                 break;
               
               default:
