@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -93,6 +93,17 @@ namespace Orthanc
   {
     try
     {
+      if (cond.bad() &&
+          cond == DUL_ASSOCIATIONREJECTED)
+      {
+        T_ASC_RejectParameters rej;
+        ASC_getRejectParameters(params_, &rej);
+
+        OFString str;
+        CLOG(TRACE, DICOM) << "Association Rejected:" << std::endl
+                           << ASC_printRejectParameters(str, &rej);
+      }
+      
       CheckCondition(cond, parameters, "connecting");
     }
     catch (OrthancException&)
@@ -105,6 +116,10 @@ namespace Orthanc
     
   void DicomAssociation::CloseInternal()
   {
+#if ORTHANC_ENABLE_SSL == 1
+    tls_.reset(NULL);  // Transport layer must be destroyed before the association itself
+#endif
+    
     if (assoc_ != NULL)
     {
       ASC_releaseAssociation(assoc_);
@@ -148,10 +163,10 @@ namespace Orthanc
     {
       if (found->second.find(syntax) != found->second.end())
       {
-        LOG(WARNING) << "The same transfer syntax ("
-                     << GetTransferSyntaxUid(syntax)
-                     << ") was accepted twice for the same abstract syntax UID ("
-                     << abstractSyntax << ")";
+        CLOG(WARNING, DICOM) << "The same transfer syntax ("
+                             << GetTransferSyntaxUid(syntax)
+                             << ") was accepted twice for the same abstract syntax UID ("
+                             << abstractSyntax << ")";
       }
       else
       {
@@ -170,7 +185,7 @@ namespace Orthanc
     catch (OrthancException& e)
     {
       // Don't throw exception in destructors
-      LOG(ERROR) << "Error while destroying a DICOM association: " << e.What();
+      CLOG(ERROR, DICOM) << "Error while destroying a DICOM association: " << e.What();
     }
   }
 
@@ -240,25 +255,56 @@ namespace Orthanc
            params_ == NULL &&
            assoc_ == NULL);
 
+#if ORTHANC_ENABLE_SSL == 1
+    assert(tls_.get() == NULL);
+#endif
+
     if (proposed_.empty())
     {
       throw OrthancException(ErrorCode_BadSequenceOfCalls,
                              "No presentation context was proposed");
     }
 
-    LOG(INFO) << "Opening a DICOM SCU connection from AET \""
-              << parameters.GetLocalApplicationEntityTitle() 
-              << "\" to AET \"" << parameters.GetRemoteModality().GetApplicationEntityTitle()
-              << "\" on host " << parameters.GetRemoteModality().GetHost()
-              << ":" << parameters.GetRemoteModality().GetPortNumber() 
-              << " (manufacturer: " << EnumerationToString(parameters.GetRemoteModality().GetManufacturer()) << ")";
+    std::string localAet = parameters.GetLocalApplicationEntityTitle();
+    if (parameters.GetRemoteModality().HasLocalAet())
+    {
+      localAet = parameters.GetRemoteModality().GetLocalAet();
+    }
+
+    CLOG(INFO, DICOM) << "Opening a DICOM SCU connection "
+                      << (parameters.GetRemoteModality().IsDicomTlsEnabled() ? "using DICOM TLS" : "without DICOM TLS")
+                      << " from AET \"" << localAet
+                      << "\" to AET \"" << parameters.GetRemoteModality().GetApplicationEntityTitle()
+                      << "\" on host " << parameters.GetRemoteModality().GetHost()
+                      << ":" << parameters.GetRemoteModality().GetPortNumber() 
+                      << " (manufacturer: " << EnumerationToString(parameters.GetRemoteModality().GetManufacturer()) << ")";
 
     CheckConnecting(parameters, ASC_initializeNetwork(NET_REQUESTOR, 0, /*opt_acse_timeout*/ acseTimeout, &net_));
-    CheckConnecting(parameters, ASC_createAssociationParameters(&params_, /*opt_maxReceivePDULength*/ ASC_DEFAULTMAXPDU));
+    CheckConnecting(parameters, ASC_createAssociationParameters(&params_, parameters.GetMaximumPduLength()));
+
+#if ORTHANC_ENABLE_SSL == 1
+    if (parameters.GetRemoteModality().IsDicomTlsEnabled())
+    {
+      try
+      {
+        assert(net_ != NULL &&
+               params_ != NULL);
+        
+        tls_.reset(Internals::InitializeDicomTls(net_, NET_REQUESTOR, parameters.GetOwnPrivateKeyPath(),
+                                                 parameters.GetOwnCertificatePath(),
+                                                 parameters.GetTrustedCertificatesPath()));
+      }
+      catch (OrthancException&)
+      {
+        CloseInternal();
+        throw;
+      }
+    }
+#endif
 
     // Set this application's title and the called application's title in the params
     CheckConnecting(parameters, ASC_setAPTitles(
-                      params_, parameters.GetLocalApplicationEntityTitle().c_str(),
+                      params_, localAet.c_str(),
                       parameters.GetRemoteModality().GetApplicationEntityTitle().c_str(), NULL));
 
     // Set the network addresses of the local and remote entities
@@ -279,7 +325,11 @@ namespace Orthanc
     CheckConnecting(parameters, ASC_setPresentationAddresses(params_, localHost, remoteHostAndPort));
 
     // Set various options
-    CheckConnecting(parameters, ASC_setTransportLayerType(params_, /*opt_secureConnection*/ false));
+#if ORTHANC_ENABLE_SSL == 1
+    CheckConnecting(parameters, ASC_setTransportLayerType(params_, (tls_.get() != NULL) /*opt_secureConnection*/));
+#else
+    CheckConnecting(parameters, ASC_setTransportLayerType(params_, false /*opt_secureConnection*/));
+#endif
 
     // Setup the list of proposed presentation contexts
     unsigned int presentationContextId = 1;
@@ -307,10 +357,25 @@ namespace Orthanc
       presentationContextId += 2;
     }
 
+    {
+      OFString str;
+      CLOG(TRACE, DICOM) << "Request Parameters:" << std::endl
+                         << ASC_dumpParameters(str, params_, ASC_ASSOC_RQ);
+    }
+
     // Do the association
     CheckConnecting(parameters, ASC_requestAssociation(net_, params_, &assoc_));
     isOpen_ = true;
 
+    {
+      OFString str;
+      CLOG(TRACE, DICOM) << "Connection Parameters: "
+                         << ASC_dumpConnectionParameters(str, assoc_);
+      CLOG(TRACE, DICOM) << "Association Parameters Negotiated:" << std::endl
+                         << ASC_dumpParameters(str, params_, ASC_ASSOC_AC);
+    }
+
+
     // Inspect the accepted transfer syntaxes
     LST_HEAD **l = &params_->DULparams.acceptedPresentationContext;
     if (*l != NULL)
@@ -328,9 +393,9 @@ namespace Orthanc
           }
           else
           {
-            LOG(WARNING) << "Unknown transfer syntax received from AET \""
-                         << parameters.GetRemoteModality().GetApplicationEntityTitle()
-                         << "\": " << pc->acceptedTransferSyntax;
+            CLOG(WARNING, DICOM) << "Unknown transfer syntax received from AET \""
+                                 << parameters.GetRemoteModality().GetApplicationEntityTitle()
+                                 << "\": " << pc->acceptedTransferSyntax;
           }
         }
             
@@ -593,11 +658,11 @@ namespace Orthanc
      * Send the "EVENT_REPORT_RQ" request
      **/
 
-    LOG(INFO) << "Reporting modality \""
-              << parameters.GetRemoteModality().GetApplicationEntityTitle()
-              << "\" about storage commitment transaction: " << transactionUid
-              << " (" << successSopClassUids.size() << " successes, " 
-              << failedSopClassUids.size() << " failures)";
+    CLOG(INFO, DICOM) << "Reporting modality \""
+                      << parameters.GetRemoteModality().GetApplicationEntityTitle()
+                      << "\" about storage commitment transaction: " << transactionUid
+                      << " (" << successSopClassUids.size() << " successes, " 
+                      << failedSopClassUids.size() << " failures)";
     const DIC_US messageId = association.GetDcmtkAssociation().nextMsgID++;
       
     {
@@ -647,6 +712,16 @@ namespace Orthanc
                                parameters.GetRemoteModality().GetApplicationEntityTitle());
       }
 
+      {
+        std::stringstream s;  // DcmObject::PrintHelper cannot be used with VS2008
+        dataset.print(s);
+
+        OFString str;
+        CLOG(TRACE, DICOM) << "Sending Storage Commitment Report:" << std::endl
+                           << DIMSE_dumpMessage(str, message, DIMSE_OUTGOING) << std::endl
+                           << s.str();
+      }
+
       if (!DIMSE_sendMessageUsingMemoryData(
             &association.GetDcmtkAssociation(), presID, &message, NULL /* status detail */,
             &dataset, NULL /* callback */, NULL /* callback context */,
@@ -675,6 +750,12 @@ namespace Orthanc
                                parameters.GetRemoteModality().GetApplicationEntityTitle());
       }
 
+      {
+        OFString str;
+        CLOG(TRACE, DICOM) << "Received Storage Commitment Report Response:" << std::endl
+                           << DIMSE_dumpMessage(str, message, DIMSE_INCOMING, NULL, presID);
+      }
+      
       const T_DIMSE_N_EventReportRSP& content = message.msg.NEventReportRSP;
       if (content.MessageIDBeingRespondedTo != messageId ||
           !(content.opts & O_NEVENTREPORT_AFFECTEDSOPCLASSUID) ||
@@ -756,10 +837,10 @@ namespace Orthanc
      * Send the "N_ACTION_RQ" request
      **/
 
-    LOG(INFO) << "Request to modality \""
-              << parameters.GetRemoteModality().GetApplicationEntityTitle()
-              << "\" about storage commitment for " << sopClassUids.size()
-              << " instances, with transaction UID: " << transactionUid;
+    CLOG(INFO, DICOM) << "Request to modality \""
+                      << parameters.GetRemoteModality().GetApplicationEntityTitle()
+                      << "\" about storage commitment for " << sopClassUids.size()
+                      << " instances, with transaction UID: " << transactionUid;
     const DIC_US messageId = association.GetDcmtkAssociation().nextMsgID++;
       
     {
@@ -794,6 +875,16 @@ namespace Orthanc
                                parameters.GetRemoteModality().GetApplicationEntityTitle());
       }
 
+      {
+        std::stringstream s;  // DcmObject::PrintHelper cannot be used with VS2008
+        dataset.print(s);
+
+        OFString str;
+        CLOG(TRACE, DICOM) << "Sending Storage Commitment Request:" << std::endl
+                           << DIMSE_dumpMessage(str, message, DIMSE_OUTGOING) << std::endl
+                           << s.str();
+      }
+
       if (!DIMSE_sendMessageUsingMemoryData(
             &association.GetDcmtkAssociation(), presID, &message, NULL /* status detail */,
             &dataset, NULL /* callback */, NULL /* callback context */,
@@ -836,6 +927,12 @@ namespace Orthanc
                                parameters.GetRemoteModality().GetApplicationEntityTitle());
       }
 
+      {
+        OFString str;
+        CLOG(TRACE, DICOM) << "Received Storage Commitment Request Response:" << std::endl
+                           << DIMSE_dumpMessage(str, message, DIMSE_INCOMING, NULL, presID);
+      }
+
       if (content.DimseStatus != 0 /* success */)
       {
         throw OrthancException(ErrorCode_NetworkProtocol, "Storage commitment - "
