@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -351,7 +351,8 @@ DCMTK_TO_CTYPE_CONVERTER(DcmtkToFloat64Converter, Float64, DcmFloatingPointDoubl
     
     DcmEVR evr = ToDcmtkBridge::Convert(vr);
 
-    LOG(INFO) << "Registering tag in dictionary: " << tag << " " << (DcmVR(evr).getValidVRName()) << " " 
+    LOG(INFO) << "Registering tag in dictionary: (" << tag.Format() << ") "
+              << (DcmVR(evr).getValidVRName()) << " " 
               << name << " (multiplicity: " << minMultiplicity << "-" 
               << (arbitrary ? "n" : boost::lexical_cast<std::string>(maxMultiplicity)) << ")";
 
@@ -467,7 +468,15 @@ DCMTK_TO_CTYPE_CONVERTER(DcmtkToFloat64Converter, Float64, DcmFloatingPointDoubl
   }
 
 
-  void FromDcmtkBridge::ExtractDicomSummary(DicomMap& target, 
+  Encoding FromDcmtkBridge::DetectEncoding(DcmItem &dataset,
+                                           Encoding defaultEncoding)
+  {
+    bool hasCodeExtensions;  // ignored
+    return DetectEncoding(hasCodeExtensions, dataset, defaultEncoding);
+  }
+
+
+  void FromDcmtkBridge::ExtractDicomSummary(DicomMap& target,
                                             DcmItem& dataset,
                                             unsigned int maxStringLength,
                                             const std::set<DicomTag>& ignoreTagLength)
@@ -1069,6 +1078,11 @@ DCMTK_TO_CTYPE_CONVERTER(DcmtkToFloat64Converter, Float64, DcmFloatingPointDoubl
     return GetTagNameInternal(tag);
   }
 
+  std::string FromDcmtkBridge::GetTagName(const DicomElement &element)
+  {
+    return GetTagName(element.GetTag(), "");
+  }
+
 
 
   DicomTag FromDcmtkBridge::ParseTag(const char* name)
@@ -1109,6 +1123,30 @@ DCMTK_TO_CTYPE_CONVERTER(DcmtkToFloat64Converter, Float64, DcmFloatingPointDoubl
 #endif
   }
 
+  DicomTag FromDcmtkBridge::ParseTag(const std::string &name)
+  {
+    return ParseTag(name.c_str());
+  }
+
+  bool FromDcmtkBridge::HasTag(const DicomMap &fields, const std::string &tagName)
+  {
+    return fields.HasTag(ParseTag(tagName));
+  }
+
+  const DicomValue &FromDcmtkBridge::GetValue(const DicomMap &fields,
+                                              const std::string &tagName)
+  {
+    return fields.GetValue(ParseTag(tagName));
+  }
+
+  void FromDcmtkBridge::SetValue(DicomMap &target,
+                                 const std::string &tagName,
+                                 DicomValue *value)
+  {
+    const DicomTag tag = ParseTag(tagName);
+    target.SetValueInternal(tag.GetGroup(), tag.GetElement(), value);
+  }
+
 
   bool FromDcmtkBridge::IsUnknownTag(const DicomTag& tag)
   {
@@ -1444,7 +1482,7 @@ DCMTK_TO_CTYPE_CONVERTER(DcmtkToFloat64Converter, Float64, DcmFloatingPointDoubl
     {
       // This solves issue 140 (Modifying private tags with REST API
       // changes VR from LO to UN)
-      // https://bitbucket.org/sjodogne/orthanc/issues/140
+      // https://bugs.orthanc-server.com/show_bug.cgi?id=140
       LOG(WARNING) << "Private creator should not be empty while creating a private tag: " << tag.Format();
     }
     
@@ -1889,9 +1927,8 @@ DCMTK_TO_CTYPE_CONVERTER(DcmtkToFloat64Converter, Float64, DcmFloatingPointDoubl
       if (tag != DICOM_TAG_SPECIFIC_CHARACTER_SET)
       {
         std::unique_ptr<DcmElement> element(FromDcmtkBridge::FromJson(tag, value, decodeDataUriScheme, encoding, privateCreator));
-        const DcmTagKey& tag = element->getTag();
 
-        result->findAndDeleteElement(tag);
+        result->findAndDeleteElement(element->getTag());
 
         DcmElement* tmp = element.release();
         if (!result->insert(tmp, false, false).good())
@@ -2581,9 +2618,16 @@ DCMTK_TO_CTYPE_CONVERTER(DcmtkToFloat64Converter, Float64, DcmFloatingPointDoubl
     {
       throw OrthancException(ErrorCode_InternalError);
     }
-        
-    DcmDataset& dataset = *dicom.getDataset();
+    else
+    {
+      return LookupOrthancTransferSyntax(target, *dicom.getDataset());
+    }
+  }
 
+
+  bool FromDcmtkBridge::LookupOrthancTransferSyntax(DicomTransferSyntax& target,
+                                                    DcmDataset& dataset)
+  {
     E_TransferSyntax xfer = dataset.getCurrentXfer();
     if (xfer == EXS_Unknown)
     {
