@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -30,7 +30,7 @@
 #include <dcmtk/dcmdata/dcmetinf.h>
 #include <dcmtk/dcmdata/dcpixseq.h>
 #include <dcmtk/dcmdata/dcfilefo.h>
-#include <json/json.h>
+#include <json/value.h>
 
 #if ORTHANC_ENABLE_DCMTK != 1
 #  error The macro ORTHANC_ENABLE_DCMTK must be set to 1
@@ -99,13 +99,9 @@ namespace Orthanc
                                    DcmItem& dataset,
                                    Encoding defaultEncoding);
 
+    // Compatibility wrapper for Orthanc <= 1.5.4
     static Encoding DetectEncoding(DcmItem& dataset,
-                                   Encoding defaultEncoding)
-    {
-      // Compatibility wrapper for Orthanc <= 1.5.4
-      bool hasCodeExtensions;  // ignored
-      return DetectEncoding(hasCodeExtensions, dataset, defaultEncoding);
-    }
+                                   Encoding defaultEncoding);
 
     static DicomTag Convert(const DcmTag& tag);
 
@@ -131,37 +127,21 @@ namespace Orthanc
 
     static std::string GetTagName(const DcmElement& element);
 
-    static std::string GetTagName(const DicomElement& element)
-    {
-      return GetTagName(element.GetTag(), "");
-    }
+    static std::string GetTagName(const DicomElement& element);
 
     static DicomTag ParseTag(const char* name);
 
-    static DicomTag ParseTag(const std::string& name)
-    {
-      return ParseTag(name.c_str());
-    }
+    static DicomTag ParseTag(const std::string& name);
 
     static bool HasTag(const DicomMap& fields,
-                       const std::string& tagName)
-    {
-      return fields.HasTag(ParseTag(tagName));
-    }
+                       const std::string& tagName);
 
     static const DicomValue& GetValue(const DicomMap& fields,
-                                      const std::string& tagName)
-    {
-      return fields.GetValue(ParseTag(tagName));
-    }
+                                      const std::string& tagName);
 
     static void SetValue(DicomMap& target,
                          const std::string& tagName,
-                         DicomValue* value)
-    {
-      const DicomTag tag = ParseTag(tagName);
-      target.SetValueInternal(tag.GetGroup(), tag.GetElement(), value);
-    }
+                         DicomValue* value);
 
     static void ToJson(Json::Value& result,
                        const DicomMap& values,
@@ -239,5 +219,8 @@ namespace Orthanc
 
     static bool LookupOrthancTransferSyntax(DicomTransferSyntax& target,
                                             DcmFileFormat& dicom);
+
+    static bool LookupOrthancTransferSyntax(DicomTransferSyntax& target,
+                                            DcmDataset& dicom);
   };
 }
