@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -134,6 +134,7 @@ namespace Orthanc
     // Check whether the DICOM instance contains an image encoded with
     // the PMSCT_RLE1 scheme.
     if (!dataset.findAndGetElement(ToDcmtkBridge::Convert(DICOM_TAG_COMPRESSION_TYPE), e).good() ||
+        !dataset.tagExistsWithValue(ToDcmtkBridge::Convert(DICOM_TAG_CONTENT)) ||  // New in Orthanc 1.7.4
         e == NULL ||
         !e->isaString() ||
         !e->getString(c).good() ||
@@ -251,7 +252,7 @@ namespace Orthanc
 
       DicomMap m;
       std::set<DicomTag> ignoreTagLength;
-      FromDcmtkBridge::ExtractDicomSummary(m, dataset, DicomImageInformation::USEFUL_TAG_LENGTH, ignoreTagLength);
+      FromDcmtkBridge::ExtractDicomSummary(m, dataset, DicomImageInformation::GetUsefulTagLength(), ignoreTagLength);
 
       /**
        * Create an accessor to the raw values of the DICOM image.
@@ -324,7 +325,7 @@ namespace Orthanc
   {
     DicomMap m;
     std::set<DicomTag> ignoreTagLength;
-    FromDcmtkBridge::ExtractDicomSummary(m, dataset, DicomImageInformation::USEFUL_TAG_LENGTH, ignoreTagLength);
+    FromDcmtkBridge::ExtractDicomSummary(m, dataset, DicomImageInformation::GetUsefulTagLength(), ignoreTagLength);
 
     DicomImageInformation info(m);
     PixelFormat format;
@@ -353,12 +354,16 @@ namespace Orthanc
     const PixelType minValue = std::numeric_limits<PixelType>::min();
     const PixelType maxValue = std::numeric_limits<PixelType>::max();
 
-    for (unsigned int y = 0; y < source.GetInformation().GetHeight(); y++)
+    const unsigned int height = source.GetInformation().GetHeight();
+    const unsigned int width = source.GetInformation().GetWidth();
+    const unsigned int channels = source.GetInformation().GetChannelCount();
+    
+    for (unsigned int y = 0; y < height; y++)
     {
       PixelType* pixel = reinterpret_cast<PixelType*>(target.GetRow(y));
-      for (unsigned int x = 0; x < source.GetInformation().GetWidth(); x++)
+      for (unsigned int x = 0; x < width; x++)
       {
-        for (unsigned int c = 0; c < source.GetInformation().GetChannelCount(); c++, pixel++)
+        for (unsigned int c = 0; c < channels; c++, pixel++)
         {
           int32_t v = source.GetValue(x, y, c);
           if (v < static_cast<int32_t>(minValue))
@@ -436,12 +441,14 @@ namespace Orthanc
         }
 
         const uint8_t* source = reinterpret_cast<const uint8_t*>(pixelData);
+        const unsigned int width = target->GetWidth();
+        const unsigned int height = target->GetHeight();
         
-        for (unsigned int y = 0; y < target->GetHeight(); y++)
+        for (unsigned int y = 0; y < height; y++)
         {
           uint8_t* p = reinterpret_cast<uint8_t*>(target->GetRow(y));
 
-          for (unsigned int x = 0; x < target->GetWidth(); x++)
+          for (unsigned int x = 0; x < width; x++)
           {
             p[0] = lutRed[*source] >> 8;
             p[1] = lutGreen[*source] >> 8;
@@ -466,12 +473,14 @@ namespace Orthanc
         }
 
         const uint16_t* source = reinterpret_cast<const uint16_t*>(pixelData);
+        const unsigned int width = target->GetWidth();
+        const unsigned int height = target->GetHeight();
         
-        for (unsigned int y = 0; y < target->GetHeight(); y++)
+        for (unsigned int y = 0; y < height; y++)
         {
           uint16_t* p = reinterpret_cast<uint16_t*>(target->GetRow(y));
 
-          for (unsigned int x = 0; x < target->GetWidth(); x++)
+          for (unsigned int x = 0; x < width; x++)
           {
             p[0] = lutRed[*source];
             p[1] = lutGreen[*source];
@@ -605,7 +614,7 @@ namespace Orthanc
 
     DicomMap m;
     std::set<DicomTag> ignoreTagLength;
-    FromDcmtkBridge::ExtractDicomSummary(m, dataset, DicomImageInformation::USEFUL_TAG_LENGTH, ignoreTagLength);
+    FromDcmtkBridge::ExtractDicomSummary(m, dataset, DicomImageInformation::GetUsefulTagLength(), ignoreTagLength);
     DicomImageInformation info(m);
 
     std::unique_ptr<ImageAccessor> target(CreateImage(dataset, true));
@@ -651,20 +660,6 @@ namespace Orthanc
   }
 
 
-  ImageAccessor* DicomImageDecoder::Decode(ParsedDicomFile& dicom,
-                                           unsigned int frame)
-  {
-    if (dicom.GetDcmtkObject().getDataset() == NULL)
-    {
-      throw OrthancException(ErrorCode_InternalError);
-    }
-    else
-    {
-      return Decode(*dicom.GetDcmtkObject().getDataset(), frame);
-    }
-  }
-
-
   ImageAccessor* DicomImageDecoder::Decode(DcmDataset& dataset,
                                            unsigned int frame)
   {
@@ -976,7 +971,7 @@ namespace Orthanc
 
       if (invert)
       {
-        Orthanc::ImageProcessing::Invert(*image);
+        ImageProcessing::Invert(*image);
       }
     }
     else
@@ -994,7 +989,7 @@ namespace Orthanc
     ApplyExtractionMode(image, mode, invert);
 
     PamWriter writer;
-    writer.WriteToMemory(result, *image);
+    IImageWriter::WriteToMemory(writer, result, *image);
   }
 
 #if ORTHANC_ENABLE_PNG == 1
@@ -1006,7 +1001,7 @@ namespace Orthanc
     ApplyExtractionMode(image, mode, invert);
 
     PngWriter writer;
-    writer.WriteToMemory(result, *image);
+    IImageWriter::WriteToMemory(writer, result, *image);
   }
 #endif
 
@@ -1028,7 +1023,16 @@ namespace Orthanc
 
     JpegWriter writer;
     writer.SetQuality(quality);
-    writer.WriteToMemory(result, *image);
+    IImageWriter::WriteToMemory(writer, result, *image);
+  }
+#endif
+
+
+#if ORTHANC_BUILDING_FRAMEWORK_LIBRARY == 1
+  ImageAccessor *DicomImageDecoder::Decode(ParsedDicomFile& dataset,
+                                           unsigned int frame)
+  {
+    return Decode(*dataset.GetDcmtkObject().getDataset(), frame);
   }
 #endif
 }
