@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -72,6 +72,7 @@
 
 #include "FromDcmtkBridge.h"
 #include "Internals/DicomFrameIndex.h"
+#include "Internals/DicomImageDecoder.h"
 #include "ToDcmtkBridge.h"
 
 #include "../Images/PamReader.h"
@@ -442,10 +443,10 @@ namespace Orthanc
 
 #if ORTHANC_ENABLE_CIVETWEB == 1 || ORTHANC_ENABLE_MONGOOSE == 1
   void ParsedDicomFile::SendPathValue(RestApiOutput& output,
-                                      const UriComponents& uri)
+                                      const UriComponents& uri) const
   {
-    DcmItem* dicom = GetDcmtkObject().getDataset();
-    E_TransferSyntax transferSyntax = GetDcmtkObject().getDataset()->getCurrentXfer();
+    DcmItem* dicom = GetDcmtkObjectConst().getDataset();
+    E_TransferSyntax transferSyntax = GetDcmtkObjectConst().getDataset()->getCurrentXfer();
 
     // Special case: Accessing the pixel data
     if (uri.size() == 1 || 
@@ -669,6 +670,16 @@ namespace Orthanc
     }
   }
 
+  void ParsedDicomFile::RemovePrivateTags()
+  {
+    RemovePrivateTagsInternal(NULL);
+  }
+
+  void ParsedDicomFile::RemovePrivateTags(const std::set<DicomTag> &toKeep)
+  {
+    RemovePrivateTagsInternal(&toKeep);
+  }
+
 
   static bool CanReplaceProceed(DcmDataset& dicom,
                                 const DcmTagKey& tag,
@@ -853,10 +864,10 @@ namespace Orthanc
 
     
 #if ORTHANC_ENABLE_CIVETWEB == 1 || ORTHANC_ENABLE_MONGOOSE == 1
-  void ParsedDicomFile::Answer(RestApiOutput& output)
+  void ParsedDicomFile::Answer(RestApiOutput& output) const
   {
     std::string serialized;
-    if (FromDcmtkBridge::SaveToMemoryBuffer(serialized, *GetDcmtkObject().getDataset()))
+    if (FromDcmtkBridge::SaveToMemoryBuffer(serialized, *GetDcmtkObjectConst().getDataset()))
     {
       output.AnswerBuffer(serialized, MimeType_Dicom);
     }
@@ -865,10 +876,10 @@ namespace Orthanc
 
 
   bool ParsedDicomFile::GetTagValue(std::string& value,
-                                    const DicomTag& tag)
+                                    const DicomTag& tag) const
   {
     DcmTagKey k(tag.GetGroup(), tag.GetElement());
-    DcmDataset& dataset = *GetDcmtkObject().getDataset();
+    DcmDataset& dataset = *GetDcmtkObjectConst().getDataset();
 
     if (tag.IsPrivate() ||
         FromDcmtkBridge::IsUnknownTag(tag) ||
@@ -930,7 +941,7 @@ namespace Orthanc
   }
 
 
-  DicomInstanceHasher ParsedDicomFile::GetHasher()
+  DicomInstanceHasher ParsedDicomFile::GetHasher() const
   {
     std::string patientId, studyUid, seriesUid, instanceUid;
 
@@ -1103,11 +1114,11 @@ namespace Orthanc
   }
 
 
-  ParsedDicomFile::ParsedDicomFile(ParsedDicomFile& other,
+  ParsedDicomFile::ParsedDicomFile(const ParsedDicomFile& other,
                                    bool keepSopInstanceUid) : 
     pimpl_(new PImpl)
   {
-    pimpl_->file_.reset(dynamic_cast<DcmFileFormat*>(other.GetDcmtkObject().clone()));
+    pimpl_->file_.reset(dynamic_cast<DcmFileFormat*>(other.GetDcmtkObjectConst().clone()));
 
     if (!keepSopInstanceUid)
     {
@@ -1135,7 +1146,7 @@ namespace Orthanc
   }
 
 
-  DcmFileFormat& ParsedDicomFile::GetDcmtkObject() const
+  DcmFileFormat& ParsedDicomFile::GetDcmtkObjectConst() const
   {
     if (pimpl_->file_.get() == NULL)
     {
@@ -1148,6 +1159,16 @@ namespace Orthanc
     }
   }
 
+  ParsedDicomFile *ParsedDicomFile::AcquireDcmtkObject(DcmFileFormat *dicom)  // No clone here
+  {
+    return new ParsedDicomFile(dicom);
+  }
+
+  DcmFileFormat &ParsedDicomFile::GetDcmtkObject()
+  {
+    return GetDcmtkObjectConst();
+  }
+
 
   DcmFileFormat* ParsedDicomFile::ReleaseDcmtkObject()
   {
@@ -1164,7 +1185,7 @@ namespace Orthanc
   }
 
 
-  ParsedDicomFile* ParsedDicomFile::Clone(bool keepSopInstanceUid)
+  ParsedDicomFile* ParsedDicomFile::Clone(bool keepSopInstanceUid) const
   {
     return new ParsedDicomFile(*this, keepSopInstanceUid);
   }
@@ -1256,7 +1277,9 @@ namespace Orthanc
 
       case MimeType_Pam:
       {
-        PamReader reader;
+        // "true" means "enforce memory alignment": This is slower,
+        // but possibly avoids crash related to non-aligned memory access
+        PamReader reader(true);
         reader.ReadFromMemory(content);
         EmbedImage(reader);
         break;
@@ -1360,7 +1383,10 @@ namespace Orthanc
     Uint8* target = NULL;
     pixels->createUint8Array(accessor.GetHeight() * pitch, target);
 
-    for (unsigned int y = 0; y < accessor.GetHeight(); y++)
+    const unsigned int height = accessor.GetHeight();
+    const unsigned int width = accessor.GetWidth();
+
+    for (unsigned int y = 0; y < height; y++)
     {
       switch (accessor.GetFormat())
       {
@@ -1378,7 +1404,7 @@ namespace Orthanc
         {
           // The alpha channel is not supported by the DICOM standard
           const Uint8* source = reinterpret_cast<const Uint8*>(accessor.GetConstRow(y));
-          for (unsigned int x = 0; x < accessor.GetWidth(); x++, target += 3, source += 4)
+          for (unsigned int x = 0; x < width; x++, target += 3, source += 4)
           {
             target[0] = source[0];
             target[1] = source[1];
@@ -1403,7 +1429,7 @@ namespace Orthanc
   Encoding ParsedDicomFile::DetectEncoding(bool& hasCodeExtensions) const
   {
     return FromDcmtkBridge::DetectEncoding(hasCodeExtensions,
-                                           *GetDcmtkObject().getDataset(),
+                                           *GetDcmtkObjectConst().getDataset(),
                                            GetDefaultDicomEncoding());
   }
 
@@ -1424,10 +1450,10 @@ namespace Orthanc
   void ParsedDicomFile::DatasetToJson(Json::Value& target, 
                                       DicomToJsonFormat format,
                                       DicomToJsonFlags flags,
-                                      unsigned int maxStringLength)
+                                      unsigned int maxStringLength) const
   {
     std::set<DicomTag> ignoreTagLength;
-    FromDcmtkBridge::ExtractDicomAsJson(target, *GetDcmtkObject().getDataset(),
+    FromDcmtkBridge::ExtractDicomAsJson(target, *GetDcmtkObjectConst().getDataset(),
                                         format, flags, maxStringLength, ignoreTagLength);
   }
 
@@ -1436,24 +1462,24 @@ namespace Orthanc
                                       DicomToJsonFormat format,
                                       DicomToJsonFlags flags,
                                       unsigned int maxStringLength,
-                                      const std::set<DicomTag>& ignoreTagLength)
+                                      const std::set<DicomTag>& ignoreTagLength) const
   {
-    FromDcmtkBridge::ExtractDicomAsJson(target, *GetDcmtkObject().getDataset(),
+    FromDcmtkBridge::ExtractDicomAsJson(target, *GetDcmtkObjectConst().getDataset(),
                                         format, flags, maxStringLength, ignoreTagLength);
   }
 
 
   void ParsedDicomFile::HeaderToJson(Json::Value& target, 
-                                     DicomToJsonFormat format)
+                                     DicomToJsonFormat format) const
   {
-    FromDcmtkBridge::ExtractHeaderAsJson(target, *GetDcmtkObject().getMetaInfo(), format, DicomToJsonFlags_None, 0);
+    FromDcmtkBridge::ExtractHeaderAsJson(target, *GetDcmtkObjectConst().getMetaInfo(), format, DicomToJsonFlags_None, 0);
   }
 
 
   bool ParsedDicomFile::HasTag(const DicomTag& tag) const
   {
     DcmTag key(tag.GetGroup(), tag.GetElement());
-    return GetDcmtkObject().getDataset()->tagExists(key);
+    return GetDcmtkObjectConst().getDataset()->tagExists(key);
   }
 
 
@@ -1505,7 +1531,7 @@ namespace Orthanc
   }
 
 
-  bool ParsedDicomFile::ExtractPdf(std::string& pdf)
+  bool ParsedDicomFile::ExtractPdf(std::string& pdf) const
   {
     std::string sop, mime;
     
@@ -1582,18 +1608,18 @@ namespace Orthanc
 
   void ParsedDicomFile::GetRawFrame(std::string& target,
                                     MimeType& mime,
-                                    unsigned int frameId)
+                                    unsigned int frameId) const
   {
     if (pimpl_->frameIndex_.get() == NULL)
     {
       assert(pimpl_->file_ != NULL &&
-             GetDcmtkObject().getDataset() != NULL);
-      pimpl_->frameIndex_.reset(new DicomFrameIndex(*GetDcmtkObject().getDataset()));
+             GetDcmtkObjectConst().getDataset() != NULL);
+      pimpl_->frameIndex_.reset(new DicomFrameIndex(*GetDcmtkObjectConst().getDataset()));
     }
 
     pimpl_->frameIndex_->GetRawFrame(target, frameId);
 
-    E_TransferSyntax transferSyntax = GetDcmtkObject().getDataset()->getCurrentXfer();
+    E_TransferSyntax transferSyntax = GetDcmtkObjectConst().getDataset()->getCurrentXfer();
     switch (transferSyntax)
     {
       case EXS_JPEGProcess1:
@@ -1621,8 +1647,8 @@ namespace Orthanc
   unsigned int ParsedDicomFile::GetFramesCount() const
   {
     assert(pimpl_->file_ != NULL &&
-           GetDcmtkObject().getDataset() != NULL);
-    return DicomFrameIndex::GetFramesCount(*GetDcmtkObject().getDataset());
+           GetDcmtkObjectConst().getDataset() != NULL);
+    return DicomFrameIndex::GetFramesCount(*GetDcmtkObjectConst().getDataset());
   }
 
 
@@ -1643,7 +1669,7 @@ namespace Orthanc
                                             unsigned int maxTagLength) const
   {
     std::set<DicomTag> ignoreTagLength;
-    FromDcmtkBridge::ExtractDicomSummary(target, *GetDcmtkObject().getDataset(),
+    FromDcmtkBridge::ExtractDicomSummary(target, *GetDcmtkObjectConst().getDataset(),
                                          maxTagLength, ignoreTagLength);
   }
 
@@ -1652,12 +1678,12 @@ namespace Orthanc
                                             unsigned int maxTagLength,
                                             const std::set<DicomTag>& ignoreTagLength) const
   {
-    FromDcmtkBridge::ExtractDicomSummary(target, *GetDcmtkObject().getDataset(),
+    FromDcmtkBridge::ExtractDicomSummary(target, *GetDcmtkObjectConst().getDataset(),
                                          maxTagLength, ignoreTagLength);
   }
 
 
-  bool ParsedDicomFile::LookupTransferSyntax(std::string& result)
+  bool ParsedDicomFile::LookupTransferSyntax(std::string& result) const
   {
 #if 0
     // This was the implementation in Orthanc <= 1.6.1
@@ -1666,8 +1692,8 @@ namespace Orthanc
     // using the meta header?
     const char* value = NULL;
 
-    if (GetDcmtkObject().getMetaInfo() != NULL &&
-        GetDcmtkObject().getMetaInfo()->findAndGetString(DCM_TransferSyntaxUID, value).good() &&
+    if (GetDcmtkObjectConst().getMetaInfo() != NULL &&
+        GetDcmtkObjectConst().getMetaInfo()->findAndGetString(DCM_TransferSyntaxUID, value).good() &&
         value != NULL)
     {
       result.assign(value);
@@ -1679,7 +1705,7 @@ namespace Orthanc
     }
 #else
     DicomTransferSyntax s;
-    if (FromDcmtkBridge::LookupOrthancTransferSyntax(s, GetDcmtkObject()))
+    if (FromDcmtkBridge::LookupOrthancTransferSyntax(s, GetDcmtkObjectConst()))
     {
       result.assign(GetTransferSyntaxUid(s));
       return true;
@@ -1697,7 +1723,7 @@ namespace Orthanc
     DcmTagKey k(DICOM_TAG_PHOTOMETRIC_INTERPRETATION.GetGroup(),
                 DICOM_TAG_PHOTOMETRIC_INTERPRETATION.GetElement());
 
-    DcmDataset& dataset = *GetDcmtkObject().getDataset();
+    DcmDataset& dataset = *GetDcmtkObjectConst().getDataset();
 
     const char *c = NULL;
     if (dataset.findAndGetString(k, c).good() &&
@@ -1713,8 +1739,59 @@ namespace Orthanc
   }
 
 
+  void ParsedDicomFile::Apply(ITagVisitor& visitor) const
+  {
+    FromDcmtkBridge::Apply(*GetDcmtkObjectConst().getDataset(), visitor, GetDefaultDicomEncoding());
+  }
+
+
+  ImageAccessor* ParsedDicomFile::DecodeFrame(unsigned int frame) const
+  {
+    if (GetDcmtkObjectConst().getDataset() == NULL)
+    {
+      throw OrthancException(ErrorCode_InternalError);
+    }
+    else
+    {
+      return DicomImageDecoder::Decode(*GetDcmtkObjectConst().getDataset(), frame);
+    }
+  }
+
+
+#if ORTHANC_BUILDING_FRAMEWORK_LIBRARY == 1
+  // Alias for binary compatibility with Orthanc Framework 1.7.2 => don't use it anymore
+  void ParsedDicomFile::DatasetToJson(Json::Value& target,
+                                      DicomToJsonFormat format,
+                                      DicomToJsonFlags flags,
+                                      unsigned int maxStringLength)
+  {
+    return const_cast<const ParsedDicomFile&>(*this).DatasetToJson(target, format, flags, maxStringLength);
+  }
+
+  DcmFileFormat& ParsedDicomFile::GetDcmtkObject() const
+  {
+    return const_cast<ParsedDicomFile&>(*this).GetDcmtkObject();
+  }
+
   void ParsedDicomFile::Apply(ITagVisitor& visitor)
   {
-    FromDcmtkBridge::Apply(*GetDcmtkObject().getDataset(), visitor, GetDefaultDicomEncoding());
+    const_cast<const ParsedDicomFile&>(*this).Apply(visitor);
   }
+
+  ParsedDicomFile* ParsedDicomFile::Clone(bool keepSopInstanceUid)
+  {
+    return const_cast<const ParsedDicomFile&>(*this).Clone(keepSopInstanceUid);
+  }
+  
+  bool ParsedDicomFile::LookupTransferSyntax(std::string& result)
+  {
+    return const_cast<const ParsedDicomFile&>(*this).LookupTransferSyntax(result);
+  }
+
+  bool ParsedDicomFile::GetTagValue(std::string& value,
+                                    const DicomTag& tag)
+  {
+    return const_cast<const ParsedDicomFile&>(*this).GetTagValue(value, tag);
+  }
+#endif
 }
