@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -22,10 +22,10 @@
 
 #pragma once
 
+#include "../IMemoryBuffer.h"
 #include "../Enumerations.h"
 
 #include <string>
-#include <boost/noncopyable.hpp>
 
 namespace Orthanc
 {
@@ -41,9 +41,8 @@ namespace Orthanc
                         size_t size,
                         FileContentType type) = 0;
 
-    virtual void Read(std::string& content,
-                      const std::string& uuid,
-                      FileContentType type) = 0;
+    virtual IMemoryBuffer* Read(const std::string& uuid,
+                                FileContentType type) = 0;
 
     virtual void Remove(const std::string& uuid,
                         FileContentType type) = 0;
