@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -117,7 +117,7 @@ namespace Orthanc
     {
     }
 
-    CurlHeaders(const HttpClient::HttpHeaders& headers)
+    explicit CurlHeaders(const HttpClient::HttpHeaders& headers)
     {
       for (HttpClient::HttpHeaders::const_iterator
              it = headers.begin(); it != headers.end(); ++it)
@@ -318,7 +318,6 @@ namespace Orthanc
       try
       {
         assert(userdata != NULL);
-        CurlAnswer& that = *(static_cast<CurlAnswer*>(userdata));
 
         size_t length = size * nmemb;
         if (length == 0)
@@ -333,6 +332,7 @@ namespace Orthanc
           if (colon != std::string::npos &&
               eol != std::string::npos)
           {
+            CurlAnswer& that = *(static_cast<CurlAnswer*>(userdata));
             std::string tmp(s.substr(0, colon));
 
             if (that.headersLowerCase_)
@@ -345,6 +345,7 @@ namespace Orthanc
             if (!key.empty())
             {
               std::string value = Toolbox::StripSpaces(s.substr(colon + 1, eol));
+
               that.answer_.AddHeader(key, value);
             }
           }
@@ -369,7 +370,6 @@ namespace Orthanc
       try
       {
         assert(userdata != NULL);
-        CurlAnswer& that = *(static_cast<CurlAnswer*>(userdata));
 
         size_t length = size * nmemb;
         if (length == 0)
@@ -378,6 +378,7 @@ namespace Orthanc
         }
         else
         {
+          CurlAnswer& that = *(static_cast<CurlAnswer*>(userdata));
           that.answer_.AddChunk(buffer, length);
           return length;
         }
@@ -419,7 +420,7 @@ namespace Orthanc
     }
 
     virtual void AddHeader(const std::string& key,
-                           const std::string& value)
+                           const std::string& value) ORTHANC_OVERRIDE
     {
       if (headers_ != NULL)
       {
@@ -428,7 +429,7 @@ namespace Orthanc
     }
       
     virtual void AddChunk(const void* data,
-                          size_t size)
+                          size_t size) ORTHANC_OVERRIDE
     {
       answer_.AddChunk(data, size);
     }
@@ -676,6 +677,41 @@ namespace Orthanc
     curl_easy_cleanup(pimpl_->curl_);
   }
 
+  void HttpClient::SetUrl(const char *url)
+  {
+    url_ = std::string(url);
+  }
+
+  void HttpClient::SetUrl(const std::string &url)
+  {
+    url_ = url;
+  }
+
+  const std::string &HttpClient::GetUrl() const
+  {
+    return url_;
+  }
+
+  void HttpClient::SetMethod(HttpMethod method)
+  {
+    method_ = method;
+  }
+
+  HttpMethod HttpClient::GetMethod() const
+  {
+    return method_;
+  }
+
+  void HttpClient::SetTimeout(long seconds)
+  {
+    timeout_ = seconds;
+  }
+
+  long HttpClient::GetTimeout() const
+  {
+    return timeout_;
+  }
+
 
   void HttpClient::SetBody(const std::string& data)
   {
@@ -683,6 +719,16 @@ namespace Orthanc
     pimpl_->requestBody_.Clear();
   }
 
+  std::string &HttpClient::GetBody()
+  {
+    return body_;
+  }
+
+  const std::string &HttpClient::GetBody() const
+  {
+    return body_;
+  }
+
 
   void HttpClient::SetBody(IRequestBody& body)
   {
@@ -713,6 +759,11 @@ namespace Orthanc
     }
   }
 
+  bool HttpClient::IsVerbose() const
+  {
+    return isVerbose_;
+  }
+
 
   void HttpClient::AddHeader(const std::string& key,
                              const std::string& value)
@@ -788,11 +839,7 @@ namespace Orthanc
 #if ORTHANC_ENABLE_SSL == 1
       CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_SSLCERTTYPE, "PEM"));
       CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_SSLCERT, clientCertificateFile_.c_str()));
-
-      if (!clientCertificateKeyPassword_.empty())
-      {
-        CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_KEYPASSWD, clientCertificateKeyPassword_.c_str()));
-      }
+      CheckCode(curl_easy_setopt(pimpl_->curl_, CURLOPT_KEYPASSWD, clientCertificateKeyPassword_.c_str()));
 
       // NB: If no "clientKeyFile_" is provided, the key must be
       // prepended to the certificate file
@@ -1020,8 +1067,7 @@ namespace Orthanc
     std::string s;
     if (ApplyInternal(s, answerHeaders))
     {
-      Json::Reader reader;
-      return reader.parse(s, answerBody);
+      return Toolbox::ReadJson(answerBody, s);
     }
     else
     {
@@ -1036,6 +1082,31 @@ namespace Orthanc
     credentials_ = std::string(username) + ":" + std::string(password);
   }
 
+  void HttpClient::SetProxy(const std::string &proxy)
+  {
+    proxy_ = proxy;
+  }
+
+  void HttpClient::SetHttpsVerifyPeers(bool verify)
+  {
+    verifyPeers_ = verify;
+  }
+
+  bool HttpClient::IsHttpsVerifyPeers() const
+  {
+    return verifyPeers_;
+  }
+
+  void HttpClient::SetHttpsCACertificates(const std::string &certificates)
+  {
+    caCertificates_ = certificates;
+  }
+
+  const std::string &HttpClient::GetHttpsCACertificates() const
+  {
+    return caCertificates_;
+  }
+
 
   void HttpClient::ConfigureSsl(bool httpsVerifyPeers,
                                 const std::string& httpsVerifyCertificates)
@@ -1107,6 +1178,33 @@ namespace Orthanc
     return ApplyInternal(wrapper);
   }
 
+  bool HttpClient::Apply(std::string &answerBody)
+  {
+    return ApplyInternal(answerBody, NULL);
+  }
+
+  bool HttpClient::Apply(Json::Value &answerBody)
+  {
+    return ApplyInternal(answerBody, NULL);
+  }
+
+  bool HttpClient::Apply(std::string &answerBody,
+                         HttpClient::HttpHeaders &answerHeaders)
+  {
+    return ApplyInternal(answerBody, &answerHeaders);
+  }
+
+  bool HttpClient::Apply(Json::Value &answerBody,
+                         HttpClient::HttpHeaders &answerHeaders)
+  {
+    return ApplyInternal(answerBody, &answerHeaders);
+  }
+
+  HttpStatus HttpClient::GetLastStatus() const
+  {
+    return lastStatus_;
+  }
+
 
   void HttpClient::ApplyAndThrowException(IAnswer& answer)
   {
@@ -1184,6 +1282,51 @@ namespace Orthanc
     clientCertificateKeyPassword_ = certificateKeyPassword;
   }
 
+  void HttpClient::SetPkcs11Enabled(bool enabled)
+  {
+    pkcs11Enabled_ = enabled;
+  }
+
+  bool HttpClient::IsPkcs11Enabled() const
+  {
+    return pkcs11Enabled_;
+  }
+
+  const std::string &HttpClient::GetClientCertificateFile() const
+  {
+    return clientCertificateFile_;
+  }
+
+  const std::string &HttpClient::GetClientCertificateKeyFile() const
+  {
+    return clientCertificateKeyFile_;
+  }
+
+  const std::string &HttpClient::GetClientCertificateKeyPassword() const
+  {
+    return clientCertificateKeyPassword_;
+  }
+
+  void HttpClient::SetConvertHeadersToLowerCase(bool lowerCase)
+  {
+    headersToLowerCase_ = lowerCase;
+  }
+
+  bool HttpClient::IsConvertHeadersToLowerCase() const
+  {
+    return headersToLowerCase_;
+  }
+
+  void HttpClient::SetRedirectionFollowed(bool follow)
+  {
+    redirectionFollowed_ = follow;
+  }
+
+  bool HttpClient::IsRedirectionFollowed() const
+  {
+    return redirectionFollowed_;
+  }
+
 
   void HttpClient::InitializePkcs11(const std::string& module,
                                     const std::string& pin,
