@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -49,6 +49,29 @@
 
 namespace Orthanc
 {
+  ImageProcessing::ImagePoint::ImagePoint(int32_t x,
+                                          int32_t y) :
+    x_(x),
+    y_(y)
+  {
+  }
+
+  int32_t ImageProcessing::ImagePoint::GetX() const
+  {
+    return x_;
+  }
+
+  int32_t ImageProcessing::ImagePoint::GetY() const
+  {
+    return y_;
+  }
+
+  void ImageProcessing::ImagePoint::Set(int32_t x, int32_t y)
+  {
+    x_ = x;
+    y_ = y;
+  }
+
   void ImageProcessing::ImagePoint::ClipTo(int32_t minX, int32_t maxX, int32_t minY, int32_t maxY)
   {
     x_ = std::max(minX, std::min(maxX, x_));
@@ -418,7 +441,7 @@ namespace Orthanc
     for (unsigned int y = 0; y < height; y++)
     {
       TargetType* p = reinterpret_cast<TargetType*>(target.GetRow(y));
-      const SourceType* q = reinterpret_cast<const SourceType*>(source.GetRow(y));
+      const SourceType* q = reinterpret_cast<const SourceType*>(source.GetConstRow(y));
 
       for (unsigned int x = 0; x < width; x++, p++, q++)
       {
@@ -501,11 +524,11 @@ namespace Orthanc
       throw OrthancException(ErrorCode_IncompatibleImageFormat);
     }
 
-    unsigned int lineSize = source.GetBytesPerPixel() * source.GetWidth();
-
+    const unsigned int lineSize = source.GetBytesPerPixel() * source.GetWidth();
     assert(source.GetPitch() >= lineSize && target.GetPitch() >= lineSize);
 
-    for (unsigned int y = 0; y < source.GetHeight(); y++)
+    const unsigned int height = source.GetHeight();
+    for (unsigned int y = 0; y < height; y++)
     {
       memcpy(target.GetRow(y), source.GetConstRow(y), lineSize);
     }
@@ -562,42 +585,42 @@ namespace Orthanc
 
     switch (source.GetFormat())
     {
-      case Orthanc::PixelFormat_Float32:
+      case PixelFormat_Float32:
       {
         switch (target.GetFormat())
         {
-          case Orthanc::PixelFormat_Grayscale8:
+          case PixelFormat_Grayscale8:
             ApplyWindowingInternal<uint8_t, float>(target, source, windowCenter, windowWidth, rescaleSlope, rescaleIntercept, invert);
             break;
-          case Orthanc::PixelFormat_Grayscale16:
+          case PixelFormat_Grayscale16:
             ApplyWindowingInternal<uint16_t, float>(target, source, windowCenter, windowWidth, rescaleSlope, rescaleIntercept, invert);
             break;
           default:
             throw OrthancException(ErrorCode_NotImplemented);
         }
       };break;
-      case Orthanc::PixelFormat_Grayscale8:
+      case PixelFormat_Grayscale8:
       {
         switch (target.GetFormat())
         {
-          case Orthanc::PixelFormat_Grayscale8:
+          case PixelFormat_Grayscale8:
             ApplyWindowingInternal<uint8_t, uint8_t>(target, source, windowCenter, windowWidth, rescaleSlope, rescaleIntercept, invert);
             break;
-          case Orthanc::PixelFormat_Grayscale16:
+          case PixelFormat_Grayscale16:
             ApplyWindowingInternal<uint16_t, uint8_t>(target, source, windowCenter, windowWidth, rescaleSlope, rescaleIntercept, invert);
             break;
           default:
             throw OrthancException(ErrorCode_NotImplemented);
         }
       };break;
-      case Orthanc::PixelFormat_Grayscale16:
+      case PixelFormat_Grayscale16:
       {
         switch (target.GetFormat())
         {
-          case Orthanc::PixelFormat_Grayscale8:
+          case PixelFormat_Grayscale8:
             ApplyWindowingInternal<uint8_t, uint16_t>(target, source, windowCenter, windowWidth, rescaleSlope, rescaleIntercept, invert);
             break;
-          case Orthanc::PixelFormat_Grayscale16:
+          case PixelFormat_Grayscale16:
             ApplyWindowingInternal<uint16_t, uint16_t>(target, source, windowCenter, windowWidth, rescaleSlope, rescaleIntercept, invert);
             break;
           default:
@@ -1057,6 +1080,16 @@ namespace Orthanc
 
     switch (image.GetFormat())
     {
+      case PixelFormat_Grayscale8:
+      {
+        // New in Orthanc 1.9.0
+        uint8_t grayscale = (2126 * static_cast<uint16_t>(red) + 
+                             7152 * static_cast<uint16_t>(green) +
+                             0722 * static_cast<uint16_t>(blue)) / 10000;
+        Orthanc::ImageProcessing::Set(image, grayscale);
+        return;
+      }
+      
       case PixelFormat_RGBA32:
         p[0] = red;
         p[1] = green;
@@ -1555,13 +1588,13 @@ namespace Orthanc
 
   namespace
   {
-    template <Orthanc::PixelFormat Format>
+    template <PixelFormat Format>
     class BresenhamPixelWriter
     {
     private:
       typedef typename PixelTraits<Format>::PixelType  PixelType;
 
-      Orthanc::ImageAccessor&  image_;
+      ImageAccessor&  image_;
       PixelType                value_;
 
       void PlotLineLow(int x0,
@@ -1629,14 +1662,14 @@ namespace Orthanc
       }
 
     public:
-      BresenhamPixelWriter(Orthanc::ImageAccessor& image,
+      BresenhamPixelWriter(ImageAccessor& image,
                            int64_t value) :
         image_(image),
         value_(PixelTraits<Format>::IntegerToPixel(value))
       {
       }
 
-      BresenhamPixelWriter(Orthanc::ImageAccessor& image,
+      BresenhamPixelWriter(ImageAccessor& image,
                            const PixelType& value) :
         image_(image),
         value_(value)
@@ -1700,29 +1733,29 @@ namespace Orthanc
   {
     switch (image.GetFormat())
     {
-      case Orthanc::PixelFormat_Grayscale8:
+      case PixelFormat_Grayscale8:
       {
-        BresenhamPixelWriter<Orthanc::PixelFormat_Grayscale8> writer(image, value);
+        BresenhamPixelWriter<PixelFormat_Grayscale8> writer(image, value);
         writer.DrawSegment(x0, y0, x1, y1);
         break;
       }
 
-      case Orthanc::PixelFormat_Grayscale16:
+      case PixelFormat_Grayscale16:
       {
-        BresenhamPixelWriter<Orthanc::PixelFormat_Grayscale16> writer(image, value);
+        BresenhamPixelWriter<PixelFormat_Grayscale16> writer(image, value);
         writer.DrawSegment(x0, y0, x1, y1);
         break;
       }
 
-      case Orthanc::PixelFormat_SignedGrayscale16:
+      case PixelFormat_SignedGrayscale16:
       {
-        BresenhamPixelWriter<Orthanc::PixelFormat_SignedGrayscale16> writer(image, value);
+        BresenhamPixelWriter<PixelFormat_SignedGrayscale16> writer(image, value);
         writer.DrawSegment(x0, y0, x1, y1);
         break;
       }
 
       default:
-        throw Orthanc::OrthancException(Orthanc::ErrorCode_NotImplemented);
+        throw OrthancException(ErrorCode_NotImplemented);
     }
   }
 
@@ -1739,46 +1772,46 @@ namespace Orthanc
   {
     switch (image.GetFormat())
     {
-      case Orthanc::PixelFormat_BGRA32:
+      case PixelFormat_BGRA32:
       {
-        PixelTraits<Orthanc::PixelFormat_BGRA32>::PixelType pixel;
+        PixelTraits<PixelFormat_BGRA32>::PixelType pixel;
         pixel.red_ = red;
         pixel.green_ = green;
         pixel.blue_ = blue;
         pixel.alpha_ = alpha;
 
-        BresenhamPixelWriter<Orthanc::PixelFormat_BGRA32> writer(image, pixel);
+        BresenhamPixelWriter<PixelFormat_BGRA32> writer(image, pixel);
         writer.DrawSegment(x0, y0, x1, y1);
         break;
       }
 
-      case Orthanc::PixelFormat_RGBA32:
+      case PixelFormat_RGBA32:
       {
-        PixelTraits<Orthanc::PixelFormat_RGBA32>::PixelType pixel;
+        PixelTraits<PixelFormat_RGBA32>::PixelType pixel;
         pixel.red_ = red;
         pixel.green_ = green;
         pixel.blue_ = blue;
         pixel.alpha_ = alpha;
 
-        BresenhamPixelWriter<Orthanc::PixelFormat_RGBA32> writer(image, pixel);
+        BresenhamPixelWriter<PixelFormat_RGBA32> writer(image, pixel);
         writer.DrawSegment(x0, y0, x1, y1);
         break;
       }
 
-      case Orthanc::PixelFormat_RGB24:
+      case PixelFormat_RGB24:
       {
-        PixelTraits<Orthanc::PixelFormat_RGB24>::PixelType pixel;
+        PixelTraits<PixelFormat_RGB24>::PixelType pixel;
         pixel.red_ = red;
         pixel.green_ = green;
         pixel.blue_ = blue;
 
-        BresenhamPixelWriter<Orthanc::PixelFormat_RGB24> writer(image, pixel);
+        BresenhamPixelWriter<PixelFormat_RGB24> writer(image, pixel);
         writer.DrawSegment(x0, y0, x1, y1);
         break;
       }
 
       default:
-        throw Orthanc::OrthancException(Orthanc::ErrorCode_NotImplemented);
+        throw OrthancException(ErrorCode_NotImplemented);
     }
   }
 
@@ -1845,7 +1878,7 @@ namespace Orthanc
       if (points[i].GetX() < 0 || points[i].GetX() >= imageWidth
           || points[i].GetY() < 0 || points[i].GetY() >= imageHeight)
       {
-        throw Orthanc::OrthancException(ErrorCode_ParameterOutOfRange);
+        throw OrthancException(ErrorCode_ParameterOutOfRange);
       }
       cpx.push_back((double)points[i].GetX());
       cpy.push_back((double)points[i].GetY());
@@ -1860,15 +1893,15 @@ namespace Orthanc
 
     std::vector<int32_t> nodeX;
     nodeX.resize(cpSize);
-    int  nodes, pixelX, pixelY, i, j, swap ;
+    int  pixelX, pixelY, i, swap ;
 
     //  Loop through the rows of the image.
     for (pixelY = top; pixelY < bottom; pixelY++)
     {
       double y = (double)pixelY;
       //  Build a list of nodes.
-      nodes = 0;
-      j = static_cast<int>(cpSize) - 1;
+      int nodes = 0;
+      int j = static_cast<int>(cpSize) - 1;
 
       for (i = 0; i < static_cast<int>(cpSize); i++)
       {
@@ -1933,23 +1966,23 @@ namespace Orthanc
   {
     switch (image.GetFormat())
     {
-      case Orthanc::PixelFormat_Grayscale8:
+      case PixelFormat_Grayscale8:
       {
-        FillPolygon_<Orthanc::PixelFormat_Grayscale8>(image, points, value);
+        FillPolygon_<PixelFormat_Grayscale8>(image, points, value);
         break;
       }
-      case Orthanc::PixelFormat_Grayscale16:
+      case PixelFormat_Grayscale16:
       {
-        FillPolygon_<Orthanc::PixelFormat_Grayscale16>(image, points, value);
+        FillPolygon_<PixelFormat_Grayscale16>(image, points, value);
         break;
       }
-      case Orthanc::PixelFormat_SignedGrayscale16:
+      case PixelFormat_SignedGrayscale16:
       {
-        FillPolygon_<Orthanc::PixelFormat_SignedGrayscale16>(image, points, value);
+        FillPolygon_<PixelFormat_SignedGrayscale16>(image, points, value);
         break;
       }
       default:
-        throw Orthanc::OrthancException(Orthanc::ErrorCode_NotImplemented);
+        throw OrthancException(ErrorCode_NotImplemented);
     }
   }
 
@@ -2047,7 +2080,7 @@ namespace Orthanc
   void ImageProcessing::Resize(ImageAccessor& target,
                                const ImageAccessor& source)
   {
-    if (source.GetFormat() != source.GetFormat())
+    if (source.GetFormat() != target.GetFormat())
     {
       throw OrthancException(ErrorCode_IncompatibleImageFormat);
     }
@@ -2172,7 +2205,7 @@ namespace Orthanc
   // This is a slow implementation of horizontal convolution on one
   // individual channel, that checks for out-of-image values
   template <typename RawPixel, unsigned int ChannelsCount>
-  static float GetHorizontalConvolutionFloatSecure(const Orthanc::ImageAccessor& source,
+  static float GetHorizontalConvolutionFloatSecure(const ImageAccessor& source,
                                                    const std::vector<float>& horizontal,
                                                    size_t horizontalAnchor,
                                                    unsigned int x,
@@ -2547,4 +2580,60 @@ namespace Orthanc
 
     return target.release();
   }
+
+
+  void ImageProcessing::ConvertJpegYCbCrToRgb(ImageAccessor& image)
+  {
+    // http://dicom.nema.org/medical/dicom/current/output/chtml/part03/sect_C.7.6.3.html#sect_C.7.6.3.1.2
+    // https://en.wikipedia.org/wiki/YCbCr#JPEG_conversion
+    
+    // TODO - Check out the outcome of Mathieu's discussion about
+    // truncation of YCbCr-to-RGB conversion:
+    // https://groups.google.com/forum/#!msg/comp.protocols.dicom/JHuGeyWbTz8/ARoTWrJzAQAJ
+
+    const unsigned int width = image.GetWidth();
+    const unsigned int height = image.GetHeight();
+    const unsigned int pitch = image.GetPitch();
+    uint8_t* buffer = reinterpret_cast<uint8_t*>(image.GetBuffer());
+        
+    if (image.GetFormat() != PixelFormat_RGB24 ||
+        pitch < 3 * width)
+    {
+      throw OrthancException(ErrorCode_IncompatibleImageFormat);
+    }
+
+    for (unsigned int y = 0; y < height; y++)
+    {
+      uint8_t* p = buffer + y * pitch;
+          
+      for (unsigned int x = 0; x < width; x++, p += 3)
+      {
+        const float Y  = p[0];
+        const float Cb = p[1];
+        const float Cr = p[2];
+
+        const float result[3] = {
+          Y                             + 1.402f    * (Cr - 128.0f),
+          Y - 0.344136f * (Cb - 128.0f) - 0.714136f * (Cr - 128.0f),
+          Y + 1.772f    * (Cb - 128.0f)
+        };
+
+        for (uint8_t i = 0; i < 3 ; i++)
+        {
+          if (result[i] < 0)
+          {
+            p[i] = 0;
+          }
+          else if (result[i] > 255)
+          {
+            p[i] = 255;
+          }
+          else
+          {
+            p[i] = static_cast<uint8_t>(result[i]);
+          }
+        }    
+      }
+    }
+  }
 }
