@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -49,44 +49,23 @@ namespace Orthanc
             const JobStatus& status,
             const boost::posix_time::ptime& creationTime,
             const boost::posix_time::ptime& lastStateChangeTime,
-            const boost::posix_time::time_duration& runtime);
+            const boost::posix_time::time_duration& runtime) ORTHANC_LOCAL;
 
     JobInfo();
 
-    const std::string& GetIdentifier() const
-    {
-      return id_;
-    }
+    const std::string& GetIdentifier() const;
 
-    int GetPriority() const
-    {
-      return priority_;
-    }
+    int GetPriority() const;
 
-    JobState GetState() const
-    {
-      return state_;
-    }
+    JobState GetState() const;
 
-    const boost::posix_time::ptime& GetInfoTime() const
-    {
-      return timestamp_;
-    }
+    const boost::posix_time::ptime& GetInfoTime() const;
 
-    const boost::posix_time::ptime& GetCreationTime() const
-    {
-      return creationTime_;
-    }
+    const boost::posix_time::ptime& GetCreationTime() const;
 
-    const boost::posix_time::time_duration& GetRuntime() const
-    {
-      return runtime_;
-    }
+    const boost::posix_time::time_duration& GetRuntime() const;
 
-    bool HasEstimatedTimeOfArrival() const
-    {
-      return hasEta_;
-    }
+    bool HasEstimatedTimeOfArrival() const;
 
     bool HasCompletionTime() const;
 
@@ -94,15 +73,9 @@ namespace Orthanc
 
     const boost::posix_time::ptime& GetCompletionTime() const;
 
-    const JobStatus& GetStatus() const
-    {
-      return status_;
-    }
+    const JobStatus& GetStatus() const;
 
-    JobStatus& GetStatus()
-    {
-      return status_;
-    }
+    JobStatus& GetStatus();
 
     void Format(Json::Value& target) const;
   };
