@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -24,18 +24,16 @@
 
 #include "IJobOperation.h"
 
+#include "../../Compatibility.h"  // For ORTHANC_OVERRIDE
+
 namespace Orthanc
 {
   class ORTHANC_PUBLIC LogJobOperation : public IJobOperation
   {
   public:
     virtual void Apply(JobOperationValues& outputs,
-                       const JobOperationValue& input);
+                       const IJobOperationValue& input) ORTHANC_OVERRIDE;
 
-    virtual void Serialize(Json::Value& result) const
-    {
-      result = Json::objectValue;
-      result["Type"] = "Log";
-    }
+    virtual void Serialize(Json::Value& result) const ORTHANC_OVERRIDE;
   };
 }
