@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -31,8 +31,6 @@
 #include "../DicomFormat/DicomArray.h"
 #include "../DicomFormat/DicomMap.h"
 
-#include <json/json.h>
-
 namespace Orthanc
 {
   class ORTHANC_PUBLIC LuaFunctionCall : public boost::noncopyable
@@ -71,10 +69,7 @@ namespace Orthanc
 
     void PushDicom(const DicomArray& dicom);
 
-    void Execute()
-    {
-      ExecuteInternal(0);
-    }
+    void Execute();
 
     bool ExecutePredicate();
 
