@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -34,11 +34,13 @@
 #include "../Sources/Compatibility.h"
 #include "../Sources/OrthancException.h"
 #include "../Sources/DicomFormat/DicomMap.h"
+#include "../Sources/DicomFormat/DicomStreamReader.h"
 #include "../Sources/DicomParsing/FromDcmtkBridge.h"
 #include "../Sources/DicomParsing/ToDcmtkBridge.h"
 #include "../Sources/DicomParsing/ParsedDicomFile.h"
 #include "../Sources/DicomParsing/DicomWebJsonVisitor.h"
 
+#include <boost/lexical_cast.hpp>
 
 using namespace Orthanc;
 
@@ -650,3 +652,369 @@ TEST(DicomMap, MainTagNames)
     }
   }
 }
+
+
+
+TEST(DicomTag, Comparisons)
+{
+  DicomTag a(0x0000, 0x0000);
+  DicomTag b(0x0010, 0x0010);
+  DicomTag c(0x0010, 0x0020);
+  DicomTag d(0x0020, 0x0000);
+
+  // operator==()
+  ASSERT_TRUE(a == a);
+  ASSERT_FALSE(a == b);
+
+  // operator!=()
+  ASSERT_FALSE(a != a);
+  ASSERT_TRUE(a != b);
+
+  // operator<=()
+  ASSERT_TRUE(a <= a);
+  ASSERT_TRUE(a <= b);
+  ASSERT_TRUE(a <= c);
+  ASSERT_TRUE(a <= d);
+
+  ASSERT_FALSE(b <= a);
+  ASSERT_TRUE(b <= b);
+  ASSERT_TRUE(b <= c);
+  ASSERT_TRUE(b <= d);
+
+  ASSERT_FALSE(c <= a);
+  ASSERT_FALSE(c <= b);
+  ASSERT_TRUE(c <= c);
+  ASSERT_TRUE(c <= d);
+
+  ASSERT_FALSE(d <= a);
+  ASSERT_FALSE(d <= b);
+  ASSERT_FALSE(d <= c);
+  ASSERT_TRUE(d <= d);
+
+  // operator<()
+  ASSERT_FALSE(a < a);
+  ASSERT_TRUE(a < b);
+  ASSERT_TRUE(a < c);
+  ASSERT_TRUE(a < d);
+
+  ASSERT_FALSE(b < a);
+  ASSERT_FALSE(b < b);
+  ASSERT_TRUE(b < c);
+  ASSERT_TRUE(b < d);
+
+  ASSERT_FALSE(c < a);
+  ASSERT_FALSE(c < b);
+  ASSERT_FALSE(c < c);
+  ASSERT_TRUE(c < d);
+
+  ASSERT_FALSE(d < a);
+  ASSERT_FALSE(d < b);
+  ASSERT_FALSE(d < c);
+  ASSERT_FALSE(d < d);
+
+  // operator>=()
+  ASSERT_TRUE(a >= a);
+  ASSERT_FALSE(a >= b);
+  ASSERT_FALSE(a >= c);
+  ASSERT_FALSE(a >= d);
+
+  ASSERT_TRUE(b >= a);
+  ASSERT_TRUE(b >= b);
+  ASSERT_FALSE(b >= c);
+  ASSERT_FALSE(b >= d);
+
+  ASSERT_TRUE(c >= a);
+  ASSERT_TRUE(c >= b);
+  ASSERT_TRUE(c >= c);
+  ASSERT_FALSE(c >= d);
+
+  ASSERT_TRUE(d >= a);
+  ASSERT_TRUE(d >= b);
+  ASSERT_TRUE(d >= c);
+  ASSERT_TRUE(d >= d);
+
+  // operator>()
+  ASSERT_FALSE(a > a);
+  ASSERT_FALSE(a > b);
+  ASSERT_FALSE(a > c);
+  ASSERT_FALSE(a > d);
+
+  ASSERT_TRUE(b > a);
+  ASSERT_FALSE(b > b);
+  ASSERT_FALSE(b > c);
+  ASSERT_FALSE(b > d);
+
+  ASSERT_TRUE(c > a);
+  ASSERT_TRUE(c > b);
+  ASSERT_FALSE(c > c);
+  ASSERT_FALSE(c > d);
+
+  ASSERT_TRUE(d > a);
+  ASSERT_TRUE(d > b);
+  ASSERT_TRUE(d > c);
+  ASSERT_FALSE(d > d);
+}
+
+
+
+#if ORTHANC_SANDBOXED != 1
+
+#include "../Sources/SystemToolbox.h"
+
+TEST(DicomMap, DISABLED_ParseDicomMetaInformation)
+{
+  static const std::string PATH = "/home/jodogne/Subversion/orthanc-tests/Database/TransferSyntaxes/";
+  
+  std::map<std::string, DicomTransferSyntax> f;
+  f.insert(std::make_pair(PATH + "../ColorTestMalaterre.dcm", DicomTransferSyntax_LittleEndianImplicit));  // 1.2.840.10008.1.2
+  f.insert(std::make_pair(PATH + "1.2.840.10008.1.2.1.dcm", DicomTransferSyntax_LittleEndianExplicit));
+  f.insert(std::make_pair(PATH + "1.2.840.10008.1.2.2.dcm", DicomTransferSyntax_BigEndianExplicit));
+  f.insert(std::make_pair(PATH + "1.2.840.10008.1.2.4.50.dcm", DicomTransferSyntax_JPEGProcess1));
+  f.insert(std::make_pair(PATH + "1.2.840.10008.1.2.4.51.dcm", DicomTransferSyntax_JPEGProcess2_4));
+  f.insert(std::make_pair(PATH + "1.2.840.10008.1.2.4.57.dcm", DicomTransferSyntax_JPEGProcess14));
+  f.insert(std::make_pair(PATH + "1.2.840.10008.1.2.4.70.dcm", DicomTransferSyntax_JPEGProcess14SV1));
+  f.insert(std::make_pair(PATH + "1.2.840.10008.1.2.4.80.dcm", DicomTransferSyntax_JPEGLSLossless));
+  f.insert(std::make_pair(PATH + "1.2.840.10008.1.2.4.81.dcm", DicomTransferSyntax_JPEGLSLossy));
+  f.insert(std::make_pair(PATH + "1.2.840.10008.1.2.4.90.dcm", DicomTransferSyntax_JPEG2000LosslessOnly));
+  f.insert(std::make_pair(PATH + "1.2.840.10008.1.2.4.91.dcm", DicomTransferSyntax_JPEG2000));
+  f.insert(std::make_pair(PATH + "1.2.840.10008.1.2.5.dcm", DicomTransferSyntax_RLELossless));
+
+  for (std::map<std::string, DicomTransferSyntax>::const_iterator it = f.begin(); it != f.end(); ++it)
+  {
+    printf("\n== %s ==\n\n", it->first.c_str());
+    
+    std::string dicom;
+    SystemToolbox::ReadFile(dicom, it->first, false);
+
+    DicomMap d;
+    ASSERT_TRUE(DicomMap::ParseDicomMetaInformation(d, dicom.c_str(), dicom.size()));
+    d.Print(stdout);
+
+    std::string s;
+    ASSERT_TRUE(d.LookupStringValue(s, DICOM_TAG_TRANSFER_SYNTAX_UID, false));
+    
+    DicomTransferSyntax ts;
+    ASSERT_TRUE(LookupTransferSyntax(ts, s));
+    ASSERT_EQ(ts, it->second);
+  }
+}
+
+
+namespace
+{
+  class V : public DicomStreamReader::IVisitor
+  {
+  private:
+    DicomMap  map_;
+    
+  public:
+    const DicomMap& GetDicomMap() const
+    {
+      return map_;
+    }
+    
+    virtual void VisitMetaHeaderTag(const DicomTag& tag,
+                                    const ValueRepresentation& vr,
+                                    const std::string& value) ORTHANC_OVERRIDE
+    {
+      std::cout << "Header: " << tag.Format() << " [" << Toolbox::ConvertToAscii(value).c_str() << "] (" << value.size() << ")" << std::endl;
+    }
+
+    virtual void VisitTransferSyntax(DicomTransferSyntax transferSyntax) ORTHANC_OVERRIDE
+    {
+      printf("TRANSFER SYNTAX: %s\n", GetTransferSyntaxUid(transferSyntax));
+    }
+    
+    virtual bool VisitDatasetTag(const DicomTag& tag,
+                                 const ValueRepresentation& vr,
+                                 const std::string& value,
+                                 bool isLittleEndian) ORTHANC_OVERRIDE
+    {
+      if (!isLittleEndian)
+        printf("** ");
+      if (tag.GetGroup() < 0x7f00)
+      {
+        std::cout << "Dataset: " << tag.Format() << " " << EnumerationToString(vr)
+                  << " [" << Toolbox::ConvertToAscii(value).c_str() << "] (" << value.size() << ")" << std::endl;
+      }
+      else
+      {
+        std::cout << "Dataset: " << tag.Format() << " " << EnumerationToString(vr)
+                  << " [PIXEL] (" << value.size() << ")" << std::endl;
+      }
+
+      map_.SetValue(tag, value, Toolbox::IsAsciiString(value));
+                                                            
+      return true;
+    }
+  };
+}
+
+
+
+TEST(DicomStreamReader, DISABLED_Tutu)
+{
+  static const std::string PATH = "/home/jodogne/Subversion/orthanc-tests/Database/TransferSyntaxes/";
+  
+  std::string dicom;
+  SystemToolbox::ReadFile(dicom, PATH + "../ColorTestMalaterre.dcm", false);
+  //SystemToolbox::ReadFile(dicom, PATH + "1.2.840.10008.1.2.1.dcm", false);
+  //SystemToolbox::ReadFile(dicom, PATH + "1.2.840.10008.1.2.2.dcm", false);  // Big Endian
+  //SystemToolbox::ReadFile(dicom, PATH + "1.2.840.10008.1.2.4.50.dcm", false);
+  //SystemToolbox::ReadFile(dicom, PATH + "1.2.840.10008.1.2.4.51.dcm", false);
+  //SystemToolbox::ReadFile(dicom, PATH + "1.2.840.10008.1.2.4.57.dcm", false);
+  //SystemToolbox::ReadFile(dicom, PATH + "1.2.840.10008.1.2.4.70.dcm", false);
+  //SystemToolbox::ReadFile(dicom, PATH + "1.2.840.10008.1.2.4.80.dcm", false);
+  //SystemToolbox::ReadFile(dicom, PATH + "1.2.840.10008.1.2.4.81.dcm", false);
+  //SystemToolbox::ReadFile(dicom, PATH + "1.2.840.10008.1.2.4.90.dcm", false);
+  //SystemToolbox::ReadFile(dicom, PATH + "1.2.840.10008.1.2.4.91.dcm", false);
+  //SystemToolbox::ReadFile(dicom, PATH + "1.2.840.10008.1.2.5.dcm", false);
+  
+  std::stringstream stream;
+  size_t pos = 0;
+  
+  DicomStreamReader r(stream);
+  V visitor;
+
+  while (pos < dicom.size() &&
+         !r.IsDone())
+  {
+    //printf("."); 
+    //printf("%d\n", pos);
+    r.Consume(visitor);
+    stream.clear();
+    stream.put(dicom[pos++]);
+  }
+
+  r.Consume(visitor);
+
+  printf(">> %d\n", static_cast<int>(r.GetProcessedBytes()));
+}
+
+TEST(DicomStreamReader, DISABLED_Tutu2)
+{
+  //static const std::string PATH = "/home/jodogne/Subversion/orthanc-tests/Database/TransferSyntaxes/";
+
+  //const std::string path = PATH + "1.2.840.10008.1.2.4.50.dcm";
+  //const std::string path = PATH + "1.2.840.10008.1.2.2.dcm";
+  const std::string path = "/home/jodogne/Subversion/orthanc-tests/Database/HierarchicalAnonymization/RTH/RT.dcm";
+  
+  std::ifstream stream(path.c_str());
+  
+  DicomStreamReader r(stream);
+  V visitor;
+
+  r.Consume(visitor);
+
+  printf(">> %d\n", static_cast<int>(r.GetProcessedBytes()));
+}
+
+
+#include <boost/filesystem.hpp>
+
+TEST(DicomStreamReader, DISABLED_Tutu3)
+{
+  static const std::string PATH = "/home/jodogne/Subversion/orthanc-tests/Database/";
+
+  std::set<std::string> errors;
+  unsigned int success = 0;
+  
+  for (boost::filesystem::recursive_directory_iterator current(PATH), end;
+       current != end ; ++current)
+  {
+    if (SystemToolbox::IsRegularFile(current->path().string()))
+    {
+      try
+      {
+        if (current->path().extension() == ".dcm")
+        {
+          const std::string path = current->path().string();
+          printf("[%s]\n", path.c_str());
+
+          DicomMap m1;
+
+          {
+            std::ifstream stream(path.c_str());
+            
+            DicomStreamReader r(stream);
+            V visitor;
+            
+            try
+            {
+              r.Consume(visitor, DICOM_TAG_PIXEL_DATA);
+              //r.Consume(visitor);
+              success++;
+            }
+            catch (OrthancException& e)
+            {
+              errors.insert(path);
+              continue;
+            }
+
+            m1.Assign(visitor.GetDicomMap());
+          }
+
+          m1.SetValue(DICOM_TAG_PIXEL_DATA, "", true);
+
+
+          DicomMap m2;
+          
+          {
+            std::string dicom;
+            SystemToolbox::ReadFile(dicom, path);
+            
+            ParsedDicomFile f(dicom);
+            f.ExtractDicomSummary(m2, 256);
+          }
+
+          std::set<DicomTag> tags;
+          m2.GetTags(tags);
+
+          bool first = true;
+          for (std::set<DicomTag>::const_iterator it = tags.begin(); it != tags.end(); ++it)
+          {
+            if (!m1.HasTag(*it))
+            {
+              if (first)
+              {
+                fprintf(stderr, "[%s]\n", path.c_str());
+                first = false;
+              }
+              
+              std::cerr << "ERROR: " << it->Format() << std::endl;
+            }
+            else if (!m2.GetValue(*it).IsNull() &&
+                     !m2.GetValue(*it).IsBinary() &&
+                     Toolbox::IsAsciiString(m1.GetValue(*it).GetContent()))
+            {
+              const std::string& v1 = m1.GetValue(*it).GetContent();
+              const std::string& v2 = m2.GetValue(*it).GetContent();
+              
+              if (v1 != v2 &&
+                  (v1.size() != v2.size() + 1 ||
+                   v1.substr(0, v2.size()) != v2))
+              {
+                std::cerr << "ERROR: [" << v1 << "] [" << v2 << "]" << std::endl;
+              }
+            }
+          }
+        }
+      }
+      catch (boost::filesystem::filesystem_error&)
+      {
+      }
+    }
+  }
+
+
+  printf("\n== ERRORS ==\n");
+  for (std::set<std::string>::const_iterator
+         it = errors.begin(); it != errors.end(); ++it)
+  {
+    printf("[%s]\n", it->c_str());
+  }
+
+  printf("\n== SUCCESSES: %u ==\n\n", success);
+}
+
+#endif
