@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -40,23 +40,28 @@
 #include "../Sources/DicomParsing/DicomModification.h"
 #include "../Sources/DicomParsing/DicomWebJsonVisitor.h"
 #include "../Sources/DicomParsing/FromDcmtkBridge.h"
-#include "../Sources/DicomParsing/Internals/DicomImageDecoder.h"
 #include "../Sources/DicomParsing/ToDcmtkBridge.h"
+#include "../Sources/DicomParsing/ParsedDicomCache.h"
 #include "../Sources/Endianness.h"
 #include "../Sources/Images/Image.h"
 #include "../Sources/Images/ImageBuffer.h"
 #include "../Sources/Images/ImageProcessing.h"
 #include "../Sources/Images/PngReader.h"
 #include "../Sources/Images/PngWriter.h"
+#include "../Sources/Logging.h"
 #include "../Sources/OrthancException.h"
-#include "../Sources/SystemToolbox.h"
 #include "../Resources/CodeGeneration/EncodingTests.h"
 
+#if ORTHANC_SANDBOXED != 1
+#  include "../Sources/SystemToolbox.h"
+#endif
+
 #include <dcmtk/dcmdata/dcdeftag.h>
 #include <dcmtk/dcmdata/dcelem.h>
 #include <dcmtk/dcmdata/dcvrat.h>
 
 #include <boost/algorithm/string/predicate.hpp>
+#include <boost/lexical_cast.hpp>
 
 #if ORTHANC_ENABLE_PUGIXML == 1
 #  include <pugixml.hpp>
@@ -85,6 +90,7 @@ TEST(DicomFormat, Tag)
 }
 
 
+#if ORTHANC_SANDBOXED != 1
 TEST(DicomModification, Basic)
 {
   DicomModification m;
@@ -107,6 +113,7 @@ TEST(DicomModification, Basic)
     f->SaveToFile(b);
   }
 }
+#endif
 
 
 TEST(DicomModification, Anonymization)
@@ -181,18 +188,27 @@ TEST(DicomModification, Png)
 
   ParsedDicomFile o(true);
   o.EmbedContent(s);
+
+#if ORTHANC_SANDBOXED != 1
   o.SaveToFile("UnitTestsResults/png1.dcm");
+#endif
 
   // Red dot, without alpha channel
   s = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAIAAAACDbGyAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH3gUGDTcIn2+8BgAAACJJREFUCNdj/P//PwMjIwME/P/P+J8BBTAxEOL/R9Lx/z8AynoKAXOeiV8AAAAASUVORK5CYII=";
   o.EmbedContent(s);
+
+#if ORTHANC_SANDBOXED != 1
   o.SaveToFile("UnitTestsResults/png2.dcm");
+#endif
 
   // Check box in Graylevel8
   s = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAAAAAA6mKC9AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH3gUGDDcB53FulQAAAElJREFUGNNtj0sSAEEEQ1+U+185s1CtmRkblQ9CZldsKHJDk6DLGLJa6chjh0ooQmpjXMM86zPwydGEj6Ed/UGykkEM8X+p3u8/8LcOJIWLGeMAAAAASUVORK5CYII=";
   o.EmbedContent(s);
   //o.ReplacePlainString(DICOM_TAG_SOP_CLASS_UID, UID_DigitalXRayImageStorageForProcessing);
+
+#if ORTHANC_SANDBOXED != 1
   o.SaveToFile("UnitTestsResults/png3.dcm");
+#endif
 
 
   {
@@ -217,7 +233,10 @@ TEST(DicomModification, Png)
     }
 
     o.EmbedImage(accessor);
+
+#if ORTHANC_SANDBOXED != 1
     o.SaveToFile("UnitTestsResults/png4.dcm");
+#endif
   }
 }
 
@@ -742,8 +761,6 @@ TEST(ParsedDicomFile, FromJson)
 
 TEST(TestImages, PatternGrayscale8)
 {
-  static const char* PATH = "UnitTestsResults/PatternGrayscale8.dcm";
-
   Orthanc::Image image(Orthanc::PixelFormat_Grayscale8, 256, 256, false);
 
   for (int y = 0; y < 256; y++)
@@ -763,6 +780,8 @@ TEST(TestImages, PatternGrayscale8)
   image.GetRegion(r, 160, 32, 64, 192);
   Orthanc::ImageProcessing::Set(r, 255); 
 
+  std::string saved;
+  
   {
     ParsedDicomFile f(true);
     f.ReplacePlainString(DICOM_TAG_SOP_CLASS_UID, "1.2.840.10008.5.1.4.1.1.7");
@@ -773,15 +792,13 @@ TEST(TestImages, PatternGrayscale8)
     f.ReplacePlainString(DICOM_TAG_SERIES_DESCRIPTION, "Grayscale8");
     f.EmbedImage(image);
 
-    f.SaveToFile(PATH);
+    f.SaveToMemoryBuffer(saved);
   }
 
   {
-    std::string s;
-    Orthanc::SystemToolbox::ReadFile(s, PATH);
-    Orthanc::ParsedDicomFile f(s);
+    Orthanc::ParsedDicomFile f(saved);
     
-    std::unique_ptr<Orthanc::ImageAccessor> decoded(Orthanc::DicomImageDecoder::Decode(f, 0));
+    std::unique_ptr<Orthanc::ImageAccessor> decoded(f.DecodeFrame(0));
     ASSERT_EQ(256u, decoded->GetWidth());
     ASSERT_EQ(256u, decoded->GetHeight());
     ASSERT_EQ(Orthanc::PixelFormat_Grayscale8, decoded->GetFormat());
@@ -798,8 +815,6 @@ TEST(TestImages, PatternGrayscale8)
 
 TEST(TestImages, PatternRGB)
 {
-  static const char* PATH = "UnitTestsResults/PatternRGB24.dcm";
-
   Orthanc::Image image(Orthanc::PixelFormat_RGB24, 384, 256, false);
 
   for (int y = 0; y < 256; y++)
@@ -825,6 +840,8 @@ TEST(TestImages, PatternRGB)
     }
   }
 
+  std::string saved;
+
   {
     ParsedDicomFile f(true);
     f.ReplacePlainString(DICOM_TAG_SOP_CLASS_UID, "1.2.840.10008.5.1.4.1.1.7");
@@ -835,15 +852,13 @@ TEST(TestImages, PatternRGB)
     f.ReplacePlainString(DICOM_TAG_SERIES_DESCRIPTION, "RGB24");
     f.EmbedImage(image);
 
-    f.SaveToFile(PATH);
+    f.SaveToMemoryBuffer(saved);
   }
 
   {
-    std::string s;
-    Orthanc::SystemToolbox::ReadFile(s, PATH);
-    Orthanc::ParsedDicomFile f(s);
+    Orthanc::ParsedDicomFile f(saved);
     
-    std::unique_ptr<Orthanc::ImageAccessor> decoded(Orthanc::DicomImageDecoder::Decode(f, 0));
+    std::unique_ptr<Orthanc::ImageAccessor> decoded(f.DecodeFrame(0));
     ASSERT_EQ(384u, decoded->GetWidth());
     ASSERT_EQ(256u, decoded->GetHeight());
     ASSERT_EQ(Orthanc::PixelFormat_RGB24, decoded->GetFormat());
@@ -860,8 +875,6 @@ TEST(TestImages, PatternRGB)
 
 TEST(TestImages, PatternUint16)
 {
-  static const char* PATH = "UnitTestsResults/PatternGrayscale16.dcm";
-
   Orthanc::Image image(Orthanc::PixelFormat_Grayscale16, 256, 256, false);
 
   uint16_t v = 0;
@@ -882,6 +895,8 @@ TEST(TestImages, PatternUint16)
   image.GetRegion(r, 160, 32, 64, 192);
   Orthanc::ImageProcessing::Set(r, 65535); 
 
+  std::string saved;
+  
   {
     ParsedDicomFile f(true);
     f.ReplacePlainString(DICOM_TAG_SOP_CLASS_UID, "1.2.840.10008.5.1.4.1.1.7");
@@ -892,15 +907,13 @@ TEST(TestImages, PatternUint16)
     f.ReplacePlainString(DICOM_TAG_SERIES_DESCRIPTION, "Grayscale16");
     f.EmbedImage(image);
 
-    f.SaveToFile(PATH);
+    f.SaveToMemoryBuffer(saved);
   }
 
   {
-    std::string s;
-    Orthanc::SystemToolbox::ReadFile(s, PATH);
-    Orthanc::ParsedDicomFile f(s);
+    Orthanc::ParsedDicomFile f(saved);
     
-    std::unique_ptr<Orthanc::ImageAccessor> decoded(Orthanc::DicomImageDecoder::Decode(f, 0));
+    std::unique_ptr<Orthanc::ImageAccessor> decoded(f.DecodeFrame(0));
     ASSERT_EQ(256u, decoded->GetWidth());
     ASSERT_EQ(256u, decoded->GetHeight());
     ASSERT_EQ(Orthanc::PixelFormat_Grayscale16, decoded->GetFormat());
@@ -917,8 +930,6 @@ TEST(TestImages, PatternUint16)
 
 TEST(TestImages, PatternInt16)
 {
-  static const char* PATH = "UnitTestsResults/PatternSignedGrayscale16.dcm";
-
   Orthanc::Image image(Orthanc::PixelFormat_SignedGrayscale16, 256, 256, false);
 
   int16_t v = -32768;
@@ -938,6 +949,8 @@ TEST(TestImages, PatternInt16)
   image.GetRegion(r, 160, 32, 64, 192);
   Orthanc::ImageProcessing::Set(r, 32767); 
 
+  std::string saved;
+  
   {
     ParsedDicomFile f(true);
     f.ReplacePlainString(DICOM_TAG_SOP_CLASS_UID, "1.2.840.10008.5.1.4.1.1.7");
@@ -948,15 +961,13 @@ TEST(TestImages, PatternInt16)
     f.ReplacePlainString(DICOM_TAG_SERIES_DESCRIPTION, "SignedGrayscale16");
     f.EmbedImage(image);
 
-    f.SaveToFile(PATH);
+    f.SaveToMemoryBuffer(saved);
   }
 
   {
-    std::string s;
-    Orthanc::SystemToolbox::ReadFile(s, PATH);
-    Orthanc::ParsedDicomFile f(s);
+    Orthanc::ParsedDicomFile f(saved);
     
-    std::unique_ptr<Orthanc::ImageAccessor> decoded(Orthanc::DicomImageDecoder::Decode(f, 0));
+    std::unique_ptr<Orthanc::ImageAccessor> decoded(f.DecodeFrame(0));
     ASSERT_EQ(256u, decoded->GetWidth());
     ASSERT_EQ(256u, decoded->GetHeight());
     ASSERT_EQ(Orthanc::PixelFormat_SignedGrayscale16, decoded->GetFormat());
@@ -972,7 +983,7 @@ TEST(TestImages, PatternInt16)
 
 
 
-static void CheckEncoding(const ParsedDicomFile& dicom,
+static void CheckEncoding(ParsedDicomFile& dicom,
                           Encoding expected)
 {
   const char* value = NULL;
@@ -1052,8 +1063,9 @@ TEST(ParsedDicomFile, DicomMapEncodings2)
           // A specific source string is used in "EncodingTests.py" to
           // test against Chinese, it is normal that it does not correspond to UTF8
 
-          std::string encoded = Toolbox::ConvertToUtf8(Toolbox::ConvertFromUtf8(utf8, testEncodings[i]), testEncodings[i], false);
-          ASSERT_STREQ(testEncodingsExpected[i], encoded.c_str());
+          const std::string tmp = Toolbox::ConvertToUtf8(
+            Toolbox::ConvertFromUtf8(utf8, testEncodings[i]), testEncodings[i], false);
+          ASSERT_STREQ(testEncodingsExpected[i], tmp.c_str());
         }
       }
 
@@ -1969,8 +1981,7 @@ TEST(DicomWebJson, ValueRepresentation)
     DicomMap m;
     m.FromDicomWeb(visitor.GetResult());
     ASSERT_EQ(31u, m.GetSize());
-
-    std::string s;
+    
     ASSERT_TRUE(m.LookupStringValue(s, DicomTag(0x0002, 0x0002), false));  ASSERT_EQ("UI", s);
     ASSERT_TRUE(m.LookupStringValue(s, DicomTag(0x0040, 0x0241), false));  ASSERT_EQ("AE", s);
     ASSERT_TRUE(m.LookupStringValue(s, DicomTag(0x0010, 0x1010), false));  ASSERT_EQ("AS", s);
@@ -2019,8 +2030,7 @@ TEST(DicomWebJson, ValueRepresentation)
     ASSERT_TRUE(m.LookupStringValue(s, DicomTag(0x0008, 0x0119), true));  ASSERT_EQ("UC", s);
     ASSERT_TRUE(m.LookupStringValue(s, DicomTag(0x0008, 0x0120), true));  ASSERT_EQ("UR", s);
     ASSERT_TRUE(m.LookupStringValue(s, DicomTag(0x0008, 0x0301), true));  ASSERT_EQ("17", s);  // US (but tag unknown to DCMTK 3.6.0)
-#endif
-    
+#endif    
   }
 }
 
@@ -2075,6 +2085,81 @@ TEST(DicomWebJson, Sequence)
 }
 
 
+TEST(ParsedDicomCache, Basic)
+{
+  ParsedDicomCache cache(10);
+  ASSERT_EQ(0u, cache.GetCurrentSize());
+  ASSERT_EQ(0u, cache.GetNumberOfItems());
+
+  DicomMap tags;
+  tags.SetValue(DICOM_TAG_PATIENT_ID, "patient1", false);
+  cache.Acquire("a", new ParsedDicomFile(tags, Encoding_Latin1, true), 20);
+  ASSERT_EQ(20u, cache.GetCurrentSize());
+  ASSERT_EQ(1u, cache.GetNumberOfItems());
+
+  {
+    ParsedDicomCache::Accessor accessor(cache, "b");
+    ASSERT_FALSE(accessor.IsValid());
+    ASSERT_THROW(accessor.GetDicom(), OrthancException);
+    ASSERT_THROW(accessor.GetFileSize(), OrthancException);
+  }
+  
+  {
+    ParsedDicomCache::Accessor accessor(cache, "a");
+    ASSERT_TRUE(accessor.IsValid());
+    std::string s;
+    ASSERT_TRUE(accessor.GetDicom().GetTagValue(s, DICOM_TAG_PATIENT_ID));
+    ASSERT_EQ("patient1", s);
+    ASSERT_EQ(20u, accessor.GetFileSize());
+  }
+  
+  tags.SetValue(DICOM_TAG_PATIENT_ID, "patient2", false);
+  cache.Acquire("b", new ParsedDicomFile(tags, Encoding_Latin1, true), 5);  
+  ASSERT_EQ(5u, cache.GetCurrentSize());
+  ASSERT_EQ(1u, cache.GetNumberOfItems());
+
+  cache.Acquire("c", new ParsedDicomFile(true), 5);
+  ASSERT_EQ(10u, cache.GetCurrentSize());
+  ASSERT_EQ(2u, cache.GetNumberOfItems());
+
+  {
+    ParsedDicomCache::Accessor accessor(cache, "b");
+    ASSERT_TRUE(accessor.IsValid());
+    std::string s;
+    ASSERT_TRUE(accessor.GetDicom().GetTagValue(s, DICOM_TAG_PATIENT_ID));
+    ASSERT_EQ("patient2", s);
+    ASSERT_EQ(5u, accessor.GetFileSize());
+  }
+  
+  cache.Acquire("d", new ParsedDicomFile(true), 5);
+  ASSERT_EQ(10u, cache.GetCurrentSize());
+  ASSERT_EQ(2u, cache.GetNumberOfItems());
+
+  ASSERT_TRUE(ParsedDicomCache::Accessor(cache, "b").IsValid());
+  ASSERT_FALSE(ParsedDicomCache::Accessor(cache, "c").IsValid());  // recycled by LRU
+  ASSERT_TRUE(ParsedDicomCache::Accessor(cache, "d").IsValid());
+
+  cache.Invalidate("d");
+  ASSERT_EQ(5u, cache.GetCurrentSize());
+  ASSERT_EQ(1u, cache.GetNumberOfItems());
+  ASSERT_TRUE(ParsedDicomCache::Accessor(cache, "b").IsValid());
+  ASSERT_FALSE(ParsedDicomCache::Accessor(cache, "d").IsValid());
+
+  cache.Acquire("e", new ParsedDicomFile(true), 15);
+  ASSERT_EQ(15u, cache.GetCurrentSize());
+  ASSERT_EQ(1u, cache.GetNumberOfItems());
+
+  ASSERT_FALSE(ParsedDicomCache::Accessor(cache, "c").IsValid());
+  ASSERT_FALSE(ParsedDicomCache::Accessor(cache, "d").IsValid());
+  ASSERT_TRUE(ParsedDicomCache::Accessor(cache, "e").IsValid());
+
+  cache.Invalidate("e");
+  ASSERT_EQ(0u, cache.GetCurrentSize());
+  ASSERT_EQ(0u, cache.GetNumberOfItems());
+  ASSERT_FALSE(ParsedDicomCache::Accessor(cache, "e").IsValid());
+}
+
+
 
 
 #if ORTHANC_ENABLE_DCMTK_TRANSCODING == 1
@@ -2107,10 +2192,11 @@ TEST(Toto, DISABLED_Transcode3)
         std::string source;
         Orthanc::SystemToolbox::ReadFile(source, path);
 
-        std::string c, i;
+        std::string c, k;
         try
         {
-          scu.Transcode(c, i, transcoder, source.c_str(), source.size(), false, "", 0);
+          scu.Transcode(c, k, transcoder, source.c_str(), source.size(),
+                        DicomTransferSyntax_LittleEndianExplicit, false, "", 0);
         }
         catch (OrthancException& e)
         {
@@ -2120,7 +2206,7 @@ TEST(Toto, DISABLED_Transcode3)
           }
           else
           {
-            throw e;
+            throw;
           }
         }
       }
@@ -2130,10 +2216,14 @@ TEST(Toto, DISABLED_Transcode3)
 
 TEST(Toto, DISABLED_Transcode4)
 {
-  std::string source;
-  Orthanc::SystemToolbox::ReadFile(source, "/home/jodogne/Subversion/orthanc-tests/Database/KarstenHilbertRF.dcm");
+  std::unique_ptr<DcmFileFormat> toto;
 
-  std::unique_ptr<DcmFileFormat> toto(FromDcmtkBridge::LoadFromMemoryBuffer(source.c_str(), source.size()));
+  {
+    std::string source;
+    Orthanc::SystemToolbox::ReadFile(source, "/home/jodogne/Subversion/orthanc-tests/Database/KarstenHilbertRF.dcm");
+    toto.reset(FromDcmtkBridge::LoadFromMemoryBuffer(source.c_str(), source.size()));
+  }
+  
   const std::string sourceUid = IDicomTranscoder::GetSopInstanceUid(*toto);
   
   DicomTransferSyntax sourceSyntax;
