@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -28,6 +28,7 @@
 #include <gtest/gtest.h>
 
 #include "../../OrthancFramework/Sources/Compatibility.h"
+#include "../../OrthancFramework/Sources/DicomNetworking/DicomAssociationParameters.h"
 #include "../../OrthancFramework/Sources/DicomNetworking/RemoteModalityParameters.h"
 #include "../../OrthancFramework/Sources/DicomParsing/DicomModification.h"
 #include "../../OrthancFramework/Sources/DicomParsing/ParsedDicomFile.h"
@@ -168,7 +169,7 @@ namespace
     {
     }
     
-    DummyInstancesJob(const Json::Value& value) :
+    explicit DummyInstancesJob(const Json::Value& value) :
       SetOfInstancesJob(value)
     {
       if (HasTrailingStep())
@@ -831,12 +832,12 @@ static bool CheckIdempotentSetOfInstances(IJobUnserializer& unserializer,
 
 
 static bool CheckIdempotentSerialization(IJobUnserializer& unserializer,
-                                         JobOperationValue& value)
+                                         const IJobOperationValue& value)
 {
   Json::Value a = 42;
   value.Serialize(a);
   
-  std::unique_ptr<JobOperationValue> unserialized(unserializer.UnserializeValue(a));
+  std::unique_ptr<IJobOperationValue> unserialized(unserializer.UnserializeValue(a));
   
   Json::Value b = 43;
   unserialized->Serialize(b);
@@ -891,9 +892,9 @@ TEST(JobsSerialization, JobOperationValues)
     GenericJobUnserializer unserializer;
     std::unique_ptr<JobOperationValues> values(JobOperationValues::Unserialize(unserializer, s));
     ASSERT_EQ(3u, values->GetSize());
-    ASSERT_EQ(JobOperationValue::Type_Null, values->GetValue(0).GetType());
-    ASSERT_EQ(JobOperationValue::Type_String, values->GetValue(1).GetType());
-    ASSERT_EQ(JobOperationValue::Type_String, values->GetValue(2).GetType());
+    ASSERT_EQ(IJobOperationValue::Type_Null, values->GetValue(0).GetType());
+    ASSERT_EQ(IJobOperationValue::Type_String, values->GetValue(1).GetType());
+    ASSERT_EQ(IJobOperationValue::Type_String, values->GetValue(2).GetType());
 
     ASSERT_EQ("hello", dynamic_cast<const StringOperationValue&>(values->GetValue(1)).GetContent());
     ASSERT_EQ("world", dynamic_cast<const StringOperationValue&>(values->GetValue(2)).GetContent());
@@ -916,10 +917,10 @@ TEST(JobsSerialization, GenericValues)
   ASSERT_THROW(unserializer.UnserializeJob(s), OrthancException);
   ASSERT_THROW(unserializer.UnserializeOperation(s), OrthancException);
 
-  std::unique_ptr<JobOperationValue> value;
+  std::unique_ptr<IJobOperationValue> value;
   value.reset(unserializer.UnserializeValue(s));
   
-  ASSERT_EQ(JobOperationValue::Type_Null, value->GetType());
+  ASSERT_EQ(IJobOperationValue::Type_Null, value->GetType());
 
   {
     StringOperationValue str("Hello");
@@ -932,7 +933,7 @@ TEST(JobsSerialization, GenericValues)
   ASSERT_THROW(unserializer.UnserializeOperation(s), OrthancException);
   value.reset(unserializer.UnserializeValue(s));
 
-  ASSERT_EQ(JobOperationValue::Type_String, value->GetType());
+  ASSERT_EQ(IJobOperationValue::Type_String, value->GetType());
   ASSERT_EQ("Hello", dynamic_cast<StringOperationValue&>(*value).GetContent());
 }
 
@@ -1030,8 +1031,8 @@ TEST(JobsSerialization, GenericJobs)
 }
 
 
-static bool IsSameTagValue(ParsedDicomFile& dicom1,
-                           ParsedDicomFile& dicom2,
+static bool IsSameTagValue(const ParsedDicomFile& dicom1,
+                           const ParsedDicomFile& dicom2,
                            DicomTag tag)
 {
   std::string a, b;
@@ -1073,12 +1074,12 @@ TEST(JobsSerialization, DicomModification)
     std::unique_ptr<ParsedDicomFile> second(source.Clone(true));
     modification.Apply(*second);
 
-    std::string s;
-    ASSERT_TRUE(second->GetTagValue(s, DICOM_TAG_STUDY_DESCRIPTION));
-    ASSERT_TRUE(s.empty());
-    ASSERT_FALSE(second->GetTagValue(s, DICOM_TAG_SERIES_DESCRIPTION));
-    ASSERT_TRUE(second->GetTagValue(s, DICOM_TAG_PATIENT_NAME));
-    ASSERT_EQ("Test 4", s);
+    std::string t;
+    ASSERT_TRUE(second->GetTagValue(t, DICOM_TAG_STUDY_DESCRIPTION));
+    ASSERT_TRUE(t.empty());
+    ASSERT_FALSE(second->GetTagValue(t, DICOM_TAG_SERIES_DESCRIPTION));
+    ASSERT_TRUE(second->GetTagValue(t, DICOM_TAG_PATIENT_NAME));
+    ASSERT_EQ("Test 4", t);
 
     ASSERT_TRUE(IsSameTagValue(source, *modified, DICOM_TAG_STUDY_INSTANCE_UID));
     ASSERT_TRUE(IsSameTagValue(source, *second, DICOM_TAG_STUDY_INSTANCE_UID));
@@ -1269,10 +1270,12 @@ TEST(JobsSerialization, RemoteModalityParameters)
     ASSERT_FALSE(modality.IsAdvancedFormatNeeded());
     modality.Serialize(s, false);
     ASSERT_EQ(Json::arrayValue, s.type());
+    ASSERT_FALSE(modality.IsDicomTlsEnabled());
   }
 
   {
     RemoteModalityParameters modality(s);
+    ASSERT_FALSE(modality.IsAdvancedFormatNeeded());
     ASSERT_EQ("ORTHANC", modality.GetApplicationEntityTitle());
     ASSERT_EQ("127.0.0.1", modality.GetHost());
     ASSERT_EQ(104u, modality.GetPortNumber());
@@ -1285,21 +1288,25 @@ TEST(JobsSerialization, RemoteModalityParameters)
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_NAction));
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_NEventReport));
     ASSERT_TRUE(modality.IsTranscodingAllowed());
+    ASSERT_FALSE(modality.IsDicomTlsEnabled());
+    ASSERT_FALSE(modality.HasLocalAet());
+    ASSERT_THROW(modality.GetLocalAet(), OrthancException);
   }
 
   s = Json::nullValue;
 
   {
     RemoteModalityParameters modality;
-    ASSERT_FALSE(modality.IsAdvancedFormatNeeded());
     ASSERT_THROW(modality.SetPortNumber(0), OrthancException);
     ASSERT_THROW(modality.SetPortNumber(65535), OrthancException);
     modality.SetApplicationEntityTitle("HELLO");
     modality.SetHost("world");
     modality.SetPortNumber(45);
     modality.SetManufacturer(ModalityManufacturer_GenericNoWildcardInDates);
+    ASSERT_FALSE(modality.IsAdvancedFormatNeeded());
     modality.Serialize(s, true);
     ASSERT_EQ(Json::objectValue, s.type());
+    ASSERT_FALSE(modality.HasLocalAet());
   }
 
   {
@@ -1316,6 +1323,8 @@ TEST(JobsSerialization, RemoteModalityParameters)
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_NAction));
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_NEventReport));
     ASSERT_TRUE(modality.IsTranscodingAllowed());
+    ASSERT_FALSE(modality.IsDicomTlsEnabled());
+    ASSERT_FALSE(modality.HasLocalAet());
   }
 
   s["Port"] = "46";
@@ -1368,14 +1377,31 @@ TEST(JobsSerialization, RemoteModalityParameters)
     }
   }
 
+  s = Json::nullValue;
+
+  {
+    RemoteModalityParameters modality;
+    modality.SetLocalAet("hello");
+    ASSERT_TRUE(modality.IsAdvancedFormatNeeded());
+    modality.Serialize(s, true);
+    ASSERT_EQ(Json::objectValue, s.type());
+    ASSERT_TRUE(modality.HasLocalAet());
+  }
+
   {
-    Json::Value s;
-    s["AllowStorageCommitment"] = false;
-    s["AET"] = "AET";
-    s["Host"] = "host";
-    s["Port"] = "104";
-    
     RemoteModalityParameters modality(s);
+    ASSERT_TRUE(modality.HasLocalAet());
+    ASSERT_EQ("hello", modality.GetLocalAet());
+  }
+
+  {
+    Json::Value t;
+    t["AllowStorageCommitment"] = false;
+    t["AET"] = "AET";
+    t["Host"] = "host";
+    t["Port"] = "104";
+    
+    RemoteModalityParameters modality(t);
     ASSERT_TRUE(modality.IsAdvancedFormatNeeded());
     ASSERT_EQ("AET", modality.GetApplicationEntityTitle());
     ASSERT_EQ("host", modality.GetHost());
@@ -1383,18 +1409,23 @@ TEST(JobsSerialization, RemoteModalityParameters)
     ASSERT_FALSE(modality.IsRequestAllowed(DicomRequestType_NAction));
     ASSERT_FALSE(modality.IsRequestAllowed(DicomRequestType_NEventReport));
     ASSERT_TRUE(modality.IsTranscodingAllowed());
+    ASSERT_FALSE(modality.IsDicomTlsEnabled());
+    ASSERT_FALSE(modality.HasLocalAet());
+    ASSERT_THROW(modality.GetLocalAet(), OrthancException);
   }
 
   {
-    Json::Value s;
-    s["AllowNAction"] = false;
-    s["AllowNEventReport"] = true;
-    s["AET"] = "AET";
-    s["Host"] = "host";
-    s["Port"] = "104";
-    s["AllowTranscoding"] = false;
+    Json::Value t;
+    t["AllowNAction"] = false;
+    t["AllowNEventReport"] = true;
+    t["AET"] = "AET";
+    t["Host"] = "host";
+    t["Port"] = "104";
+    t["AllowTranscoding"] = false;
+    t["UseDicomTls"] = true;
+    t["LocalAet"] = "world";
     
-    RemoteModalityParameters modality(s);
+    RemoteModalityParameters modality(t);
     ASSERT_TRUE(modality.IsAdvancedFormatNeeded());
     ASSERT_EQ("AET", modality.GetApplicationEntityTitle());
     ASSERT_EQ("host", modality.GetHost());
@@ -1402,17 +1433,20 @@ TEST(JobsSerialization, RemoteModalityParameters)
     ASSERT_FALSE(modality.IsRequestAllowed(DicomRequestType_NAction));
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_NEventReport));
     ASSERT_FALSE(modality.IsTranscodingAllowed());
+    ASSERT_TRUE(modality.IsDicomTlsEnabled());
+    ASSERT_TRUE(modality.HasLocalAet());
+    ASSERT_EQ("world", modality.GetLocalAet());
   }
 
   {
-    Json::Value s;
-    s["AllowNAction"] = true;
-    s["AllowNEventReport"] = true;
-    s["AET"] = "AET";
-    s["Host"] = "host";
-    s["Port"] = "104";
+    Json::Value t;
+    t["AllowNAction"] = true;
+    t["AllowNEventReport"] = true;
+    t["AET"] = "AET";
+    t["Host"] = "host";
+    t["Port"] = "104";
     
-    RemoteModalityParameters modality(s);
+    RemoteModalityParameters modality(t);
     ASSERT_FALSE(modality.IsAdvancedFormatNeeded());
     ASSERT_EQ("AET", modality.GetApplicationEntityTitle());
     ASSERT_EQ("host", modality.GetHost());
@@ -1420,5 +1454,72 @@ TEST(JobsSerialization, RemoteModalityParameters)
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_NAction));
     ASSERT_TRUE(modality.IsRequestAllowed(DicomRequestType_NEventReport));
     ASSERT_TRUE(modality.IsTranscodingAllowed());
+    ASSERT_FALSE(modality.IsDicomTlsEnabled());
+    ASSERT_FALSE(modality.HasLocalAet());
+    ASSERT_THROW(modality.GetLocalAet(), OrthancException);
   }
 }
+
+
+
+TEST(JobsSerialization, DicomAssociationParameters)
+{
+  {
+    DicomAssociationParameters a;
+
+    Json::Value v = Json::objectValue;
+    a.SerializeJob(v);
+    ASSERT_EQ(Json::objectValue, v.type());
+    ASSERT_EQ("ORTHANC", v["LocalAet"].asString());
+    ASSERT_EQ(DicomAssociationParameters::GetDefaultTimeout(), v["Timeout"].asUInt());
+    ASSERT_TRUE(v.isMember("Remote"));
+    ASSERT_TRUE(v.isMember("MaximumPduLength"));
+
+    ASSERT_EQ(4u, v.getMemberNames().size());
+  
+    DicomAssociationParameters b;
+    b.UnserializeJob(v);
+    ASSERT_EQ("ANY-SCP", b.GetRemoteModality().GetApplicationEntityTitle());
+    ASSERT_EQ("127.0.0.1", b.GetRemoteModality().GetHost());
+    ASSERT_EQ(104u, b.GetRemoteModality().GetPortNumber());
+    ASSERT_EQ("ORTHANC", b.GetLocalApplicationEntityTitle());
+    ASSERT_EQ(DicomAssociationParameters::GetDefaultMaximumPduLength(), b.GetMaximumPduLength());
+    ASSERT_FALSE(b.GetRemoteModality().IsDicomTlsEnabled());
+    ASSERT_FALSE(b.GetRemoteModality().HasLocalAet());
+    ASSERT_THROW(b.GetRemoteModality().GetLocalAet(), OrthancException);
+  }
+
+  {
+    RemoteModalityParameters p;
+    p.SetApplicationEntityTitle("WORLD");
+    p.SetPortNumber(4242);
+    p.SetHost("hello.world.com");
+    p.SetDicomTlsEnabled(true);
+    
+    DicomAssociationParameters a("HELLO", p);
+    a.SetOwnCertificatePath("key", "crt");
+    a.SetTrustedCertificatesPath("trusted");
+
+    ASSERT_THROW(a.SetMaximumPduLength(4095), OrthancException);
+    ASSERT_THROW(a.SetMaximumPduLength(131073), OrthancException);
+    a.SetMaximumPduLength(4096);
+    a.SetMaximumPduLength(131072);
+
+    Json::Value v = Json::objectValue;
+    a.SerializeJob(v);
+
+    ASSERT_EQ(7u, v.getMemberNames().size());
+  
+    DicomAssociationParameters b = DicomAssociationParameters::UnserializeJob(v);
+
+    ASSERT_EQ("WORLD", b.GetRemoteModality().GetApplicationEntityTitle());
+    ASSERT_EQ("hello.world.com", b.GetRemoteModality().GetHost());
+    ASSERT_EQ(4242u, b.GetRemoteModality().GetPortNumber());
+    ASSERT_EQ("HELLO", b.GetLocalApplicationEntityTitle());
+    ASSERT_TRUE(b.GetRemoteModality().IsDicomTlsEnabled());
+    ASSERT_EQ("key", b.GetOwnPrivateKeyPath());
+    ASSERT_EQ("crt", b.GetOwnCertificatePath());
+    ASSERT_EQ("trusted", b.GetTrustedCertificatesPath());
+    ASSERT_EQ(131072u, b.GetMaximumPduLength());
+  }  
+}
