@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -28,24 +28,26 @@
 #include <gtest/gtest.h>
 
 #include "../Sources/ChunkedBuffer.h"
-#include "../Sources/HttpClient.h"
-#include "../Sources/Logging.h"
-#include "../Sources/SystemToolbox.h"
-#include "../Sources/RestApi/RestApi.h"
-#include "../Sources/OrthancException.h"
 #include "../Sources/Compression/ZlibCompressor.h"
-#include "../Sources/RestApi/RestApiHierarchy.h"
 #include "../Sources/HttpServer/HttpContentNegociation.h"
 #include "../Sources/HttpServer/MultipartStreamReader.h"
+#include "../Sources/Logging.h"
+#include "../Sources/OrthancException.h"
+#include "../Sources/RestApi/RestApiHierarchy.h"
+#include "../Sources/WebServiceParameters.h"
 
 #include <ctype.h>
 #include <boost/lexical_cast.hpp>
 #include <algorithm>
 
+#if ORTHANC_SANDBOXED != 1
+#  include "../Sources/RestApi/RestApi.h"
+#endif
+
 
 using namespace Orthanc;
 
-#if !defined(UNIT_TESTS_WITH_HTTP_CONNEXIONS)
+#if !defined(UNIT_TESTS_WITH_HTTP_CONNEXIONS) && (ORTHANC_SANDBOXED != 1)
 #  error UNIT_TESTS_WITH_HTTP_CONNEXIONS is not defined
 #endif
 
@@ -53,8 +55,13 @@ using namespace Orthanc;
 #  error ORTHANC_ENABLE_SSL is not defined
 #endif
 
+#if ORTHANC_SANDBOXED != 1
+#  include "../Sources/HttpClient.h"
+#  include "../Sources/SystemToolbox.h"
+#endif
 
 
+#if ORTHANC_SANDBOXED != 1
 TEST(HttpClient, Basic)
 {
   HttpClient c;
@@ -78,9 +85,10 @@ TEST(HttpClient, Basic)
   ASSERT_TRUE(v.isMember("Description"));
 #endif
 }
+#endif
 
 
-#if UNIT_TESTS_WITH_HTTP_CONNEXIONS == 1 && ORTHANC_ENABLE_SSL == 1
+#if (UNIT_TESTS_WITH_HTTP_CONNEXIONS == 1) && (ORTHANC_ENABLE_SSL == 1) && (ORTHANC_SANDBOXED != 1)
 
 /**
    The HTTPS CA certificates for BitBucket were extracted as follows:
@@ -176,8 +184,8 @@ TEST(ChunkedBuffer, Basic)
 
 TEST(RestApi, ParseCookies)
 {
-  IHttpHandler::Arguments headers;
-  IHttpHandler::Arguments cookies;
+  HttpToolbox::Arguments headers;
+  HttpToolbox::Arguments cookies;
 
   headers["cookie"] = "a=b;c=d;;;e=f;;g=h;";
   HttpToolbox::ParseCookies(cookies, headers);
@@ -203,9 +211,10 @@ TEST(RestApi, ParseCookies)
   ASSERT_EQ("v", cookies["n"]);
 }
 
+
 TEST(RestApi, RestApiPath)
 {
-  IHttpHandler::Arguments args;
+  HttpToolbox::Arguments args;
   UriComponents trail;
 
   {
@@ -303,7 +312,8 @@ namespace
   public:
     virtual bool Visit(const RestApiHierarchy::Resource& resource,
                        const UriComponents& uri,
-                       const IHttpHandler::Arguments& components,
+                       bool hasTrailing,
+                       const HttpToolbox::Arguments& components,
                        const UriComponents& trailing) ORTHANC_OVERRIDE
     {
       return resource.Handle(*(RestApiGetCall*) NULL);
@@ -332,7 +342,9 @@ TEST(RestApi, RestApiHierarchy)
 
   Json::Value m;
   root.CreateSiteMap(m);
-  std::cout << m;
+
+  std::string s;
+  Toolbox::WriteStyledJson(s, m);
 
   Json::Value d;
   ASSERT_FALSE(GetDirectory(d, root, "/hello"));
@@ -533,12 +545,14 @@ TEST(WebServiceParameters, Serialization)
     ASSERT_EQ(v, v2);
 
     p.Serialize(v2, false, false /* no password */);
-    WebServiceParameters p2(v2);
     ASSERT_EQ(Json::arrayValue, v2.type());
     ASSERT_EQ(3u, v2.size());
     ASSERT_EQ("http://localhost:8042/", v2[0u].asString());
     ASSERT_EQ("user", v2[1u].asString());
     ASSERT_TRUE(v2[2u].asString().empty());
+
+    WebServiceParameters p2(v2);  // Test decoding
+    ASSERT_EQ("http://localhost:8042/", p2.GetUrl());
   }
 
   {
@@ -552,7 +566,6 @@ TEST(WebServiceParameters, Serialization)
 
     Json::Value v2;
     p.Serialize(v2, false, true);
-    WebServiceParameters p2(v2);
 
     ASSERT_EQ(Json::objectValue, v2.type());
     ASSERT_EQ(3u, v2.size());
@@ -560,6 +573,9 @@ TEST(WebServiceParameters, Serialization)
     ASSERT_TRUE(v2["Pkcs11"].asBool());
     ASSERT_EQ(Json::objectValue, v2["HttpHeaders"].type());
     ASSERT_EQ(0u, v2["HttpHeaders"].size());
+
+    WebServiceParameters p2(v2);  // Test decoding
+    ASSERT_EQ("http://localhost:8042/", p2.GetUrl());
   }
 
   {
@@ -573,7 +589,6 @@ TEST(WebServiceParameters, Serialization)
 
     Json::Value v2;
     p.Serialize(v2, false, true);
-    WebServiceParameters p2(v2);
 
     ASSERT_EQ(Json::objectValue, v2.type());
     ASSERT_EQ(6u, v2.size());
@@ -584,6 +599,9 @@ TEST(WebServiceParameters, Serialization)
     ASSERT_FALSE(v2["Pkcs11"].asBool());
     ASSERT_EQ(Json::objectValue, v2["HttpHeaders"].type());
     ASSERT_EQ(0u, v2["HttpHeaders"].size());
+
+    WebServiceParameters p2(v2);  // Test decoding
+    ASSERT_EQ("http://localhost:8042/", p2.GetUrl());
   }
 
   {
@@ -818,6 +836,63 @@ TEST(MultipartStreamReader, ParseHeaders)
 }
 
 
+TEST(MultipartStreamReader, ParseHeaders2)
+{
+  std::string main;
+  std::map<std::string, std::string> args;
+  
+  ASSERT_FALSE(MultipartStreamReader::ParseHeaderArguments(main, args, ""));
+  ASSERT_FALSE(MultipartStreamReader::ParseHeaderArguments(main, args, "     "));
+  ASSERT_FALSE(MultipartStreamReader::ParseHeaderArguments(main, args, "  ;   "));
+
+  ASSERT_TRUE(MultipartStreamReader::ParseHeaderArguments(main, args, "hello"));
+  ASSERT_EQ("hello", main);
+  ASSERT_TRUE(args.empty());
+
+  ASSERT_TRUE(MultipartStreamReader::ParseHeaderArguments(main, args, "hello  ;  a  = \"  b  \";c=d  ;  e=f;"));
+  ASSERT_EQ("hello", main);
+  ASSERT_EQ(3u, args.size());
+  ASSERT_EQ("  b  ", args["a"]);
+  ASSERT_EQ("d", args["c"]);
+  ASSERT_EQ("f", args["e"]);
+
+  ASSERT_TRUE(MultipartStreamReader::ParseHeaderArguments(main, args, "    hello  ;;;;  ;  "));
+  ASSERT_EQ("hello", main);
+  ASSERT_TRUE(args.empty());
+
+  ASSERT_FALSE(MultipartStreamReader::ParseHeaderArguments(main, args, "hello;a=b;c=d;a=f"));
+
+  ASSERT_TRUE(MultipartStreamReader::ParseHeaderArguments(main, args, "multipart/related; dummy=value; boundary=1234; hello=world"));
+  ASSERT_EQ("multipart/related", main);
+  ASSERT_EQ(3u, args.size());
+  ASSERT_EQ("value", args["dummy"]);
+  ASSERT_EQ("1234", args["boundary"]);
+  ASSERT_EQ("world", args["hello"]);
+
+  ASSERT_TRUE(MultipartStreamReader::ParseHeaderArguments(main, args, "multipart/related; boundary="));
+  ASSERT_EQ("multipart/related", main);
+  ASSERT_EQ(1u, args.size());
+  ASSERT_EQ("", args["boundary"]);
+
+  ASSERT_TRUE(MultipartStreamReader::ParseHeaderArguments(main, args, "multipart/related; boundary"));
+  ASSERT_EQ("multipart/related", main);
+  ASSERT_EQ(1u, args.size());
+  ASSERT_EQ("", args["boundary"]);
+
+  ASSERT_TRUE(MultipartStreamReader::ParseHeaderArguments(main, args, "Multipart/Related; TYPE=Application/Dicom; Boundary=heLLO"));
+  ASSERT_EQ("multipart/related", main);
+  ASSERT_EQ(2u, args.size());
+  ASSERT_EQ("Application/Dicom", args["type"]);
+  ASSERT_EQ("heLLO", args["boundary"]);
+
+  ASSERT_TRUE(MultipartStreamReader::ParseHeaderArguments(main, args, "Multipart/Related; type=\"application/DICOM\"; Boundary=a"));
+  ASSERT_EQ("multipart/related", main);
+  ASSERT_EQ(2u, args.size());
+  ASSERT_EQ("application/DICOM", args["type"]);
+  ASSERT_EQ("a", args["boundary"]);
+}
+
+
 TEST(MultipartStreamReader, BytePerByte)
 {
   std::string stream = "GARBAGE";
@@ -879,6 +954,36 @@ TEST(MultipartStreamReader, BytePerByte)
 }
 
 
+TEST(MultipartStreamReader, Issue190)
+{
+  // https://bugs.orthanc-server.com/show_bug.cgi?id=190
+  // https://hg.orthanc-server.com/orthanc-dicomweb/rev/6dc2f79b5579
+
+  std::map<std::string, std::string> headers;
+  headers["content-type"] = "multipart/related; type=application/dicom; boundary=0f3cf5c0-70e0-41ef-baef-c6f9f65ec3e1";
+
+  {
+    std::string tmp, contentType, subType, boundary;
+    ASSERT_TRUE(Orthanc::MultipartStreamReader::GetMainContentType(tmp, headers));
+    ASSERT_TRUE(Orthanc::MultipartStreamReader::ParseMultipartContentType(contentType, subType, boundary, tmp));
+    ASSERT_EQ("multipart/related", contentType);
+    ASSERT_EQ("application/dicom", subType);
+    ASSERT_EQ("0f3cf5c0-70e0-41ef-baef-c6f9f65ec3e1", boundary);
+  }
+
+  headers["content-type"] = "multipart/related; type=\"application/dicom\"; boundary=\"0f3cf5c0-70e0-41ef-baef-c6f9f65ec3e1\"";
+
+  {
+    std::string tmp, contentType, subType, boundary;
+    ASSERT_TRUE(Orthanc::MultipartStreamReader::GetMainContentType(tmp, headers));
+    ASSERT_TRUE(Orthanc::MultipartStreamReader::ParseMultipartContentType(contentType, subType, boundary, tmp));
+    ASSERT_EQ("multipart/related", contentType);
+    ASSERT_EQ("application/dicom", subType);
+    ASSERT_EQ("0f3cf5c0-70e0-41ef-baef-c6f9f65ec3e1", boundary);
+  }
+}
+
+
 TEST(WebServiceParameters, Url)
 {
   WebServiceParameters w;
@@ -963,6 +1068,9 @@ TEST(ChunkedBuffer, Pending)
 
 
 
+#if ORTHANC_SANDBOXED != 1
+
+
 namespace
 {
   class TotoBody : public HttpClient::IRequestBody
@@ -981,7 +1089,7 @@ namespace
     {
     }
       
-    virtual bool ReadNextChunk(std::string& chunk)
+    virtual bool ReadNextChunk(std::string& chunk) ORTHANC_OVERRIDE
     {
       if (pos_ == size_)
       {
@@ -1018,7 +1126,7 @@ namespace
                                             const char* username,
                                             HttpMethod method,
                                             const UriComponents& uri,
-                                            const Arguments& headers)
+                                            const HttpToolbox::Arguments& headers) ORTHANC_OVERRIDE
     {
       return false;
     }
@@ -1029,10 +1137,10 @@ namespace
                         const char* username,
                         HttpMethod method,
                         const UriComponents& uri,
-                        const Arguments& headers,
-                        const GetArguments& getArguments,
+                        const HttpToolbox::Arguments& headers,
+                        const HttpToolbox::GetArguments& getArguments,
                         const void* bodyData,
-                        size_t bodySize)
+                        size_t bodySize) ORTHANC_OVERRIDE
     {
       printf("received %d\n", static_cast<int>(bodySize));
 
@@ -1053,6 +1161,7 @@ namespace
 }
 
 
+
 #include "../Sources/HttpServer/HttpServer.h"
 
 TEST(HttpClient, DISABLED_Issue156_Slow)
@@ -1111,3 +1220,4 @@ TEST(HttpClient, DISABLED_Issue156_Crash)
 
   server.Stop();
 }
+#endif
