@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -98,20 +98,23 @@ function Refresh()
 
 
 $(document).ready(function() {
-  var $tree = $('#dicom-tree');
-  $tree.tree({
-    autoEscape: false
-  });
+  var trees = [ '#dicom-tree', '#dicom-metaheader' ];
 
-  $('#dicom-tree').bind(
-    'tree.click',
-    function(event) {
-      if (event.node.is_open)
-        $tree.tree('closeNode', event.node, true);
-      else
-        $tree.tree('openNode', event.node, true);
-    }
-  );
+  for (var i = 0; i < trees.length; i++) {
+    $(trees[i]).tree({
+      autoEscape: false
+    });
+    
+    $(trees[i]).bind(
+      'tree.click',
+      function(event) {
+        if (event.node.is_open)
+          $(trees[i]).tree('closeNode', event.node, true);
+        else
+          $(trees[i]).tree('openNode', event.node, true);
+      }
+    );
+  }
 
   // Inject the template of the warning about insecure setup as the
   // first child of each page
@@ -902,6 +905,19 @@ function RefreshInstance()
               $('#dicom-tree').tree('loadData', ConvertForTree(s));
             });
 
+            GetResource('/instances/' + instance.ID + '/header', function(s) {
+              $('#dicom-metaheader').tree('loadData', ConvertForTree(s));
+            });
+
+            $('#transfer-syntax').hide();
+            GetResource('/instances/' + instance.ID + '/metadata?expand', function(s) {
+              transferSyntax = s['TransferSyntax'];
+              if (transferSyntax !== undefined) {
+                $('#transfer-syntax').show();
+                $('#transfer-syntax-text').text(transferSyntax);
+              }
+            });
+
             SetupAnonymizedOrModifiedFrom('#instance-anonymized-from', instance, 'instance', 'AnonymizedFrom');
             SetupAnonymizedOrModifiedFrom('#instance-modified-from', instance, 'instance', 'ModifiedFrom');
 
