@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -62,22 +62,22 @@ namespace Orthanc
     }
 
   public:
-    Transaction(OrthancPluginDatabase& that) :
-    that_(that)
+    explicit Transaction(OrthancPluginDatabase& that) :
+      that_(that)
     {
     }
 
-    virtual void Begin()
+    virtual void Begin() ORTHANC_OVERRIDE
     {
       CheckSuccess(that_.backend_.startTransaction(that_.payload_));
     }
 
-    virtual void Rollback()
+    virtual void Rollback() ORTHANC_OVERRIDE
     {
       CheckSuccess(that_.backend_.rollbackTransaction(that_.payload_));
     }
 
-    virtual void Commit(int64_t diskSizeDelta)
+    virtual void Commit(int64_t diskSizeDelta) ORTHANC_OVERRIDE
     {
       if (that_.fastGetTotalSize_)
       {
@@ -229,7 +229,10 @@ namespace Orthanc
     errorDictionary_(errorDictionary),
     backend_(backend),
     payload_(payload),
-    listener_(NULL)
+    listener_(NULL),
+    fastGetTotalSize_(false),
+    currentDiskSize_(0),
+    answerDoneIgnored_(false)
   {
     static const char* const MISSING = "  Missing extension in database index plugin: ";
     
@@ -249,44 +252,44 @@ namespace Orthanc
 
     if (extensions_.lookupResources == NULL)
     {
-      LOG(INFO) << MISSING << "LookupIdentifierRange()";
+      CLOG(INFO, PLUGINS) << MISSING << "LookupIdentifierRange()";
       isOptimal = false;
     }
 
     if (extensions_.createInstance == NULL)
     {
-      LOG(INFO) << MISSING << "CreateInstance()";
+      CLOG(INFO, PLUGINS) << MISSING << "CreateInstance()";
       isOptimal = false;
     }
 
     if (extensions_.setResourcesContent == NULL)
     {
-      LOG(INFO) << MISSING << "SetResourcesContent()";
+      CLOG(INFO, PLUGINS) << MISSING << "SetResourcesContent()";
       isOptimal = false;
     }
 
     if (extensions_.getChildrenMetadata == NULL)
     {
-      LOG(INFO) << MISSING << "GetChildrenMetadata()";
+      CLOG(INFO, PLUGINS) << MISSING << "GetChildrenMetadata()";
       isOptimal = false;
     }
 
     if (extensions_.getAllMetadata == NULL)
     {
-      LOG(INFO) << MISSING << "GetAllMetadata()";
+      CLOG(INFO, PLUGINS) << MISSING << "GetAllMetadata()";
       isOptimal = false;
     }
 
     if (extensions_.lookupResourceAndParent == NULL)
     {
-      LOG(INFO) << MISSING << "LookupResourceAndParent()";
+      CLOG(INFO, PLUGINS) << MISSING << "LookupResourceAndParent()";
       isOptimal = false;
     }
 
     if (isOptimal)
     {
-      LOG(INFO) << "The performance of the database index plugin "
-                << "is optimal for this version of Orthanc";
+      CLOG(INFO, PLUGINS) << "The performance of the database index plugin "
+                          << "is optimal for this version of Orthanc";
     }
     else
     {
@@ -571,11 +574,11 @@ namespace Orthanc
 
   void OrthancPluginDatabase::GetLastChange(std::list<ServerIndexChange>& target /*out*/)
   {
-    bool ignored = false;
+    answerDoneIgnored_ = false;
 
     ResetAnswers();
     answerChanges_ = &target;
-    answerDone_ = &ignored;
+    answerDone_ = &answerDoneIgnored_;
 
     CheckSuccess(backend_.getLastChange(GetContext(), payload_));
   }
@@ -583,11 +586,11 @@ namespace Orthanc
 
   void OrthancPluginDatabase::GetLastExportedResource(std::list<ExportedResource>& target /*out*/)
   {
-    bool ignored = false;
+    answerDoneIgnored_ = false;
 
     ResetAnswers();
     answerExportedResources_ = &target;
-    answerDone_ = &ignored;
+    answerDone_ = &answerDoneIgnored_;
 
     CheckSuccess(backend_.getLastExportedResource(GetContext(), payload_));
   }
