@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -288,6 +288,9 @@ namespace Orthanc
         case RequestOrigin_Unknown:
           return OrthancPluginInstanceOrigin_Unknown;
 
+        case RequestOrigin_WebDav:
+          return OrthancPluginInstanceOrigin_WebDav;
+
         default:
           throw OrthancException(ErrorCode_ParameterOutOfRange);
       }
