@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -155,10 +155,9 @@ namespace Orthanc
 
       virtual void Compute(SQLite::FunctionContext& context)
       {
-        VLOG(1) << "There exists a remaining ancestor with public ID \""
-                << context.GetStringValue(0)
-                << "\" of type "
-                << context.GetIntValue(1);
+        CLOG(TRACE, SQLITE) << "There exists a remaining ancestor with public ID \""
+                            << context.GetStringValue(0) << "\" of type "
+                            << context.GetIntValue(1);
 
         if (!hasRemainingAncestor_ ||
             remainingType_ >= context.GetIntValue(1))
