@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -74,7 +74,7 @@ namespace Orthanc
       instance.GetOrigin().Format(origin_);
     }
 
-    virtual void Apply(LuaScripting& that)
+    virtual void Apply(LuaScripting& that) ORTHANC_OVERRIDE
     {
       static const char* NAME = "OnStoredInstance";
 
@@ -103,12 +103,12 @@ namespace Orthanc
     std::string    command_;
 
   public:
-    ExecuteEvent(const std::string& command) :
+    explicit ExecuteEvent(const std::string& command) :
       command_(command)
     {
     }
 
-    virtual void Apply(LuaScripting& that)
+    virtual void Apply(LuaScripting& that) ORTHANC_OVERRIDE
     {
       LuaScripting::Lock lock(that);
 
@@ -127,12 +127,12 @@ namespace Orthanc
     ServerIndexChange  change_;
 
   public:
-    StableResourceEvent(const ServerIndexChange& change) :
-    change_(change)
+    explicit StableResourceEvent(const ServerIndexChange& change) :
+      change_(change)
     {
     }
 
-    virtual void Apply(LuaScripting& that)
+    virtual void Apply(LuaScripting& that) ORTHANC_OVERRIDE
     {
       const char* name;
 
@@ -169,7 +169,7 @@ namespace Orthanc
       if (that.context_.GetIndex().LookupResource(tags, change_.GetPublicId(), change_.GetResourceType()))
       {
         std::map<MetadataType, std::string> metadata;
-        that.context_.GetIndex().GetAllMetadata(metadata, change_.GetPublicId());
+        that.context_.GetIndex().GetAllMetadata(metadata, change_.GetPublicId(), change_.GetResourceType());
         
         Json::Value formattedMetadata = Json::objectValue;
 
@@ -223,7 +223,7 @@ namespace Orthanc
     {
     }
 
-    virtual void Apply(LuaScripting& that)
+    virtual void Apply(LuaScripting& that) ORTHANC_OVERRIDE
     {
       std::string functionName;
       
@@ -273,7 +273,7 @@ namespace Orthanc
     {
     }
 
-    virtual void Apply(LuaScripting& that)
+    virtual void Apply(LuaScripting& that) ORTHANC_OVERRIDE
     {
       std::string functionName;
       
@@ -327,7 +327,7 @@ namespace Orthanc
     {
     }
 
-    virtual void Apply(LuaScripting& that)
+    virtual void Apply(LuaScripting& that) ORTHANC_OVERRIDE
     {
       std::string functionName;
       
@@ -405,8 +405,8 @@ namespace Orthanc
     try
     {
       std::string result;
-      if (HttpToolbox::SimpleGet(result, serverContext->GetHttpHandler().RestrictToOrthancRestApi(builtin), 
-                                 RequestOrigin_Lua, uri, headers))
+      if (IHttpHandler::SimpleGet(result, serverContext->GetHttpHandler().RestrictToOrthancRestApi(builtin), 
+                                  RequestOrigin_Lua, uri, headers))
       {
         lua_pushlstring(state, result.c_str(), result.size());
         return 1;
@@ -458,10 +458,10 @@ namespace Orthanc
     {
       std::string result;
       if (isPost ?
-          HttpToolbox::SimplePost(result, serverContext->GetHttpHandler().RestrictToOrthancRestApi(builtin), 
-                                  RequestOrigin_Lua, uri, bodyData, bodySize, headers) :
-          HttpToolbox::SimplePut(result, serverContext->GetHttpHandler().RestrictToOrthancRestApi(builtin), 
-                                 RequestOrigin_Lua, uri, bodyData, bodySize, headers))
+          IHttpHandler::SimplePost(result, serverContext->GetHttpHandler().RestrictToOrthancRestApi(builtin), 
+                                   RequestOrigin_Lua, uri, bodyData, bodySize, headers) :
+          IHttpHandler::SimplePut(result, serverContext->GetHttpHandler().RestrictToOrthancRestApi(builtin), 
+                                  RequestOrigin_Lua, uri, bodyData, bodySize, headers))
       {
         lua_pushlstring(state, result.c_str(), result.size());
         return 1;
@@ -522,8 +522,8 @@ namespace Orthanc
     
     try
     {
-      if (HttpToolbox::SimpleDelete(serverContext->GetHttpHandler().RestrictToOrthancRestApi(builtin), 
-                                    RequestOrigin_Lua, uri, headers))
+      if (IHttpHandler::SimpleDelete(serverContext->GetHttpHandler().RestrictToOrthancRestApi(builtin), 
+                                     RequestOrigin_Lua, uri, headers))
       {
         lua_pushboolean(state, 1);
         return 1;
