@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -44,6 +44,8 @@
 
 #include "ServerIndex.h"
 
+#include <boost/regex.hpp>
+
 
 static const char* const DICOM_MODALITIES = "DicomModalities";
 static const char* const DICOM_MODALITIES_IN_DB = "DicomModalitiesInDatabase";
@@ -70,8 +72,7 @@ namespace Orthanc
       content = Toolbox::SubstituteVariables(content, env);
 
       Json::Value tmp;
-      Json::Reader reader;
-      if (!reader.parse(content, tmp) ||
+      if (!Toolbox::ReadJson(tmp, content) ||
           tmp.type() != Json::objectValue)
       {
         throw OrthancException(ErrorCode_BadJson,
@@ -181,10 +182,10 @@ namespace Orthanc
     for (size_t j = 0; j < s.size(); j++)
     {
       if (!isalnum(s[j]) && 
-          s[j] != '-')
+          s[j] != '-' && s[j] != '_')
       {
         throw OrthancException(ErrorCode_BadFileFormat,
-                               "Only alphanumeric and dash characters are allowed "
+                               "Only alphanumeric, dash characters and underscores are allowed "
                                "in the names of modalities/peers, but found: " + s);
       }
     }
@@ -254,9 +255,8 @@ namespace Orthanc
       {
         std::string property = serverIndex_->GetGlobalProperty(GlobalProperty_Modalities, "{}");
 
-        Json::Reader reader;
         Json::Value modalities;
-        if (reader.parse(property, modalities))
+        if (Toolbox::ReadJson(modalities, property))
         {
           LoadModalitiesFromJson(modalities);
         }
@@ -294,9 +294,8 @@ namespace Orthanc
       {
         std::string property = serverIndex_->GetGlobalProperty(GlobalProperty_Peers, "{}");
 
-        Json::Reader reader;
         Json::Value peers;
-        if (reader.parse(property, peers))
+        if (Toolbox::ReadJson(peers, property))
         {
           LoadPeersFromJson(peers);
         }
@@ -366,9 +365,9 @@ namespace Orthanc
         Json::Value modalities;
         SaveModalitiesToJson(modalities);
 
-        Json::FastWriter writer;
-        std::string s = writer.write(modalities);
-
+        std::string s;
+        Toolbox::WriteFastJson(s, modalities);
+        
         serverIndex_->SetGlobalProperty(GlobalProperty_Modalities, s);
       }
     }
@@ -398,8 +397,8 @@ namespace Orthanc
         Json::Value peers;
         SavePeersToJson(peers);
 
-        Json::FastWriter writer;
-        std::string s = writer.write(peers);
+        std::string s;
+        Toolbox::WriteFastJson(s, peers);
 
         serverIndex_->SetGlobalProperty(GlobalProperty_Peers, s);
       }
@@ -580,8 +579,15 @@ namespace Orthanc
 
   void OrthancConfiguration::LoadModalitiesAndPeers()
   {
-    LoadModalities();
-    LoadPeers();
+    if (serverIndex_ == NULL)
+    {
+      throw OrthancException(ErrorCode_BadSequenceOfCalls);
+    }
+    else
+    {
+      LoadModalities();
+      LoadPeers();
+    }
   }
 
 
@@ -746,6 +752,23 @@ namespace Orthanc
     return false;
   }
 
+  bool OrthancConfiguration::LookupDicomModalitiesUsingAETitle(std::list<RemoteModalityParameters>& modalities,
+                                                               const std::string& aet) const
+  {
+    modalities.clear();
+
+    for (Modalities::const_iterator it = modalities_.begin(); it != modalities_.end(); ++it)
+    {
+      if (IsSameAETitle(aet, it->second.GetApplicationEntityTitle()))
+      {
+        modalities.push_back(it->second);
+      }
+    }
+
+    return modalities.size() > 0;
+  }
+
+
 
   bool OrthancConfiguration::IsKnownAETitle(const std::string& aet,
                                             const std::string& ip) const
@@ -813,8 +836,16 @@ namespace Orthanc
 
   void OrthancConfiguration::RemoveModality(const std::string& symbolicName)
   {
-    modalities_.erase(symbolicName);
-    SaveModalities();
+    if (modalities_.find(symbolicName) == modalities_.end())
+    {
+      throw OrthancException(ErrorCode_InexistentItem,
+                             "Unknown DICOM modality with symbolic name: " + symbolicName);
+    }
+    else
+    {
+      modalities_.erase(symbolicName);
+      SaveModalities();
+    }
   }
 
     
@@ -832,15 +863,22 @@ namespace Orthanc
 
   void OrthancConfiguration::RemovePeer(const std::string& symbolicName)
   {
-    peers_.erase(symbolicName);
-    SavePeers();
+    if (peers_.find(symbolicName) == peers_.end())
+    {
+      throw OrthancException(ErrorCode_InexistentItem,
+                             "Unknown Orthanc peer: " + symbolicName);
+    }
+    else
+    {
+      peers_.erase(symbolicName);
+      SavePeers();
+    }
   }
 
 
   void OrthancConfiguration::Format(std::string& result) const
   {
-    Json::StyledWriter w;
-    result = w.write(json_);
+    Toolbox::WriteStyledJson(result, json_);
   }
 
 
@@ -859,9 +897,9 @@ namespace Orthanc
     Json::Value current;
     ReadConfiguration(current, configurationFileArg_);
 
-    Json::FastWriter writer;
-    std::string a = writer.write(json_);
-    std::string b = writer.write(current);
+    std::string a, b;
+    Toolbox::WriteFastJson(a, json_);
+    Toolbox::WriteFastJson(b, current);
 
     return a != b;
   }
@@ -899,8 +937,84 @@ namespace Orthanc
   }
 
 
+  static void GetAcceptOption(std::set<DicomTransferSyntax>& target,
+                              const OrthancConfiguration& configuration,
+                              const std::string& optionName,
+                              TransferSyntaxGroup optionGroup)
+  {
+    bool accept;
+    if (configuration.LookupBooleanParameter(accept, optionName))
+    {
+      std::set<DicomTransferSyntax> group;
+      GetTransferSyntaxGroup(group, optionGroup);
+
+      for (std::set<DicomTransferSyntax>::const_iterator
+             syntax = group.begin(); syntax != group.end(); ++syntax)
+      {
+        if (accept)
+        {
+          target.insert(*syntax);
+        }
+        else
+        {
+          target.erase(*syntax);
+        }
+      }
+    }
+  }
+
+
+  void OrthancConfiguration::GetAcceptedTransferSyntaxes(std::set<DicomTransferSyntax>& target) const
+  {
+    target.clear();
+
+#if 1
+    /**
+     * This is the behavior in Orthanc >= 1.9.0. All the transfer
+     * syntaxes are accepted by default, and the
+     * "TransferSyntaxAccepted" options can be used to disable groups
+     * of transfer syntaxes.
+     **/
+
+    static const char* const ACCEPTED_TRANSFER_SYNTAXES = "AcceptedTransferSyntaxes";
+
+    if (json_.isMember(ACCEPTED_TRANSFER_SYNTAXES))
+    {
+      ParseAcceptedTransferSyntaxes(target, json_[ACCEPTED_TRANSFER_SYNTAXES]);
+    }
+    else
+    {
+      GetAllDicomTransferSyntaxes(target);
+    }
+#else
+    /**
+     * This was the behavior of Orthanc <= 1.8.2. The uncompressed
+     * transfer syntaxes were always accepted, and additional transfer
+     * syntaxes were added using the configuration options
+     * "XXXTransferSyntaxAccepted".
+     **/
+
+    // The 3 transfer syntaxes below were the only ones to be supported in Orthanc <= 0.7.1
+    target.insert(DicomTransferSyntax_LittleEndianExplicit);
+    target.insert(DicomTransferSyntax_BigEndianExplicit);
+    target.insert(DicomTransferSyntax_LittleEndianImplicit);
+#endif
+
+    // Groups of transfer syntaxes, supported since Orthanc 0.7.2
+    GetAcceptOption(target, *this, "DeflatedTransferSyntaxAccepted", TransferSyntaxGroup_Deflated);
+    GetAcceptOption(target, *this, "JpegTransferSyntaxAccepted", TransferSyntaxGroup_Jpeg);
+    GetAcceptOption(target, *this, "Jpeg2000TransferSyntaxAccepted", TransferSyntaxGroup_Jpeg2000);
+    GetAcceptOption(target, *this, "JpegLosslessTransferSyntaxAccepted", TransferSyntaxGroup_JpegLossless);
+    GetAcceptOption(target, *this, "JpipTransferSyntaxAccepted", TransferSyntaxGroup_Jpip);
+    GetAcceptOption(target, *this, "Mpeg2TransferSyntaxAccepted", TransferSyntaxGroup_Mpeg2);
+    GetAcceptOption(target, *this, "Mpeg4TransferSyntaxAccepted", TransferSyntaxGroup_Mpeg4);
+    GetAcceptOption(target, *this, "RleTransferSyntaxAccepted", TransferSyntaxGroup_Rle);
+    GetAcceptOption(target, *this, "H265TransferSyntaxAccepted", TransferSyntaxGroup_H265);
+  }
+
+
   void OrthancConfiguration::DefaultExtractDicomSummary(DicomMap& target,
-                                                        ParsedDicomFile& dicom)
+                                                        const ParsedDicomFile& dicom)
   {
     std::set<DicomTag> ignoreTagLength;
     dicom.ExtractDicomSummary(target, ORTHANC_MAXIMUM_TAG_LENGTH, ignoreTagLength);
@@ -908,7 +1022,7 @@ namespace Orthanc
   
     
   void OrthancConfiguration::DefaultDicomDatasetToJson(Json::Value& target,
-                                                       ParsedDicomFile& dicom)
+                                                       const ParsedDicomFile& dicom)
   {
     std::set<DicomTag> ignoreTagLength;
     DefaultDicomDatasetToJson(target, dicom, ignoreTagLength);
@@ -916,7 +1030,7 @@ namespace Orthanc
   
     
   void OrthancConfiguration::DefaultDicomDatasetToJson(Json::Value& target,
-                                                       ParsedDicomFile& dicom,
+                                                       const ParsedDicomFile& dicom,
                                                        const std::set<DicomTag>& ignoreTagLength)
   {
     dicom.DatasetToJson(target, DicomToJsonFormat_Full, DicomToJsonFlags_Default, 
@@ -925,9 +1039,55 @@ namespace Orthanc
   
 
   void OrthancConfiguration::DefaultDicomHeaderToJson(Json::Value& target,
-                                                      ParsedDicomFile& dicom)
+                                                      const ParsedDicomFile& dicom)
   {
-    std::set<DicomTag> ignoreTagLength;
     dicom.HeaderToJson(target, DicomToJsonFormat_Full);
   }
+
+
+  static void AddTransferSyntaxes(std::set<DicomTransferSyntax>& target,
+                                  const std::string& source)
+  {
+    boost::regex pattern(Toolbox::WildcardToRegularExpression(source));
+
+    std::set<DicomTransferSyntax> allSyntaxes;
+    GetAllDicomTransferSyntaxes(allSyntaxes);
+
+    for (std::set<DicomTransferSyntax>::const_iterator
+           syntax = allSyntaxes.begin(); syntax != allSyntaxes.end(); ++syntax)
+    {
+      if (regex_match(GetTransferSyntaxUid(*syntax), pattern))
+      {
+        target.insert(*syntax);
+      }
+    }
+  }
+
+
+  void OrthancConfiguration::ParseAcceptedTransferSyntaxes(std::set<DicomTransferSyntax>& target,
+                                                           const Json::Value& source)
+  {
+    if (source.type() == Json::stringValue)
+    {
+      AddTransferSyntaxes(target, source.asString());
+    }
+    else if (source.type() == Json::arrayValue)
+    {
+      for (Json::Value::ArrayIndex i = 0; i < source.size(); i++)
+      {
+        if (source[i].type() == Json::stringValue)
+        {
+          AddTransferSyntaxes(target, source[i].asString());
+        }
+        else
+        {
+          throw OrthancException(ErrorCode_BadFileFormat);
+        }
+      }
+    }
+    else
+    {
+      throw OrthancException(ErrorCode_BadFileFormat);
+    }
+  }
 }
