@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -69,7 +69,8 @@ namespace Orthanc
     ServerIndex*             serverIndex_;
 
     OrthancConfiguration() :
-      configurationFileArg_(NULL)
+      configurationFileArg_(NULL),
+      serverIndex_(NULL)
     {
     }
 
@@ -159,6 +160,7 @@ namespace Orthanc
 
     void Read(const char* configurationFile);
 
+    // "SetServerIndex()" must have been called
     void LoadModalitiesAndPeers();
     
     void RegisterFont(ServerResources::FileResourceId resource);
@@ -202,6 +204,9 @@ namespace Orthanc
     bool IsSameAETitle(const std::string& aet1,
                        const std::string& aet2) const;
 
+    bool LookupDicomModalitiesUsingAETitle(std::list<RemoteModalityParameters>& modalities,
+                                           const std::string& aet) const;
+
     bool LookupDicomModalityUsingAETitle(RemoteModalityParameters& modality,
                                          const std::string& aet) const;
 
@@ -242,17 +247,22 @@ namespace Orthanc
       return GetStringParameter("DicomAet", "ORTHANC");
     }
 
+    void GetAcceptedTransferSyntaxes(std::set<DicomTransferSyntax>& target) const;
+
     static void DefaultExtractDicomSummary(DicomMap& target,
-                                           ParsedDicomFile& dicom);
+                                           const ParsedDicomFile& dicom);
     
     static void DefaultDicomDatasetToJson(Json::Value& target,
-                                          ParsedDicomFile& dicom);
+                                          const ParsedDicomFile& dicom);
     
     static void DefaultDicomDatasetToJson(Json::Value& target,
-                                          ParsedDicomFile& dicom,
+                                          const ParsedDicomFile& dicom,
                                           const std::set<DicomTag>& ignoreTagLength);
     
     static void DefaultDicomHeaderToJson(Json::Value& target,
-                                         ParsedDicomFile& dicom);
+                                         const ParsedDicomFile& dicom);
+
+    static void ParseAcceptedTransferSyntaxes(std::set<DicomTransferSyntax>& target,
+                                              const Json::Value& source);
   };
 }
