@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,6 +38,14 @@
 #  include <winsock2.h>
 #endif
 
+#if !defined(HAVE_MALLOPT)
+#  error Macro HAVE_MALLOPT must be defined
+#endif
+
+#if HAVE_MALLOPT == 1
+#  include <malloc.h>
+#endif
+
 #include "OrthancInitialization.h"
 
 #include "../../OrthancFramework/Sources/DicomParsing/FromDcmtkBridge.h"
@@ -51,7 +59,8 @@
 
 #include <OrthancServerResources.h>
 
-#include <dcmtk/dcmnet/dul.h>   // For dcmDisableGethostbyaddr()
+#include <dcmtk/dcmnet/dul.h>     // For dcmDisableGethostbyaddr()
+#include <dcmtk/dcmnet/diutil.h>  // For DCM_dcmnetLogger
 
 
 
@@ -226,6 +235,11 @@ namespace Orthanc
 
   void OrthancInitialize(const char* configurationFile)
   {
+    static const char* LOCALE = "Locale";
+    static const char* PKCS11 = "Pkcs11";
+    static const char* DEFAULT_ENCODING = "DefaultEncoding";
+    static const char* MALLOC_ARENA_MAX = "MallocArenaMax";
+    
     OrthancConfiguration::WriterLock lock;
 
     InitializeServerEnumerations();
@@ -236,9 +250,9 @@ namespace Orthanc
     {
       std::string locale;
       
-      if (lock.GetJson().isMember("Locale"))
+      if (lock.GetJson().isMember(LOCALE))
       {
-        locale = lock.GetConfiguration().GetStringParameter("Locale", "");
+        locale = lock.GetConfiguration().GetStringParameter(LOCALE, "");
       }
       
       bool loadPrivate = lock.GetConfiguration().GetBooleanParameter("LoadPrivateDictionary", true);
@@ -247,9 +261,9 @@ namespace Orthanc
 
     // The Orthanc framework is now initialized
 
-    if (lock.GetJson().isMember("DefaultEncoding"))
+    if (lock.GetJson().isMember(DEFAULT_ENCODING))
     {
-      std::string encoding = lock.GetConfiguration().GetStringParameter("DefaultEncoding", "");
+      std::string encoding = lock.GetConfiguration().GetStringParameter(DEFAULT_ENCODING, "");
       SetDefaultDicomEncoding(StringToEncoding(encoding.c_str()));
     }
     else
@@ -257,9 +271,9 @@ namespace Orthanc
       SetDefaultDicomEncoding(ORTHANC_DEFAULT_DICOM_ENCODING);
     }
 
-    if (lock.GetJson().isMember("Pkcs11"))
+    if (lock.GetJson().isMember(PKCS11))
     {
-      ConfigurePkcs11(lock.GetJson()["Pkcs11"]);
+      ConfigurePkcs11(lock.GetJson()[PKCS11]);
     }
 
     RegisterUserMetadata(lock.GetJson());
@@ -268,6 +282,28 @@ namespace Orthanc
     LoadCustomDictionary(lock.GetJson());
 
     lock.GetConfiguration().RegisterFont(ServerResources::FONT_UBUNTU_MONO_BOLD_16);
+
+#if HAVE_MALLOPT == 1
+    // New in Orthanc 1.8.2
+    // https://book.orthanc-server.com/faq/scalability.html#controlling-memory-usage
+    unsigned int maxArena = lock.GetConfiguration().GetUnsignedIntegerParameter(MALLOC_ARENA_MAX, 5);
+    if (maxArena != 0)
+    {
+      // https://man7.org/linux/man-pages/man3/mallopt.3.html
+      LOG(INFO) << "Calling mallopt(M_ARENA_MAX, " << maxArena << ")";
+      if (mallopt(M_ARENA_MAX, maxArena) != 1 /* success */)
+      {
+        throw OrthancException(ErrorCode_InternalError, "The call to mallopt(M_ARENA_MAX, " +
+                               boost::lexical_cast<std::string>(maxArena) + ") has failed");
+      }
+    }
+#else
+    if (lock.GetJson().isMember(MALLOC_ARENA_MAX))
+    {
+      LOG(INFO) << "Your platform does not support mallopt(), ignoring configuration option \""
+                << MALLOC_ARENA_MAX << "\"";
+    }
+#endif
   }
 
 
@@ -314,14 +350,16 @@ namespace Orthanc
       FilesystemStorage storage_;
 
     public:
-      FilesystemStorageWithoutDicom(const std::string& path) : storage_(path)
+      FilesystemStorageWithoutDicom(const std::string& path,
+                                    bool fsyncOnWrite) :
+        storage_(path, fsyncOnWrite)
       {
       }
 
       virtual void Create(const std::string& uuid,
                           const void* content, 
                           size_t size,
-                          FileContentType type)
+                          FileContentType type) ORTHANC_OVERRIDE
       {
         if (type != FileContentType_Dicom)
         {
@@ -329,13 +367,12 @@ namespace Orthanc
         }
       }
 
-      virtual void Read(std::string& content,
-                        const std::string& uuid,
-                        FileContentType type)
+      virtual IMemoryBuffer* Read(const std::string& uuid,
+                                  FileContentType type) ORTHANC_OVERRIDE
       {
         if (type != FileContentType_Dicom)
         {
-          storage_.Read(content, uuid, type);
+          return storage_.Read(uuid, type);
         }
         else
         {
@@ -344,7 +381,7 @@ namespace Orthanc
       }
 
       virtual void Remove(const std::string& uuid,
-                          FileContentType type) 
+                          FileContentType type) ORTHANC_OVERRIDE
       {
         if (type != FileContentType_Dicom)
         {
@@ -367,14 +404,17 @@ namespace Orthanc
 
     LOG(WARNING) << "Storage directory: " << storageDirectory;
 
+    // New in Orthanc 1.7.4
+    bool fsyncOnWrite = lock.GetConfiguration().GetBooleanParameter("SyncStorageArea", true);
+
     if (lock.GetConfiguration().GetBooleanParameter("StoreDicom", true))
     {
-      return new FilesystemStorage(storageDirectory.string());
+      return new FilesystemStorage(storageDirectory.string(), fsyncOnWrite);
     }
     else
     {
       LOG(WARNING) << "The DICOM files will not be stored, Orthanc running in index-only mode";
-      return new FilesystemStorageWithoutDicom(storageDirectory.string());
+      return new FilesystemStorageWithoutDicom(storageDirectory.string(), fsyncOnWrite);
     }
   }
 
@@ -388,5 +428,134 @@ namespace Orthanc
   IStorageArea* CreateStorageArea()
   {
     return CreateFilesystemStorage();
-  }  
+  }
+
+
+  static void SetDcmtkVerbosity(Verbosity verbosity)
+  {
+    // INFO_LOG_LEVEL was the DCMTK log level in Orthanc <= 1.8.0    
+    // https://support.dcmtk.org/docs-dcmrt/classOFLogger.html#ae20bf2616f15313c1f089da2eefb8245
+
+    OFLogger::LogLevel dataLevel, networkLevel;
+
+    switch (verbosity)
+    {
+      case Verbosity_Default:
+        // Turn off logging in DCMTK core
+        dataLevel = OFLogger::OFF_LOG_LEVEL;
+        networkLevel = OFLogger::OFF_LOG_LEVEL;
+        break;
+
+      case Verbosity_Verbose:
+        dataLevel = OFLogger::INFO_LOG_LEVEL;
+        networkLevel = OFLogger::INFO_LOG_LEVEL;
+        break;
+
+      case Verbosity_Trace:
+        dataLevel = OFLogger::INFO_LOG_LEVEL;  // DEBUG here makes DCMTK too verbose to be useful
+        networkLevel = OFLogger::DEBUG_LOG_LEVEL;
+        break;
+
+      default:
+        throw OrthancException(ErrorCode_ParameterOutOfRange);
+    }
+
+    OFLog::configure(dataLevel);
+    assert(dcmtk::log4cplus::Logger::getRoot().getChainedLogLevel() == dataLevel);
+    
+    DCM_dcmdataLogger.setLogLevel(dataLevel);    // This seems to be implied by "OFLog::configure()"
+    DCM_dcmnetLogger.setLogLevel(networkLevel);  // This will display PDU in DICOM networking
+  }
+
+
+  void SetGlobalVerbosity(Verbosity verbosity)
+  {
+    SetDcmtkVerbosity(verbosity);
+    
+    switch (verbosity)
+    {
+      case Verbosity_Default:
+        Logging::EnableInfoLevel(false);
+        Logging::EnableTraceLevel(false);
+        break;
+
+      case Verbosity_Verbose:
+        Logging::EnableInfoLevel(true);
+        Logging::EnableTraceLevel(false);
+        break;
+
+      case Verbosity_Trace:
+        Logging::EnableInfoLevel(true);
+        Logging::EnableTraceLevel(true);
+        break;
+
+      default:
+        throw OrthancException(ErrorCode_ParameterOutOfRange);
+    }
+  }
+
+  
+  Verbosity GetGlobalVerbosity()
+  {
+    if (Logging::IsTraceLevelEnabled())
+    {
+      return Verbosity_Trace;
+    }
+    else if (Logging::IsInfoLevelEnabled())
+    {
+      return Verbosity_Verbose;
+    }
+    else
+    {
+      return Verbosity_Default;
+    }
+  }
+
+  
+  void SetCategoryVerbosity(Logging::LogCategory category,
+                            Verbosity verbosity)
+  {
+    switch (verbosity)
+    {
+      case Verbosity_Default:
+        Logging::SetCategoryEnabled(Logging::LogLevel_INFO, category, false);
+        Logging::SetCategoryEnabled(Logging::LogLevel_TRACE, category, false);
+        break;
+
+      case Verbosity_Verbose:
+        Logging::SetCategoryEnabled(Logging::LogLevel_INFO, category, true);
+        Logging::SetCategoryEnabled(Logging::LogLevel_TRACE, category, false);
+        break;
+
+      case Verbosity_Trace:
+        Logging::SetCategoryEnabled(Logging::LogLevel_INFO, category, true);
+        Logging::SetCategoryEnabled(Logging::LogLevel_TRACE, category, true);
+        break;
+
+      default:
+        throw OrthancException(ErrorCode_ParameterOutOfRange);
+    }
+
+    if (category == Logging::LogCategory_DICOM)
+    {
+      SetDcmtkVerbosity(verbosity);
+    }
+  }
+  
+
+  Verbosity GetCategoryVerbosity(Logging::LogCategory category)
+  {
+    if (Logging::IsCategoryEnabled(Logging::LogLevel_TRACE, category))
+    {
+      return Verbosity_Trace;
+    }
+    else if (Logging::IsCategoryEnabled(Logging::LogLevel_INFO, category))
+    {
+      return Verbosity_Verbose;
+    }
+    else
+    {
+      return Verbosity_Default;
+    }
+  }
 }
