@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -58,6 +58,54 @@ namespace Orthanc
   }
 
 
+  static void DocumentModifyOptions(RestApiPostCall& call)
+  {
+    // Check out "DicomModification::ParseModifyRequest()"
+    call.GetDocumentation()
+      .SetRequestField("Transcode", RestApiCallDocumentation::Type_String,
+                       "Transcode the DICOM instances to the provided DICOM transfer syntax: "
+                       "https://book.orthanc-server.com/faq/transcoding.html", false)
+      .SetRequestField("Force", RestApiCallDocumentation::Type_Boolean,
+                       "Allow the modification of tags related to DICOM identifiers, at the risk of "
+                       "breaking the DICOM model of the real world", false)
+      .SetRequestField("RemovePrivateTags", RestApiCallDocumentation::Type_Boolean,
+                       "Remove the private tags from the DICOM instances (defaults to `false`)", false)
+      .SetRequestField("Replace", RestApiCallDocumentation::Type_JsonObject,
+                       "Associative array to change the value of some DICOM tags in the DICOM instances", false)
+      .SetRequestField("Remove", RestApiCallDocumentation::Type_JsonListOfStrings,
+                       "List of tags that must be removed from the DICOM instances", false)
+      .SetRequestField("Keep", RestApiCallDocumentation::Type_JsonListOfStrings,
+                       "Keep the original value of the specified tags, to be chosen among the `StudyInstanceUID`, "
+                       "`SeriesInstanceUID` and `SOPInstanceUID` tags. Avoid this feature as much as possible, "
+                       "as this breaks the DICOM model of the real world.", false)
+      .SetRequestField("PrivateCreator", RestApiCallDocumentation::Type_String,
+                       "The private creator to be used for private tags in `Replace`", false);
+  }
+
+
+  static void DocumentAnonymizationOptions(RestApiPostCall& call)
+  {
+    // Check out "DicomModification::ParseAnonymizationRequest()"
+    call.GetDocumentation()
+      .SetRequestField("Force", RestApiCallDocumentation::Type_Boolean,
+                       "Allow the modification of tags related to DICOM identifiers, at the risk of "
+                       "breaking the DICOM model of the real world", false)
+      .SetRequestField("DicomVersion", RestApiCallDocumentation::Type_String,
+                       "Version of the DICOM standard to be used for anonymization. Check out "
+                       "configuration option `DeidentifyLogsDicomVersion` for possible values.", false)
+      .SetRequestField("KeepPrivateTags", RestApiCallDocumentation::Type_Boolean,
+                       "Keep the private tags from the DICOM instances (defaults to `false`)", false)
+      .SetRequestField("Replace", RestApiCallDocumentation::Type_JsonObject,
+                       "Associative array to change the value of some DICOM tags in the DICOM instances", false)
+      .SetRequestField("Remove", RestApiCallDocumentation::Type_JsonListOfStrings,
+                       "List of additional tags to be removed from the DICOM instances", false)
+      .SetRequestField("Keep", RestApiCallDocumentation::Type_JsonListOfStrings,
+                       "List of DICOM tags whose value must not be destroyed by the anonymization", false)
+      .SetRequestField("PrivateCreator", RestApiCallDocumentation::Type_String,
+                       "The private creator to be used for private tags in `Replace`", false);
+  }
+
+
   static void ParseModifyRequest(Json::Value& request,
                                  DicomModification& target,
                                  const RestApiPostCall& call)
@@ -159,6 +207,19 @@ namespace Orthanc
 
   static void ModifyInstance(RestApiPostCall& call)
   {
+    if (call.IsDocumentation())
+    {
+      DocumentModifyOptions(call);
+      call.GetDocumentation()
+        .SetTag("Instances")
+        .SetSummary("Modify instance")
+        .SetDescription("Download a modified version of the DICOM instance whose Orthanc identifier is provided in the URL: "
+                        "https://book.orthanc-server.com/users/anonymization.html#modification-of-a-single-instance")
+        .SetUriArgument("id", "Orthanc identifier of the instance of interest")
+        .AddAnswerType(MimeType_Dicom, "The modified DICOM instance");
+      return;
+    }
+
     DicomModification modification;
     modification.SetAllowManualIdentifiers(true);
 
@@ -207,6 +268,19 @@ namespace Orthanc
 
   static void AnonymizeInstance(RestApiPostCall& call)
   {
+    if (call.IsDocumentation())
+    {
+      DocumentAnonymizationOptions(call);
+      call.GetDocumentation()
+        .SetTag("Instances")
+        .SetSummary("Anonymize instance")
+        .SetDescription("Download an anonymized version of the DICOM instance whose Orthanc identifier is provided in the URL: "
+                        "https://book.orthanc-server.com/users/anonymization.html#anonymization-of-a-single-instance")
+        .SetUriArgument("id", "Orthanc identifier of the instance of interest")
+        .AddAnswerType(MimeType_Dicom, "The anonymized DICOM instance");
+      return;
+    }
+
     DicomModification modification;
     modification.SetAllowManualIdentifiers(true);
 
@@ -260,6 +334,22 @@ namespace Orthanc
   template <enum ResourceType resourceType>
   static void ModifyResource(RestApiPostCall& call)
   {
+    if (call.IsDocumentation())
+    {
+      OrthancRestApi::DocumentSubmitCommandsJob(call);
+      DocumentModifyOptions(call);
+      const std::string r = GetResourceTypeText(resourceType, false /* plural */, false /* lower case */);      
+      call.GetDocumentation()
+        .SetTag(GetResourceTypeText(resourceType, true /* plural */, true /* upper case */))
+        .SetSummary("Modify " + r)
+        .SetDescription("Start a job that will modify all the DICOM instances within the " + r +
+                        " whose identifier is provided in the URL. The modified DICOM instances will be "
+                        "stored into a brand new " + r + ", whose Orthanc identifiers will be returned by the job. "
+                        "https://book.orthanc-server.com/users/anonymization.html#modification-of-studies-or-series")
+        .SetUriArgument("id", "Orthanc identifier of the " + r + " of interest");
+      return;
+    }
+    
     std::unique_ptr<DicomModification> modification(new DicomModification);
 
     Json::Value body;
@@ -275,6 +365,22 @@ namespace Orthanc
   template <enum ResourceType resourceType>
   static void AnonymizeResource(RestApiPostCall& call)
   {
+    if (call.IsDocumentation())
+    {
+      OrthancRestApi::DocumentSubmitCommandsJob(call);
+      DocumentAnonymizationOptions(call);
+      const std::string r = GetResourceTypeText(resourceType, false /* plural */, false /* lower case */);      
+      call.GetDocumentation()
+        .SetTag(GetResourceTypeText(resourceType, true /* plural */, true /* upper case */))
+        .SetSummary("Anonymize " + r)
+        .SetDescription("Start a job that will anonymize all the DICOM instances within the " + r +
+                        " whose identifier is provided in the URL. The modified DICOM instances will be "
+                        "stored into a brand new " + r + ", whose Orthanc identifiers will be returned by the job. "
+                        "https://book.orthanc-server.com/users/anonymization.html#anonymization-of-patients-studies-or-series")
+        .SetUriArgument("id", "Orthanc identifier of the " + r + " of interest");
+      return;
+    }
+
     std::unique_ptr<DicomModification> modification(new DicomModification);
 
     Json::Value body;
@@ -347,7 +453,8 @@ namespace Orthanc
   static void InjectTags(ParsedDicomFile& dicom,
                          const Json::Value& tags,
                          bool decodeBinaryTags,
-                         const std::string& privateCreator)
+                         const std::string& privateCreator,
+                         bool force)
   {
     if (tags.type() != Json::objectValue)
     {
@@ -363,7 +470,8 @@ namespace Orthanc
 
       if (tag != DICOM_TAG_SPECIFIC_CHARACTER_SET)
       {
-        if (tag != DICOM_TAG_PATIENT_ID &&
+        if (!force &&
+            tag != DICOM_TAG_PATIENT_ID &&
             tag != DICOM_TAG_ACQUISITION_DATE &&
             tag != DICOM_TAG_ACQUISITION_TIME &&
             tag != DICOM_TAG_CONTENT_DATE &&
@@ -396,7 +504,8 @@ namespace Orthanc
                            ParsedDicomFile& base /* in */,
                            const Json::Value& content,
                            bool decodeBinaryTags,
-                           const std::string& privateCreator)
+                           const std::string& privateCreator,
+                           bool force)
   {
     assert(content.isArray());
     assert(content.size() > 0);
@@ -429,7 +538,7 @@ namespace Orthanc
 
           if (content[i].isMember("Tags"))
           {
-            InjectTags(*dicom, content[i]["Tags"], decodeBinaryTags, privateCreator);
+            InjectTags(*dicom, content[i]["Tags"], decodeBinaryTags, privateCreator, force);
           }
         }
 
@@ -471,11 +580,21 @@ namespace Orthanc
   static void CreateDicomV2(RestApiPostCall& call,
                             const Json::Value& request)
   {
+    static const char* const CONTENT = "Content";
+    static const char* const FORCE = "Force";
+    static const char* const INTERPRET_BINARY_TAGS = "InterpretBinaryTags";
+    static const char* const PARENT = "Parent";
+    static const char* const PRIVATE_CREATOR = "PrivateCreator";
+    static const char* const SPECIFIC_CHARACTER_SET_2 = "SpecificCharacterSet";
+    static const char* const TAGS = "Tags";
+    static const char* const TYPE = "Type";
+    static const char* const VALUE = "Value";
+    
     assert(request.isObject());
     ServerContext& context = OrthancRestApi::GetContext(call);
 
-    if (!request.isMember("Tags") ||
-        request["Tags"].type() != Json::objectValue)
+    if (!request.isMember(TAGS) ||
+        request[TAGS].type() != Json::objectValue)
     {
       throw OrthancException(ErrorCode_BadRequest);
     }
@@ -485,9 +604,9 @@ namespace Orthanc
     {
       Encoding encoding;
 
-      if (request["Tags"].isMember("SpecificCharacterSet"))
+      if (request[TAGS].isMember(SPECIFIC_CHARACTER_SET_2))
       {
-        const char* tmp = request["Tags"]["SpecificCharacterSet"].asCString();
+        const char* tmp = request[TAGS][SPECIFIC_CHARACTER_SET_2].asCString();
         if (!GetDicomEncoding(encoding, tmp))
         {
           throw OrthancException(ErrorCode_ParameterOutOfRange,
@@ -504,10 +623,10 @@ namespace Orthanc
 
     ResourceType parentType = ResourceType_Instance;
 
-    if (request.isMember("Parent"))
+    if (request.isMember(PARENT))
     {
       // Locate the parent tags
-      std::string parent = request["Parent"].asString();
+      std::string parent = request[PARENT].asString();
       if (!context.GetIndex().LookupResourceType(parentType, parent))
       {
         throw OrthancException(ErrorCode_CreateDicomBadParent);
@@ -547,9 +666,9 @@ namespace Orthanc
         {
           Encoding encoding;
 
-          if (!siblingTags[SPECIFIC_CHARACTER_SET].isMember("Value") ||
-              siblingTags[SPECIFIC_CHARACTER_SET]["Value"].type() != Json::stringValue ||
-              !GetDicomEncoding(encoding, siblingTags[SPECIFIC_CHARACTER_SET]["Value"].asCString()))
+          if (!siblingTags[SPECIFIC_CHARACTER_SET].isMember(VALUE) ||
+              siblingTags[SPECIFIC_CHARACTER_SET][VALUE].type() != Json::stringValue ||
+              !GetDicomEncoding(encoding, siblingTags[SPECIFIC_CHARACTER_SET][VALUE].asCString()))
           {
             LOG(WARNING) << "Instance with an incorrect Specific Character Set, "
                          << "using the default Orthanc encoding: " << siblingInstanceId;
@@ -593,13 +712,13 @@ namespace Orthanc
         if (siblingTags.isMember(t))
         {
           const Json::Value& tag = siblingTags[t];
-          if (tag["Type"] == "Null")
+          if (tag[TYPE] == "Null")
           {
             dicom.ReplacePlainString(*it, "");
           }
-          else if (tag["Type"] == "String")
+          else if (tag[TYPE] == "String")
           {
-            std::string value = tag["Value"].asString();  // This is an UTF-8 value (as it comes from JSON)
+            std::string value = tag[VALUE].asString();  // This is an UTF-8 value (as it comes from JSON)
             dicom.ReplacePlainString(*it, value);
           }
         }
@@ -608,9 +727,9 @@ namespace Orthanc
 
 
     bool decodeBinaryTags = true;
-    if (request.isMember("InterpretBinaryTags"))
+    if (request.isMember(INTERPRET_BINARY_TAGS))
     {
-      const Json::Value& v = request["InterpretBinaryTags"];
+      const Json::Value& v = request[INTERPRET_BINARY_TAGS];
       if (v.type() != Json::booleanValue)
       {
         throw OrthancException(ErrorCode_BadRequest);
@@ -622,9 +741,9 @@ namespace Orthanc
 
     // New argument in Orthanc 1.6.0
     std::string privateCreator;
-    if (request.isMember("PrivateCreator"))
+    if (request.isMember(PRIVATE_CREATOR))
     {
-      const Json::Value& v = request["PrivateCreator"];
+      const Json::Value& v = request[PRIVATE_CREATOR];
       if (v.type() != Json::stringValue)
       {
         throw OrthancException(ErrorCode_BadRequest);
@@ -638,7 +757,21 @@ namespace Orthanc
       privateCreator = lock.GetConfiguration().GetDefaultPrivateCreator();
     }
 
-    
+
+    // New in Orthanc 1.9.0
+    bool force = false;
+    if (request.isMember(FORCE))
+    {
+      const Json::Value& v = request[FORCE];
+      if (v.type() != Json::booleanValue)
+      {
+        throw OrthancException(ErrorCode_BadRequest);
+      }
+
+      force = v.asBool();
+    }
+
+
     // Inject time-related information
     std::string date, time;
     SystemToolbox::GetNowDicom(date, time, true /* use UTC time (not local time) */);
@@ -665,17 +798,17 @@ namespace Orthanc
     }
 
 
-    InjectTags(dicom, request["Tags"], decodeBinaryTags, privateCreator);
+    InjectTags(dicom, request[TAGS], decodeBinaryTags, privateCreator, force);
 
 
     // Inject the content (either an image, or a PDF file)
-    if (request.isMember("Content"))
+    if (request.isMember(CONTENT))
     {
-      const Json::Value& content = request["Content"];
+      const Json::Value& content = request[CONTENT];
 
       if (content.type() == Json::stringValue)
       {
-        dicom.EmbedContent(request["Content"].asString());
+        dicom.EmbedContent(request[CONTENT].asString());
 
       }
       else if (content.type() == Json::arrayValue)
@@ -683,7 +816,7 @@ namespace Orthanc
         if (content.size() > 0)
         {
           // Let's create a series instead of a single instance
-          CreateSeries(call, dicom, content, decodeBinaryTags, privateCreator);
+          CreateSeries(call, dicom, content, decodeBinaryTags, privateCreator, force);
           return;
         }
       }
@@ -700,6 +833,39 @@ namespace Orthanc
 
   static void CreateDicom(RestApiPostCall& call)
   {
+    if (call.IsDocumentation())
+    {
+      call.GetDocumentation()
+        .SetTag("System")
+        .SetSummary("Create one DICOM instance")
+        .SetDescription("Create one DICOM instance, and store it into Orthanc")
+        .SetRequestField("Tags", RestApiCallDocumentation::Type_JsonObject,
+                         "Associative array containing the tags of the new instance to be created", true)
+        .SetRequestField("Content", RestApiCallDocumentation::Type_String,
+                         "This field can be used to embed an image (pixel data) or a PDF inside the created DICOM instance. "
+                         "The PNG image, the JPEG image or the PDF file must be provided using their "
+                         "[data URI scheme encoding](https://en.wikipedia.org/wiki/Data_URI_scheme). "
+                         "This field can possibly contain a JSON array, in which case a DICOM series is created "
+                         "containing one DICOM instance for each item in the `Content` field.", false)
+        .SetRequestField("Parent", RestApiCallDocumentation::Type_String,
+                         "If present, the newly created instance will be attached to the parent DICOM resource "
+                         "whose Orthanc identifier is contained in this field. The DICOM tags of the parent "
+                         "modules in the DICOM hierarchy will be automatically copied to the newly created instance.", false)
+        .SetRequestField("InterpretBinaryTags", RestApiCallDocumentation::Type_Boolean,
+                         "If some value in the `Tags` associative array is formatted according to some "
+                         "[data URI scheme encoding](https://en.wikipedia.org/wiki/Data_URI_scheme), "
+                         "whether this value is decoded to a binary value or kept as such (`true` by default)", false)
+        .SetRequestField("PrivateCreator", RestApiCallDocumentation::Type_String,
+                         "The private creator to be used for private tags in `Tags`", false)
+        .SetRequestField("Force", RestApiCallDocumentation::Type_Boolean,
+                         "Avoid the consistency checks for the DICOM tags that enforce the DICOM model of the real-world. "
+                         "You can notably use this flag if you need to manually set the tags `StudyInstanceUID`, "
+                         "`SeriesInstanceUID`, or `SOPInstanceUID`. Be careful with this feature.", false)
+        .SetAnswerField("ID", RestApiCallDocumentation::Type_String, "Orthanc identifier of the newly created instance")
+        .SetAnswerField("Path", RestApiCallDocumentation::Type_String, "Path to access the instance in the REST API");
+      return;
+    }
+
     Json::Value request;
     if (!call.ParseJsonRequest(request) ||
         !request.isObject())
@@ -725,6 +891,32 @@ namespace Orthanc
 
   static void SplitStudy(RestApiPostCall& call)
   {
+    if (call.IsDocumentation())
+    {
+      OrthancRestApi::DocumentSubmitCommandsJob(call);
+      call.GetDocumentation()
+        .SetTag("Studies")
+        .SetSummary("Split study")
+        .SetDescription("Start a new job so as to split the DICOM study whose Orthanc identifier is provided in the URL, "
+                        "by taking some of its children series out of it and putting them into a brand new study (this "
+                        "new study is created by setting the `StudyInstanceUID` tag to a random identifier): "
+                        "https://book.orthanc-server.com/users/anonymization.html#splitting")
+        .SetUriArgument("id", "Orthanc identifier of the study of interest")
+        .SetRequestField("Series", RestApiCallDocumentation::Type_JsonListOfStrings,
+                         "The list of series to be separated from the parent study (mandatory option). "
+                         "These series must all be children of the same source study, that is specified in the URI.", true)
+        .SetRequestField("Replace", RestApiCallDocumentation::Type_JsonObject,
+                         "Associative array to change the value of some DICOM tags in the new study. "
+                         "These tags must be part of the \"Patient Module Attributes\" or the \"General Study "
+                         "Module Attributes\", as specified by the DICOM 2011 standard in Tables C.7-1 and C.7-3.", false)
+        .SetRequestField("Remove", RestApiCallDocumentation::Type_JsonListOfStrings,
+                         "List of tags that must be removed in the new study (from the same modules as in the `Replace` option)", false)
+        .SetRequestField("KeepSource", RestApiCallDocumentation::Type_Boolean,
+                         "If set to `true`, instructs Orthanc to keep a copy of the original series in the source study. "
+                         "By default, the original series are deleted from Orthanc.", false);
+      return;
+    }
+    
     ServerContext& context = OrthancRestApi::GetContext(call);
 
     Json::Value request;
@@ -804,6 +996,24 @@ namespace Orthanc
 
   static void MergeStudy(RestApiPostCall& call)
   {
+    if (call.IsDocumentation())
+    {
+      OrthancRestApi::DocumentSubmitCommandsJob(call);
+      call.GetDocumentation()
+        .SetTag("Studies")
+        .SetSummary("Merge study")
+        .SetDescription("Start a new job so as to move some DICOM series into the DICOM study whose Orthanc identifier "
+                        "is provided in the URL: https://book.orthanc-server.com/users/anonymization.html#merging")
+        .SetUriArgument("id", "Orthanc identifier of the study of interest")
+        .SetRequestField("Resources", RestApiCallDocumentation::Type_JsonListOfStrings,
+                         "The list of DICOM resources (patients, studies, series, and/or instances) to be merged "
+                         "into the study of interest (mandatory option)", true)
+        .SetRequestField("KeepSource", RestApiCallDocumentation::Type_Boolean,
+                         "If set to `true`, instructs Orthanc to keep a copy of the original resources in their source study. "
+                         "By default, the original resources are deleted from Orthanc.", false);
+      return;
+    }
+
     ServerContext& context = OrthancRestApi::GetContext(call);
 
     Json::Value request;
