@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -115,6 +115,13 @@ namespace Orthanc
   }
 
 
+  void QueryRetrieveHandler::SetLocalAet(const std::string& localAet)
+  {
+    Invalidate();
+    localAet_ = localAet;
+  }
+
+
   void QueryRetrieveHandler::SetLevel(ResourceType level)
   {
     Invalidate();
