@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,10 +34,12 @@
 #include "PrecompiledHeadersServer.h"
 #include "ServerContext.h"
 
-#include "../../OrthancFramework/Sources/DicomParsing/Internals/DicomImageDecoder.h"
 #include "../../OrthancFramework/Sources/Cache/SharedArchive.h"
+#include "../../OrthancFramework/Sources/DicomFormat/DicomElement.h"
 #include "../../OrthancFramework/Sources/DicomParsing/DcmtkTranscoder.h"
+#include "../../OrthancFramework/Sources/DicomParsing/DicomModification.h"
 #include "../../OrthancFramework/Sources/DicomParsing/FromDcmtkBridge.h"
+#include "../../OrthancFramework/Sources/DicomParsing/Internals/DicomImageDecoder.h"
 #include "../../OrthancFramework/Sources/FileStorage/StorageAccessor.h"
 #include "../../OrthancFramework/Sources/HttpServer/FilesystemHttpSender.h"
 #include "../../OrthancFramework/Sources/HttpServer/HttpStreamTranscoder.h"
@@ -56,10 +58,8 @@
 #include <dcmtk/dcmdata/dcfilefo.h>
 
 
+static size_t DICOM_CACHE_SIZE = 128 * 1024 * 1024;  // 128 MB
 
-#define ENABLE_DICOM_CACHE  1
-
-static const size_t DICOM_CACHE_SIZE = 2;
 
 /**
  * IMPORTANT: We make the assumption that the same instance of
@@ -72,6 +72,34 @@ static const size_t DICOM_CACHE_SIZE = 2;
 
 namespace Orthanc
 {
+  static bool IsUncompressedTransferSyntax(DicomTransferSyntax transferSyntax)
+  {
+    return (transferSyntax == DicomTransferSyntax_LittleEndianImplicit ||
+            transferSyntax == DicomTransferSyntax_LittleEndianExplicit ||
+            transferSyntax == DicomTransferSyntax_BigEndianExplicit);
+  }
+
+
+  static bool IsTranscodableTransferSyntax(DicomTransferSyntax transferSyntax)
+  {
+    return (
+      // Do not try to transcode DICOM videos (new in Orthanc 1.8.2)
+      transferSyntax != DicomTransferSyntax_MPEG2MainProfileAtMainLevel &&
+      transferSyntax != DicomTransferSyntax_MPEG2MainProfileAtHighLevel &&
+      transferSyntax != DicomTransferSyntax_MPEG4HighProfileLevel4_1 &&
+      transferSyntax != DicomTransferSyntax_MPEG4BDcompatibleHighProfileLevel4_1 &&
+      transferSyntax != DicomTransferSyntax_MPEG4HighProfileLevel4_2_For2DVideo &&
+      transferSyntax != DicomTransferSyntax_MPEG4HighProfileLevel4_2_For3DVideo &&
+      transferSyntax != DicomTransferSyntax_MPEG4StereoHighProfileLevel4_2 &&
+      transferSyntax != DicomTransferSyntax_HEVCMainProfileLevel5_1 &&
+      transferSyntax != DicomTransferSyntax_HEVCMain10ProfileLevel5_1 &&
+
+      // Do not try to transcode special transfer syntaxes
+      transferSyntax != DicomTransferSyntax_RFC2557MimeEncapsulation &&
+      transferSyntax != DicomTransferSyntax_XML);
+  }
+
+  
   void ServerContext::ChangeThread(ServerContext* that,
                                    unsigned int sleepDelay)
   {
@@ -189,10 +217,10 @@ namespace Orthanc
       if (index_.LookupGlobalProperty(serialized, GlobalProperty_JobsRegistry))
       {
         LOG(WARNING) << "Reloading the jobs from the last execution of Orthanc";
-        OrthancJobUnserializer unserializer(*this);
 
         try
         {
+          OrthancJobUnserializer unserializer(*this);
           jobsEngine_.LoadRegistryFromString(unserializer, serialized);
         }
         catch (OrthancException& e)
@@ -222,15 +250,15 @@ namespace Orthanc
   {
     if (saveJobs_)
     {
-      VLOG(1) << "Serializing the content of the jobs engine";
+      LOG(TRACE) << "Serializing the content of the jobs engine";
     
       try
       {
         Json::Value value;
         jobsEngine_.GetRegistry().Serialize(value);
 
-        Json::FastWriter writer;
-        std::string serialized = writer.write(value);
+        std::string serialized;
+        Toolbox::WriteFastJson(serialized, value);
 
         index_.SetGlobalProperty(GlobalProperty_JobsRegistry, serialized);
       }
@@ -242,6 +270,15 @@ namespace Orthanc
   }
 
 
+  void ServerContext::PublishDicomCacheMetrics()
+  {
+    metricsRegistry_->SetValue("orthanc_dicom_cache_size",
+                               static_cast<float>(dicomCache_.GetCurrentSize()) / static_cast<float>(1024 * 1024));
+    metricsRegistry_->SetValue("orthanc_dicom_cache_count",
+                               static_cast<float>(dicomCache_.GetNumberOfItems()));
+  }
+
+
   ServerContext::ServerContext(IDatabaseWrapper& database,
                                IStorageArea& area,
                                bool unitTesting,
@@ -250,8 +287,8 @@ namespace Orthanc
     area_(area),
     compressionEnabled_(false),
     storeMD5_(true),
-    provider_(*this),
-    dicomCache_(provider_, DICOM_CACHE_SIZE),
+    largeDicomThrottler_(1),
+    dicomCache_(DICOM_CACHE_SIZE),
     mainLua_(*this),
     filterLua_(*this),
     luaListener_(*this),
@@ -267,7 +304,11 @@ namespace Orthanc
     isExecuteLuaEnabled_(false),
     overwriteInstances_(false),
     dcmtkTranscoder_(new DcmtkTranscoder),
-    isIngestTranscoding_(false)
+    isIngestTranscoding_(false),
+    ingestTranscodingOfUncompressed_(true),
+    ingestTranscodingOfCompressed_(true),
+    preferredTransferSyntax_(DicomTransferSyntax_LittleEndianExplicit),
+    deidentifyLogs_(false)
   {
     try
     {
@@ -306,6 +347,18 @@ namespace Orthanc
             isIngestTranscoding_ = true;
             LOG(WARNING) << "Incoming DICOM instances will automatically be transcoded to "
                          << "transfer syntax: " << GetTransferSyntaxUid(ingestTransferSyntax_);
+
+            // New options in Orthanc 1.8.2
+            ingestTranscodingOfUncompressed_ = lock.GetConfiguration().GetBooleanParameter("IngestTranscodingOfUncompressed", true);
+            ingestTranscodingOfCompressed_ = lock.GetConfiguration().GetBooleanParameter("IngestTranscodingOfCompressed", true);
+
+            LOG(WARNING) << "  Ingest transcoding will "
+                         << (ingestTranscodingOfUncompressed_ ? "be applied" : "*not* be applied")
+                         << " to uncompressed transfer syntaxes (Little Endian Implicit/Explicit, Big Endian Explicit)";
+
+            LOG(WARNING) << "  Ingest transcoding will "
+                         << (ingestTranscodingOfCompressed_ ? "be applied" : "*not* be applied")
+                         << " to compressed transfer syntaxes";
           }
           else
           {
@@ -318,6 +371,40 @@ namespace Orthanc
           isIngestTranscoding_ = false;
           LOG(INFO) << "Automated transcoding of incoming DICOM instances is disabled";
         }
+
+        // New options in Orthanc 1.8.2
+        if (lock.GetConfiguration().GetBooleanParameter("DeidentifyLogs", true))
+        {
+          deidentifyLogs_ = true;
+          CLOG(INFO, DICOM) << "Deidentification of log contents (notably for DIMSE queries) is enabled";
+
+          DicomVersion version = StringToDicomVersion(
+              lock.GetConfiguration().GetStringParameter("DeidentifyLogsDicomVersion", "2017c"));
+          CLOG(INFO, DICOM) << "Version of DICOM standard used for deidentification is "
+                            << EnumerationToString(version);
+
+          logsDeidentifierRules_.SetupAnonymization(version);
+        }
+        else
+        {
+          deidentifyLogs_ = false;
+          CLOG(INFO, DICOM) << "Deidentification of log contents (notably for DIMSE queries) is disabled";
+        }
+
+        // New options in Orthanc 1.9.0
+        if (lock.GetConfiguration().LookupStringParameter(s, "DicomScuPreferredTransferSyntax") &&
+            !LookupTransferSyntax(preferredTransferSyntax_, s))
+        {
+          throw OrthancException(ErrorCode_ParameterOutOfRange,
+                                 "Unknown preferred transfer syntax: " + s);
+        }
+        
+        CLOG(INFO, DICOM) << "Preferred transfer syntax for Orthanc C-STORE SCU: "
+                          << GetTransferSyntaxUid(preferredTransferSyntax_);
+
+        lock.GetConfiguration().GetAcceptedTransferSyntaxes(acceptedTransferSyntaxes_);
+
+        isUnknownSopClassAccepted_ = lock.GetConfiguration().GetBooleanParameter("UnknownSopClassAccepted", false);
       }
 
       jobsEngine_.SetThreadSleep(unitTesting ? 20 : 200);
@@ -466,12 +553,10 @@ namespace Orthanc
         return StoreStatus_FilteredOut;
       }
 
-      {
-        // Remove the file from the DicomCache (useful if
-        // "OverwriteInstances" is set to "true")
-        boost::mutex::scoped_lock lock(dicomCacheMutex_);
-        dicomCache_.Invalidate(resultPublicId);
-      }
+      // Remove the file from the DicomCache (useful if
+      // "OverwriteInstances" is set to "true")
+      dicomCache_.Invalidate(resultPublicId);
+      PublishDicomCacheMetrics();
 
       // TODO Should we use "gzip" instead?
       CompressionType compression = (compressionEnabled_ ? CompressionType_ZlibWithSize : CompressionType_None);
@@ -570,24 +655,50 @@ namespace Orthanc
     }
     else
     {
-      // Automated transcoding of incoming DICOM files
+      // Automated transcoding of incoming DICOM instance
+
+      bool transcode = false;
 
       DicomTransferSyntax sourceSyntax;
       if (!FromDcmtkBridge::LookupOrthancTransferSyntax(
             sourceSyntax, dicom.GetParsedDicomFile().GetDcmtkObject()) ||
           sourceSyntax == ingestTransferSyntax_)
+      {
+        // Don't transcode if the incoming DICOM is already in the proper transfer syntax
+        transcode = false;
+      }
+      else if (!IsTranscodableTransferSyntax(sourceSyntax))
+      {
+        // Don't try to transcode video files, this is useless (new in
+        // Orthanc 1.8.2). This could be accepted in the future if
+        // video transcoding gets implemented.
+        transcode = false;
+      }
+      else if (IsUncompressedTransferSyntax(sourceSyntax))
+      {
+        // This is an uncompressed transfer syntax (new in Orthanc 1.8.2)
+        transcode = ingestTranscodingOfUncompressed_;
+      }
+      else
+      {
+        // This is an compressed transfer syntax (new in Orthanc 1.8.2)
+        transcode = ingestTranscodingOfCompressed_;
+      }
+
+      if (!transcode)
       {
         // No transcoding
         return StoreAfterTranscoding(resultPublicId, dicom, mode);
       }
       else
-      {      
+      {
+        // Trancoding
         std::set<DicomTransferSyntax> syntaxes;
         syntaxes.insert(ingestTransferSyntax_);
-
+        
         IDicomTranscoder::DicomImage source;
         source.SetExternalBuffer(dicom.GetBufferData(), dicom.GetBufferSize());
-
+        
         IDicomTranscoder::DicomImage transcoded;
         if (Transcode(transcoded, source, syntaxes, true /* allow new SOP instance UID */))
         {
@@ -734,8 +845,7 @@ namespace Orthanc
       std::string tmp;
       ReadDicomAsJsonInternal(tmp, instancePublicId);
 
-      Json::Reader reader;
-      if (!reader.parse(tmp, result))
+      if (!Toolbox::ReadJson(result, tmp))
       {
         throw OrthancException(ErrorCode_CorruptedFile);
       }
@@ -791,34 +901,69 @@ namespace Orthanc
   }
 
 
-  IDynamicObject* ServerContext::DicomCacheProvider::Provide(const std::string& instancePublicId)
+  ServerContext::DicomCacheLocker::DicomCacheLocker(ServerContext& context,
+                                                    const std::string& instancePublicId) :
+    context_(context),
+    instancePublicId_(instancePublicId)
   {
-    std::string content;
-    context_.ReadDicom(content, instancePublicId);
-    return new ParsedDicomFile(content);
-  }
+    accessor_.reset(new ParsedDicomCache::Accessor(context_.dicomCache_, instancePublicId));
+    
+    if (!accessor_->IsValid())
+    {
+      accessor_.reset(NULL);
 
+      // Throttle to avoid loading several large DICOM files simultaneously
+      largeDicomLocker_.reset(new Semaphore::Locker(context.largeDicomThrottler_));
+      
+      std::string content;
+      context_.ReadDicom(content, instancePublicId);
 
-  ServerContext::DicomCacheLocker::DicomCacheLocker(ServerContext& that,
-                                                    const std::string& instancePublicId) : 
-    that_(that),
-    lock_(that_.dicomCacheMutex_)
-  {
-#if ENABLE_DICOM_CACHE == 0
-    static std::unique_ptr<IDynamicObject> p;
-    p.reset(that_.provider_.Provide(instancePublicId));
-    dicom_ = dynamic_cast<ParsedDicomFile*>(p.get());
-#else
-    dicom_ = &dynamic_cast<ParsedDicomFile&>(that_.dicomCache_.Access(instancePublicId));
-#endif
+      // Release the throttle if loading "small" DICOM files (under
+      // 50MB, which is an arbitrary value)
+      if (content.size() < 50 * 1024 * 1024)
+      {
+        largeDicomLocker_.reset(NULL);
+      }
+      
+      dicom_.reset(new ParsedDicomFile(content));
+      dicomSize_ = content.size();
+    }
+
+    assert(accessor_.get() != NULL ||
+           dicom_.get() != NULL);
   }
 
 
   ServerContext::DicomCacheLocker::~DicomCacheLocker()
   {
+    if (dicom_.get() != NULL)
+    {
+      try
+      {
+        context_.dicomCache_.Acquire(instancePublicId_, dicom_.release(), dicomSize_);
+        context_.PublishDicomCacheMetrics();
+      }
+      catch (OrthancException&)
+      {
+      }
+    }
   }
 
 
+  ParsedDicomFile& ServerContext::DicomCacheLocker::GetDicom() const
+  {
+    if (dicom_.get() != NULL)
+    {
+      return *dicom_;
+    }
+    else
+    {
+      assert(accessor_.get() != NULL);
+      return accessor_->GetDicom();
+    }
+  }
+
+  
   void ServerContext::SetStoreMD5ForAttachments(bool storeMD5)
   {
     LOG(INFO) << "Storing MD5 for attachments: " << (storeMD5 ? "yes" : "no");
@@ -859,8 +1004,8 @@ namespace Orthanc
     if (expectedType == ResourceType_Instance)
     {
       // remove the file from the DicomCache
-      boost::mutex::scoped_lock lock(dicomCacheMutex_);
       dicomCache_.Invalidate(uuid);
+      PublishDicomCacheMetrics();
     }
 
     return index_.DeleteResource(target, uuid, expectedType);
@@ -869,6 +1014,13 @@ namespace Orthanc
 
   void ServerContext::SignalChange(const ServerIndexChange& change)
   {
+    if (change.GetResourceType() == ResourceType_Instance &&
+        change.GetChangeType() == ChangeType_Deleted)
+    {
+      dicomCache_.Invalidate(change.GetPublicId());
+      PublishDicomCacheMetrics();
+    }
+    
     pendingChanges_.Enqueue(change.Clone());
   }
 
@@ -936,12 +1088,12 @@ namespace Orthanc
   }
 
 
-  void ServerContext::Apply(ILookupVisitor& visitor,
-                            const DatabaseLookup& lookup,
-                            ResourceType queryLevel,
-                            size_t since,
-                            size_t limit)
-  {
+  void ServerContext::ApplyInternal(ILookupVisitor& visitor,
+                                    const DatabaseLookup& lookup,
+                                    ResourceType queryLevel,
+                                    size_t since,
+                                    size_t limit)
+  {    
     unsigned int databaseLimit = (queryLevel == ResourceType_Instance ?
                                   limitFindInstances_ : limitFindResults_);
       
@@ -994,7 +1146,6 @@ namespace Orthanc
           continue;
         }
 
-#if 1
         // New in Orthanc 1.6.0: Only keep the main DICOM tags at the
         // level of interest for the query
         switch (queryLevel)
@@ -1016,17 +1167,6 @@ namespace Orthanc
           default:
             throw OrthancException(ErrorCode_InternalError);
         }
-
-        // Special case of the "Modality" at the study level, in order
-        // to deal with C-FIND on "ModalitiesInStudy" (0008,0061).
-        // Check out integration test "test_rest_modalities_in_study".
-        if (queryLevel == ResourceType_Study)
-        {
-          dicom.CopyTagIfExists(tmp, DICOM_TAG_MODALITY);
-        }
-#else
-        dicom.Assign(tmp);  // This emulates Orthanc <= 1.5.8
-#endif
         
         hasOnlyMainDicomTags = true;
       }
@@ -1096,8 +1236,230 @@ namespace Orthanc
   }
 
 
+
+  namespace
+  {
+    class ModalitiesInStudyVisitor : public ServerContext::ILookupVisitor
+    {
+    private:
+      class Study : public boost::noncopyable
+      {
+      private:
+        std::string            orthancId_;
+        std::string            instanceId_;
+        DicomMap               mainDicomTags_;
+        Json::Value            dicomAsJson_;
+        std::set<std::string>  modalitiesInStudy_;
+
+      public:
+        Study(const std::string& instanceId,
+              const DicomMap& seriesTags) :
+          instanceId_(instanceId),
+          dicomAsJson_(Json::nullValue)
+        {
+          {
+            DicomMap tmp;
+            tmp.Assign(seriesTags);
+            tmp.SetValue(DICOM_TAG_SOP_INSTANCE_UID, "dummy", false);
+            DicomInstanceHasher hasher(tmp);
+            orthancId_ = hasher.HashStudy();
+          }
+          
+          mainDicomTags_.MergeMainDicomTags(seriesTags, ResourceType_Study);
+          mainDicomTags_.MergeMainDicomTags(seriesTags, ResourceType_Patient);
+          AddModality(seriesTags);
+        }
+
+        void AddModality(const DicomMap& seriesTags)
+        {
+          std::string modality;
+          if (seriesTags.LookupStringValue(modality, DICOM_TAG_MODALITY, false) &&
+              !modality.empty())
+          {
+            modalitiesInStudy_.insert(modality);
+          }
+        }
+
+        void SetDicomAsJson(const Json::Value& dicomAsJson)
+        {
+          dicomAsJson_ = dicomAsJson;
+        }
+
+        const std::string& GetOrthancId() const
+        {
+          return orthancId_;
+        }
+
+        const std::string& GetInstanceId() const
+        {
+          return instanceId_;
+        }
+
+        const DicomMap& GetMainDicomTags() const
+        {
+          return mainDicomTags_;
+        }
+
+        const Json::Value* GetDicomAsJson() const
+        {
+          if (dicomAsJson_.type() == Json::nullValue)
+          {
+            return NULL;
+          }
+          else
+          {
+            return &dicomAsJson_;
+          }
+        } 
+      };
+      
+      typedef std::map<std::string, Study*>  Studies;
+      
+      bool     isDicomAsJsonNeeded_;
+      bool     complete_;
+      Studies  studies_;
+      
+    public:
+      explicit ModalitiesInStudyVisitor(bool isDicomAsJsonNeeded) :
+        isDicomAsJsonNeeded_(isDicomAsJsonNeeded),
+        complete_(false)
+      {
+      }
+
+      ~ModalitiesInStudyVisitor()
+      {
+        for (Studies::const_iterator it = studies_.begin(); it != studies_.end(); ++it)
+        {
+          assert(it->second != NULL);
+          delete it->second;
+        }
+
+        studies_.clear();
+      }
+      
+      virtual bool IsDicomAsJsonNeeded() const ORTHANC_OVERRIDE
+      {
+        return isDicomAsJsonNeeded_;
+      }
+      
+      virtual void MarkAsComplete() ORTHANC_OVERRIDE
+      {
+        complete_ = true;
+      }
+      
+      virtual void Visit(const std::string& publicId,
+                         const std::string& instanceId,
+                         const DicomMap& seriesTags,
+                         const Json::Value* dicomAsJson) ORTHANC_OVERRIDE
+      {
+        std::string studyInstanceUid;
+        if (seriesTags.LookupStringValue(studyInstanceUid, DICOM_TAG_STUDY_INSTANCE_UID, false))
+        {
+          Studies::iterator found = studies_.find(studyInstanceUid);
+          if (found == studies_.end())
+          {
+            // New study
+            std::unique_ptr<Study> study(new Study(instanceId, seriesTags));
+            
+            if (dicomAsJson != NULL)
+            {
+              study->SetDicomAsJson(*dicomAsJson);
+            }
+            
+            studies_[studyInstanceUid] = study.release();
+          }
+          else
+          {
+            // Already existing study
+            found->second->AddModality(seriesTags);
+          }
+        }
+      }
+
+      void Forward(ILookupVisitor& callerVisitor,
+                   size_t since,
+                   size_t limit) const
+      {
+        size_t index = 0;
+        size_t countForwarded = 0;
+        
+        for (Studies::const_iterator it = studies_.begin(); it != studies_.end(); ++it, index++)
+        {
+          if (limit == 0 ||
+              (index >= since &&
+               index < limit))
+          {
+            assert(it->second != NULL);
+            const Study& study = *it->second;
+
+            countForwarded++;
+            callerVisitor.Visit(study.GetOrthancId(), study.GetInstanceId(),
+                                study.GetMainDicomTags(), study.GetDicomAsJson());
+          }
+        }
+
+        if (countForwarded == studies_.size())
+        {
+          callerVisitor.MarkAsComplete();
+        }
+      }
+    };
+  }
+  
+
+  void ServerContext::Apply(ILookupVisitor& visitor,
+                            const DatabaseLookup& lookup,
+                            ResourceType queryLevel,
+                            size_t since,
+                            size_t limit)
+  {
+    if (queryLevel == ResourceType_Study &&
+        lookup.HasTag(DICOM_TAG_MODALITIES_IN_STUDY))
+    {
+      // Convert the study-level query, into a series-level query,
+      // where "ModalitiesInStudy" is replaced by "Modality"
+      DatabaseLookup seriesLookup;
+
+      for (size_t i = 0; i < lookup.GetConstraintsCount(); i++)
+      {
+        const DicomTagConstraint& constraint = lookup.GetConstraint(i);
+        if (constraint.GetTag() == DICOM_TAG_MODALITIES_IN_STUDY)
+        {
+          if ((constraint.GetConstraintType() == ConstraintType_Equal && constraint.GetValue().empty()) ||
+              (constraint.GetConstraintType() == ConstraintType_List && constraint.GetValues().empty()))
+          {
+            // Ignore universal lookup on "ModalitiesInStudy" (0008,0061),
+            // this should have been handled by the caller
+            ApplyInternal(visitor, lookup, queryLevel, since, limit);
+            return;
+          }
+          else
+          {
+            DicomTagConstraint modality(constraint);
+            modality.SetTag(DICOM_TAG_MODALITY);
+            seriesLookup.AddConstraint(modality);
+          }
+        }
+        else
+        {
+          seriesLookup.AddConstraint(constraint);
+        }
+      }
+
+      ModalitiesInStudyVisitor seriesVisitor(visitor.IsDicomAsJsonNeeded());
+      ApplyInternal(seriesVisitor, seriesLookup, ResourceType_Series, 0, 0);
+      seriesVisitor.Forward(visitor, since, limit);
+    }
+    else
+    {
+      ApplyInternal(visitor, lookup, queryLevel, since, limit);
+    }
+  }
+  
+
   bool ServerContext::LookupOrReconstructMetadata(std::string& target,
                                                   const std::string& publicId,
+                                                  ResourceType level,
                                                   MetadataType metadata)
   {
     // This is a backwards-compatibility function, that can
@@ -1107,7 +1469,7 @@ namespace Orthanc
     if (metadata == MetadataType_Instance_SopClassUid ||
         metadata == MetadataType_Instance_TransferSyntax)
     {
-      if (index_.LookupMetadata(target, publicId, metadata))
+      if (index_.LookupMetadata(target, publicId, level, metadata))
       {
         return true;
       }
@@ -1162,7 +1524,7 @@ namespace Orthanc
     else
     {
       // No backward
-      return index_.LookupMetadata(target, publicId, metadata);
+      return index_.LookupMetadata(target, publicId, level, metadata);
     }
   }
 
@@ -1232,9 +1594,17 @@ namespace Orthanc
     {
       // Use Orthanc's built-in decoder, using the cache to speed-up
       // things on multi-frame images
-      ServerContext::DicomCacheLocker locker(*this, publicId);        
-      std::unique_ptr<ImageAccessor> decoded(
-        DicomImageDecoder::Decode(locker.GetDicom(), frameIndex));
+
+      std::unique_ptr<ImageAccessor> decoded;
+      try
+      {
+        ServerContext::DicomCacheLocker locker(*this, publicId);
+        decoded.reset(locker.GetDicom().DecodeFrame(frameIndex));
+      }
+      catch (OrthancException& e)
+      {
+      }
+      
       if (decoded.get() != NULL)
       {
         return decoded.release();
@@ -1248,8 +1618,16 @@ namespace Orthanc
       // TODO: Store the raw buffer in the DicomCacheLocker
       std::string dicomContent;
       ReadDicom(dicomContent, publicId);
-      std::unique_ptr<ImageAccessor> decoded(
-        GetPlugins().Decode(dicomContent.c_str(), dicomContent.size(), frameIndex));
+      
+      std::unique_ptr<ImageAccessor> decoded;
+      try
+      {
+        decoded.reset(GetPlugins().Decode(dicomContent.c_str(), dicomContent.size(), frameIndex));
+      }
+      catch (OrthancException& e)
+      {
+      }
+      
       if (decoded.get() != NULL)
       {
         return decoded.release();
@@ -1265,7 +1643,7 @@ namespace Orthanc
     if (builtinDecoderTranscoderOrder_ == BuiltinDecoderTranscoderOrder_After)
     {
       ServerContext::DicomCacheLocker locker(*this, publicId);        
-      return DicomImageDecoder::Decode(locker.GetDicom(), frameIndex);
+      return locker.GetDicom().DecodeFrame(frameIndex);
     }
     else
     {
@@ -1279,8 +1657,15 @@ namespace Orthanc
   {
     if (builtinDecoderTranscoderOrder_ == BuiltinDecoderTranscoderOrder_Before)
     {
-      std::unique_ptr<ImageAccessor> decoded(
-        DicomImageDecoder::Decode(dicom.GetParsedDicomFile(), frameIndex));
+      std::unique_ptr<ImageAccessor> decoded;
+      try
+      {
+        decoded.reset(dicom.GetParsedDicomFile().DecodeFrame(frameIndex));
+      }
+      catch (OrthancException& e)
+      {
+      }
+        
       if (decoded.get() != NULL)
       {
         return decoded.release();
@@ -1291,8 +1676,15 @@ namespace Orthanc
     if (HasPlugins() &&
         GetPlugins().HasCustomImageDecoder())
     {
-      std::unique_ptr<ImageAccessor> decoded(
-        GetPlugins().Decode(dicom.GetBufferData(), dicom.GetBufferSize(), frameIndex));
+      std::unique_ptr<ImageAccessor> decoded;
+      try
+      {
+        decoded.reset(GetPlugins().Decode(dicom.GetBufferData(), dicom.GetBufferSize(), frameIndex));
+      }
+      catch (OrthancException& e)
+      {
+      }
+    
       if (decoded.get() != NULL)
       {
         return decoded.release();
@@ -1307,7 +1699,7 @@ namespace Orthanc
 
     if (builtinDecoderTranscoderOrder_ == BuiltinDecoderTranscoderOrder_After)
     {
-      return DicomImageDecoder::Decode(dicom.GetParsedDicomFile(), frameIndex);
+      return dicom.GetParsedDicomFile().DecodeFrame(frameIndex);
     }
     else
     {
@@ -1344,7 +1736,7 @@ namespace Orthanc
     }
     else
     {
-      connection.Transcode(sopClassUid, sopInstanceUid, *this, data, dicom.size(),
+      connection.Transcode(sopClassUid, sopInstanceUid, *this, data, dicom.size(), preferredTransferSyntax_,
                            hasMoveOriginator, moveOriginatorAet, moveOriginatorId);
     }
   }
@@ -1388,4 +1780,50 @@ namespace Orthanc
       return false;
     }
   }
+
+  const std::string& ServerContext::GetDeidentifiedContent(const DicomElement &element) const
+  {
+    static const std::string redactedContent = "*** POTENTIAL PHI ***";
+
+    const DicomTag& tag = element.GetTag();
+    if (deidentifyLogs_ && (
+          logsDeidentifierRules_.IsCleared(tag) ||
+          logsDeidentifierRules_.IsRemoved(tag) ||
+          logsDeidentifierRules_.IsReplaced(tag)))
+    {
+      return redactedContent;
+    }
+    else
+    {
+      return element.GetValue().GetContent();
+    }
+  }
+
+
+  void ServerContext::GetAcceptedTransferSyntaxes(std::set<DicomTransferSyntax>& syntaxes)
+  {
+    boost::mutex::scoped_lock lock(dynamicOptionsMutex_);
+    syntaxes = acceptedTransferSyntaxes_;
+  }
+  
+
+  void ServerContext::SetAcceptedTransferSyntaxes(const std::set<DicomTransferSyntax>& syntaxes)
+  {
+    boost::mutex::scoped_lock lock(dynamicOptionsMutex_);
+    acceptedTransferSyntaxes_ = syntaxes;
+  }
+
+
+  bool ServerContext::IsUnknownSopClassAccepted()
+  {
+    boost::mutex::scoped_lock lock(dynamicOptionsMutex_);
+    return isUnknownSopClassAccepted_;
+  }
+
+  
+  void ServerContext::SetUnknownSopClassAccepted(bool accepted)
+  {
+    boost::mutex::scoped_lock lock(dynamicOptionsMutex_);
+    isUnknownSopClassAccepted_ = accepted;
+  }
 }
