@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -39,8 +39,11 @@
 #include "ServerIndex.h"
 #include "ServerJobs/IStorageCommitmentFactory.h"
 
-#include "../../OrthancFramework/Sources/Cache/MemoryCache.h"
+#include "../../OrthancFramework/Sources/DicomFormat/DicomElement.h"
+#include "../../OrthancFramework/Sources/DicomParsing/DicomModification.h"
 #include "../../OrthancFramework/Sources/DicomParsing/IDicomTranscoder.h"
+#include "../../OrthancFramework/Sources/DicomParsing/ParsedDicomCache.h"
+#include "../../OrthancFramework/Sources/MultiThreading/Semaphore.h"
 
 
 namespace Orthanc
@@ -80,6 +83,8 @@ namespace Orthanc
       
       virtual void MarkAsComplete() = 0;
 
+      // NB: "dicomAsJson" must *not* be deleted, and can be NULL if
+      // "!IsDicomAsJsonNeeded()"
       virtual void Visit(const std::string& publicId,
                          const std::string& instanceId,
                          const DicomMap& mainDicomTags,
@@ -94,43 +99,30 @@ namespace Orthanc
       ServerContext& context_;
 
     public:
-      LuaServerListener(ServerContext& context) :
+      explicit LuaServerListener(ServerContext& context) :
         context_(context)
       {
       }
 
       virtual void SignalStoredInstance(const std::string& publicId,
                                         const DicomInstanceToStore& instance,
-                                        const Json::Value& simplifiedTags)
+                                        const Json::Value& simplifiedTags) ORTHANC_OVERRIDE
       {
         context_.mainLua_.SignalStoredInstance(publicId, instance, simplifiedTags);
       }
     
-      virtual void SignalChange(const ServerIndexChange& change)
+      virtual void SignalChange(const ServerIndexChange& change) ORTHANC_OVERRIDE
       {
         context_.mainLua_.SignalChange(change);
       }
 
       virtual bool FilterIncomingInstance(const DicomInstanceToStore& instance,
-                                          const Json::Value& simplified)
+                                          const Json::Value& simplified) ORTHANC_OVERRIDE
       {
         return context_.filterLua_.FilterIncomingInstance(instance, simplified);
       }
     };
     
-    class DicomCacheProvider : public Deprecated::ICachePageProvider  // TODO
-    {
-    private:
-      ServerContext& context_;
-
-    public:
-      DicomCacheProvider(ServerContext& context) : context_(context)
-      {
-      }
-      
-      virtual IDynamicObject* Provide(const std::string& id);
-    };
-
     class ServerListener
     {
     private:
@@ -181,10 +173,9 @@ namespace Orthanc
 
     bool compressionEnabled_;
     bool storeMD5_;
-    
-    DicomCacheProvider provider_;
-    boost::mutex dicomCacheMutex_;
-    Deprecated::MemoryCache dicomCache_;  // TODO
+
+    Semaphore largeDicomThrottler_;  // New in Orthanc 1.9.0 (notably for very large DICOM files in WSI)
+    ParsedDicomCache  dicomCache_;
 
     LuaScripting mainLua_;
     LuaScripting filterLua_;
@@ -232,29 +223,51 @@ namespace Orthanc
     BuiltinDecoderTranscoderOrder builtinDecoderTranscoderOrder_;
     bool isIngestTranscoding_;
     DicomTransferSyntax ingestTransferSyntax_;
+    bool ingestTranscodingOfUncompressed_;
+    bool ingestTranscodingOfCompressed_;
+
+    // New in Orthanc 1.9.0
+    DicomTransferSyntax preferredTransferSyntax_;
+    boost::mutex dynamicOptionsMutex_;
+    bool isUnknownSopClassAccepted_;
+    std::set<DicomTransferSyntax>  acceptedTransferSyntaxes_;
 
     StoreStatus StoreAfterTranscoding(std::string& resultPublicId,
                                       DicomInstanceToStore& dicom,
                                       StoreInstanceMode mode);
 
+    void ApplyInternal(ILookupVisitor& visitor,
+                       const DatabaseLookup& lookup,
+                       ResourceType queryLevel,
+                       size_t since,
+                       size_t limit);
+
+    void PublishDicomCacheMetrics();
+
+    // This DicomModification object is intended to be used as a
+    // "rules engine" when de-identifying logs for C-Find, C-Get, and
+    // C-Move queries (new in Orthanc 1.8.2)
+    DicomModification logsDeidentifierRules_;
+    bool              deidentifyLogs_;
+
   public:
     class DicomCacheLocker : public boost::noncopyable
     {
     private:
-      ServerContext& that_;
-      ParsedDicomFile *dicom_;
-      boost::mutex::scoped_lock lock_;
+      ServerContext&                               context_;
+      std::string                                  instancePublicId_;
+      std::unique_ptr<ParsedDicomCache::Accessor>  accessor_;
+      std::unique_ptr<ParsedDicomFile>             dicom_;
+      size_t                                       dicomSize_;
+      std::unique_ptr<Semaphore::Locker>           largeDicomLocker_;
 
     public:
-      DicomCacheLocker(ServerContext& that,
+      DicomCacheLocker(ServerContext& context,
                        const std::string& instancePublicId);
 
       ~DicomCacheLocker();
 
-      ParsedDicomFile& GetDicom()
-      {
-        return *dicom_;
-      }
+      ParsedDicomFile& GetDicom() const;
     };
 
     ServerContext(IDatabaseWrapper& database,
@@ -389,6 +402,7 @@ namespace Orthanc
 
     bool LookupOrReconstructMetadata(std::string& target,
                                      const std::string& publicId,
+                                     ResourceType level,
                                      MetadataType type);
 
 
@@ -492,5 +506,15 @@ namespace Orthanc
     {
       return transcodeDicomProtocol_;
     }
+
+    const std::string& GetDeidentifiedContent(const DicomElement& element) const;
+
+    void GetAcceptedTransferSyntaxes(std::set<DicomTransferSyntax>& syntaxes);
+
+    void SetAcceptedTransferSyntaxes(const std::set<DicomTransferSyntax>& syntaxes);
+
+    bool IsUnknownSopClassAccepted();
+
+    void SetUnknownSopClassAccepted(bool accepted);
   };
 }
