@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -55,6 +55,7 @@
 
 #include <boost/lexical_cast.hpp>
 #include <stdio.h>
+#include <stack>
 
 static const uint64_t MEGA_BYTES = 1024 * 1024;
 
@@ -86,8 +87,9 @@ namespace Orthanc
       FileContentType  type_;
 
     public:
-      FileToRemove(const FileInfo& info) : uuid_(info.GetUuid()), 
-                                           type_(info.GetContentType())
+      explicit FileToRemove(const FileInfo& info) :
+        uuid_(info.GetUuid()), 
+        type_(info.GetContentType())
       {
       }
 
@@ -120,8 +122,9 @@ namespace Orthanc
     }
 
   public:
-    Listener(ServerContext& context) : context_(context),
-                                       insideTransaction_(false)      
+    explicit Listener(ServerContext& context) :
+      context_(context),
+      insideTransaction_(false)      
     {
       Reset();
       assert(ResourceType_Patient < ResourceType_Study &&
@@ -176,7 +179,7 @@ namespace Orthanc
     virtual void SignalRemainingAncestor(ResourceType parentType,
                                          const std::string& publicId)
     {
-      VLOG(1) << "Remaining ancestor \"" << publicId << "\" (" << parentType << ")";
+      LOG(TRACE) << "Remaining ancestor \"" << publicId << "\" (" << parentType << ")";
 
       if (hasRemainingLevel_)
       {
@@ -203,9 +206,9 @@ namespace Orthanc
 
     virtual void SignalChange(const ServerIndexChange& change)
     {
-      VLOG(1) << "Change related to resource " << change.GetPublicId() << " of type " 
-              << EnumerationToString(change.GetResourceType()) << ": " 
-              << EnumerationToString(change.GetChangeType());
+      LOG(TRACE) << "Change related to resource " << change.GetPublicId() << " of type " 
+                 << EnumerationToString(change.GetResourceType()) << ": " 
+                 << EnumerationToString(change.GetChangeType());
 
       if (insideTransaction_)
       {
@@ -242,9 +245,9 @@ namespace Orthanc
     ServerIndex& index_;
     std::unique_ptr<IDatabaseWrapper::ITransaction> transaction_;
     bool isCommitted_;
-
+    
   public:
-    Transaction(ServerIndex& index) : 
+    explicit Transaction(ServerIndex& index) : 
       index_(index),
       isCommitted_(false)
     {
@@ -302,9 +305,9 @@ namespace Orthanc
     UnstableResourcePayload(Orthanc::ResourceType type,
                             const std::string& publicId) : 
       type_(type),
-      publicId_(publicId)
+      publicId_(publicId),
+      time_(boost::posix_time::second_clock::local_time())
     {
-      time_ = boost::posix_time::second_clock::local_time();
     }
 
     unsigned int GetAge() const
@@ -451,13 +454,13 @@ namespace Orthanc
 
     if (listener_->HasRemainingLevel())
     {
-      ResourceType type = listener_->GetRemainingType();
-      const std::string& uuid = listener_->GetRemainingPublicId();
+      ResourceType remainingType = listener_->GetRemainingType();
+      const std::string& remainingUuid = listener_->GetRemainingPublicId();
 
       target["RemainingAncestor"] = Json::Value(Json::objectValue);
-      target["RemainingAncestor"]["Path"] = GetBasePath(type, uuid);
-      target["RemainingAncestor"]["Type"] = EnumerationToString(type);
-      target["RemainingAncestor"]["ID"] = uuid;
+      target["RemainingAncestor"]["Path"] = GetBasePath(remainingType, remainingUuid);
+      target["RemainingAncestor"]["Type"] = EnumerationToString(remainingType);
+      target["RemainingAncestor"]["ID"] = remainingUuid;
     }
     else
     {
@@ -645,17 +648,20 @@ namespace Orthanc
     if (db_.LookupGlobalProperty(oldValue, property))
     {
       uint64_t oldNumber;
-
+      
       try
       {
         oldNumber = boost::lexical_cast<uint64_t>(oldValue);
-        db_.SetGlobalProperty(property, boost::lexical_cast<std::string>(oldNumber + 1));
-        return oldNumber + 1;
       }
       catch (boost::bad_lexical_cast&)
       {
-        throw OrthancException(ErrorCode_InternalError);
+        LOG(ERROR) << "Cannot read the global sequence "
+                   << boost::lexical_cast<std::string>(property) << ", resetting it";
+        oldNumber = 0;
       }
+
+      db_.SetGlobalProperty(property, boost::lexical_cast<std::string>(oldNumber + 1));
+      return oldNumber + 1;
     }
     else
     {
@@ -909,11 +915,17 @@ namespace Orthanc
         content.AddMetadata(status.studyId_, MetadataType_LastUpdate, now);
         content.AddMetadata(status.patientId_, MetadataType_LastUpdate, now);
 
-        if (status.isNewSeries_ &&
-            hasExpectedInstances)
+        if (status.isNewSeries_)
         {
-          content.AddMetadata(status.seriesId_, MetadataType_Series_ExpectedNumberOfInstances,
-                              boost::lexical_cast<std::string>(expectedInstances));
+          if (hasExpectedInstances)
+          {
+            content.AddMetadata(status.seriesId_, MetadataType_Series_ExpectedNumberOfInstances,
+                                boost::lexical_cast<std::string>(expectedInstances));
+          }
+
+          // New in Orthanc 1.9.0
+          content.AddMetadata(status.seriesId_, MetadataType_RemoteAet,
+                              instanceToStore.GetOrigin().GetRemoteAetC());
         }
 
         
@@ -921,7 +933,7 @@ namespace Orthanc
         // reflecting these additions into the input metadata map
         SetInstanceMetadata(content, instanceMetadata, instanceId,
                             MetadataType_Instance_ReceptionDate, now);
-        SetInstanceMetadata(content, instanceMetadata, instanceId, MetadataType_Instance_RemoteAet,
+        SetInstanceMetadata(content, instanceMetadata, instanceId, MetadataType_RemoteAet,
                             instanceToStore.GetOrigin().GetRemoteAetC());
         SetInstanceMetadata(content, instanceMetadata, instanceId, MetadataType_Instance_Origin, 
                             EnumerationToString(instanceToStore.GetOrigin().GetRequestOrigin()));
@@ -977,7 +989,7 @@ namespace Orthanc
               !value->IsBinary())
           {
             SetInstanceMetadata(content, instanceMetadata, instanceId,
-                                MetadataType_Instance_IndexInSeries, value->GetContent());
+                                MetadataType_Instance_IndexInSeries, Toolbox::StripSpaces(value->GetContent()));
           }
         }
 
@@ -1615,7 +1627,7 @@ namespace Orthanc
         throw OrthancException(ErrorCode_FullStorage);
       }
       
-      VLOG(1) << "Recycling one patient";
+      LOG(TRACE) << "Recycling one patient";
       db_.DeleteResource(patientToRecycle);
 
       if (!IsRecyclingNeeded(instanceSize))
@@ -1844,13 +1856,15 @@ namespace Orthanc
 
   bool ServerIndex::LookupMetadata(std::string& target,
                                    const std::string& publicId,
+                                   ResourceType expectedType,
                                    MetadataType type)
   {
     boost::mutex::scoped_lock lock(mutex_);
 
     ResourceType rtype;
     int64_t id;
-    if (!db_.LookupResource(id, rtype, publicId))
+    if (!db_.LookupResource(id, rtype, publicId) ||
+        rtype != expectedType)
     {
       throw OrthancException(ErrorCode_UnknownResource);
     }
@@ -1860,13 +1874,15 @@ namespace Orthanc
 
 
   void ServerIndex::GetAllMetadata(std::map<MetadataType, std::string>& target,
-                                   const std::string& publicId)
+                                   const std::string& publicId,
+                                   ResourceType expectedType)
   {
     boost::mutex::scoped_lock lock(mutex_);
 
     ResourceType type;
     int64_t id;
-    if (!db_.LookupResource(id, type, publicId))
+    if (!db_.LookupResource(id, type, publicId) ||
+        expectedType != type)
     {
       throw OrthancException(ErrorCode_UnknownResource);
     }
@@ -2465,7 +2481,7 @@ namespace Orthanc
   }
 
 
-  void ServerIndex::ReconstructInstance(ParsedDicomFile& dicom)
+  void ServerIndex::ReconstructInstance(const ParsedDicomFile& dicom)
   {
     DicomMap summary;
     OrthancConfiguration::DefaultExtractDicomSummary(summary, dicom);
