@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -56,6 +56,7 @@ static const char* const MEDIA_IMAGES_FOLDER = "IMAGES";
 static const char* const KEY_DESCRIPTION = "Description";
 static const char* const KEY_INSTANCES_COUNT = "InstancesCount";
 static const char* const KEY_UNCOMPRESSED_SIZE_MB = "UncompressedSizeMB";
+static const char* const KEY_ARCHIVE_SIZE_MB = "ArchiveSizeMB";
 static const char* const KEY_TRANSCODE = "Transcode";
 
 
@@ -240,7 +241,7 @@ namespace Orthanc
 
 
   public:
-    ArchiveIndex(ResourceType level) :
+    explicit ArchiveIndex(ResourceType level) :
       level_(level)
     {
     }
@@ -631,7 +632,7 @@ namespace Orthanc
     }
 
     virtual void Open(ResourceType level,
-                      const std::string& publicId)
+                      const std::string& publicId) ORTHANC_OVERRIDE
     {
       std::string path;
 
@@ -688,13 +689,13 @@ namespace Orthanc
       commands_.AddOpenDirectory(path.c_str());
     }
 
-    virtual void Close()
+    virtual void Close() ORTHANC_OVERRIDE
     {
       commands_.AddCloseDirectory();
     }
 
     virtual void AddInstance(const std::string& instanceId,
-                             const FileInfo& dicom)
+                             const FileInfo& dicom) ORTHANC_OVERRIDE
     {
       char filename[24];
       snprintf(filename, sizeof(filename) - 1, instanceFormat_, counter_);
@@ -722,16 +723,16 @@ namespace Orthanc
     }
 
     virtual void Open(ResourceType level,
-                      const std::string& publicId)
+                      const std::string& publicId) ORTHANC_OVERRIDE
     {
     }
 
-    virtual void Close()
+    virtual void Close() ORTHANC_OVERRIDE
     {
     }
 
     virtual void AddInstance(const std::string& instanceId,
-                             const FileInfo& dicom)
+                             const FileInfo& dicom) ORTHANC_OVERRIDE
     {
       // "DICOM restricts the filenames on DICOM media to 8
       // characters (some systems wrongly use 8.3, but this does not
@@ -851,6 +852,7 @@ namespace Orthanc
     currentStep_(0),
     instancesCount_(0),
     uncompressedSize_(0),
+    archiveSize_(0),
     transcode_(false),
     transferSyntax_(DicomTransferSyntax_LittleEndianImplicit)
   {
@@ -976,7 +978,7 @@ namespace Orthanc
       std::unique_ptr<TemporaryFile>   file_;
 
     public:
-      DynamicTemporaryFile(TemporaryFile* f) : file_(f)
+      explicit DynamicTemporaryFile(TemporaryFile* f) : file_(f)
       {
         if (f == NULL)
         {
@@ -993,10 +995,26 @@ namespace Orthanc
   }
   
 
+  void ArchiveJob::RefreshArchiveSize()
+  {
+    if (synchronousTarget_.get() != NULL)
+    {
+      archiveSize_ = synchronousTarget_->GetFileSize();
+    }
+        
+    if (asynchronousTarget_.get() != NULL)
+    {
+      archiveSize_ = asynchronousTarget_->GetFileSize();
+    }
+  }
+    
+
   void ArchiveJob::FinalizeTarget()
   {
     writer_.reset();  // Flush all the results
 
+    RefreshArchiveSize();
+    
     if (asynchronousTarget_.get() != NULL)
     {
       // Asynchronous behavior: Move the resulting file into the media archive
@@ -1017,7 +1035,7 @@ namespace Orthanc
       return JobStepResult::Failure(ErrorCode_NetworkProtocol,
                                     "A client has disconnected while creating an archive");
     }
-        
+
     if (writer_->GetStepsCount() == 0)
     {
       FinalizeTarget();
@@ -1036,6 +1054,7 @@ namespace Orthanc
       }
       else
       {
+        RefreshArchiveSize();
         return JobStepResult::Continue();
       }
     }
@@ -1077,6 +1096,8 @@ namespace Orthanc
     value[KEY_INSTANCES_COUNT] = instancesCount_;
     value[KEY_UNCOMPRESSED_SIZE_MB] =
       static_cast<unsigned int>(uncompressedSize_ / MEGA_BYTES);
+    value[KEY_ARCHIVE_SIZE_MB] =
+      static_cast<unsigned int>(archiveSize_ / MEGA_BYTES);
 
     if (transcode_)
     {
