@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -70,7 +70,7 @@ namespace Orthanc
     StorageCommitmentScpJob&  that_;
 
   public:
-    SetupCommand(StorageCommitmentScpJob& that) :
+    explicit SetupCommand(StorageCommitmentScpJob& that) :
       that_(that)
     {
     }
@@ -107,7 +107,8 @@ namespace Orthanc
                   size_t index) :
       that_(that),
       index_(index),
-      hasFailureReason_(false)
+      hasFailureReason_(false),
+      failureReason_(StorageCommitmentFailureReason_Success)
     {
     }
 
@@ -155,7 +156,7 @@ namespace Orthanc
     StorageCommitmentScpJob&  that_;
 
   public:
-    AnswerCommand(StorageCommitmentScpJob& that) :
+    explicit AnswerCommand(StorageCommitmentScpJob& that) :
       that_(that)
     {
       if (that_.ready_)
@@ -193,13 +194,13 @@ namespace Orthanc
     StorageCommitmentScpJob&  that_;
 
   public:
-    Unserializer(StorageCommitmentScpJob& that) :
+    explicit Unserializer(StorageCommitmentScpJob& that) :
       that_(that)
     {
       that_.ready_ = false;
     }
 
-    virtual ICommand* Unserialize(const Json::Value& source) const
+    virtual ICommand* Unserialize(const Json::Value& source) const ORTHANC_OVERRIDE
     {
       const std::string type = SerializationToolbox::ReadString(source, TYPE);
 
@@ -426,11 +427,18 @@ namespace Orthanc
   StorageCommitmentScpJob::StorageCommitmentScpJob(ServerContext& context,
                                                    const Json::Value& serialized) :
     SetOfCommandsJob(new Unserializer(*this), serialized),
-    context_(context)
+    context_(context),
+    transactionUid_(SerializationToolbox::ReadString(serialized, TRANSACTION_UID)),
+    calledAet_(SerializationToolbox::ReadString(serialized, CALLED_AET))
+    // "ready_" is initialized by the unserializer
   {
-    transactionUid_ = SerializationToolbox::ReadString(serialized, TRANSACTION_UID);
+    if (serialized.type() != Json::objectValue ||
+        !serialized.isMember(REMOTE_MODALITY))
+    {
+      throw OrthancException(ErrorCode_BadFileFormat);
+    }
+    
     remoteModality_ = RemoteModalityParameters(serialized[REMOTE_MODALITY]);
-    calledAet_ = SerializationToolbox::ReadString(serialized, CALLED_AET);
     SerializationToolbox::ReadArrayOfStrings(sopClassUids_, serialized, SOP_CLASS_UIDS);
     SerializationToolbox::ReadArrayOfStrings(sopInstanceUids_, serialized, SOP_INSTANCE_UIDS);
   }
