@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,39 +34,50 @@
 #include "PrecompiledHeadersServer.h"
 #include "OrthancRestApi/OrthancRestApi.h"
 
-#include <boost/algorithm/string/predicate.hpp>
-
 #include "../../OrthancFramework/Sources/Compatibility.h"
 #include "../../OrthancFramework/Sources/DicomFormat/DicomArray.h"
 #include "../../OrthancFramework/Sources/DicomNetworking/DicomAssociationParameters.h"
 #include "../../OrthancFramework/Sources/DicomNetworking/DicomServer.h"
 #include "../../OrthancFramework/Sources/DicomParsing/FromDcmtkBridge.h"
+#include "../../OrthancFramework/Sources/FileStorage/MemoryStorageArea.h"
 #include "../../OrthancFramework/Sources/HttpServer/FilesystemHttpHandler.h"
 #include "../../OrthancFramework/Sources/HttpServer/HttpServer.h"
 #include "../../OrthancFramework/Sources/Logging.h"
 #include "../../OrthancFramework/Sources/Lua/LuaFunctionCall.h"
 #include "../Plugins/Engine/OrthancPlugins.h"
+#include "Database/SQLiteDatabaseWrapper.h"
 #include "EmbeddedResourceHttpHandler.h"
 #include "OrthancConfiguration.h"
 #include "OrthancFindRequestHandler.h"
 #include "OrthancGetRequestHandler.h"
 #include "OrthancInitialization.h"
 #include "OrthancMoveRequestHandler.h"
+#include "OrthancWebDav.h"
 #include "ServerContext.h"
 #include "ServerJobs/StorageCommitmentScpJob.h"
 #include "ServerToolbox.h"
 #include "StorageCommitmentReports.h"
 
+#include <boost/algorithm/string/predicate.hpp>
+
+
 using namespace Orthanc;
 
 
+static const char* const KEY_DICOM_TLS_PRIVATE_KEY = "DicomTlsPrivateKey";
+static const char* const KEY_DICOM_TLS_ENABLED = "DicomTlsEnabled";
+static const char* const KEY_DICOM_TLS_CERTIFICATE = "DicomTlsCertificate";
+static const char* const KEY_DICOM_TLS_TRUSTED_CERTIFICATES = "DicomTlsTrustedCertificates";
+static const char* const KEY_MAXIMUM_PDU_LENGTH = "MaximumPduLength";
+
+
 class OrthancStoreRequestHandler : public IStoreRequestHandler
 {
 private:
   ServerContext& context_;
 
 public:
-  OrthancStoreRequestHandler(ServerContext& context) :
+  explicit OrthancStoreRequestHandler(ServerContext& context) :
     context_(context)
   {
   }
@@ -77,7 +88,7 @@ public:
                       const Json::Value& dicomJson,
                       const std::string& remoteIp,
                       const std::string& remoteAet,
-                      const std::string& calledAet) 
+                      const std::string& calledAet) ORTHANC_OVERRIDE 
   {
     if (dicomFile.size() > 0)
     {
@@ -102,7 +113,7 @@ private:
   ServerContext& context_;
   
 public:
-  OrthancStorageCommitmentRequestHandler(ServerContext& context) :
+  explicit OrthancStorageCommitmentRequestHandler(ServerContext& context) :
     context_(context)
   {
   }
@@ -112,7 +123,7 @@ public:
                              const std::vector<std::string>& referencedSopInstanceUids,
                              const std::string& remoteIp,
                              const std::string& remoteAet,
-                             const std::string& calledAet)
+                             const std::string& calledAet) ORTHANC_OVERRIDE
   {
     if (referencedSopClassUids.size() != referencedSopInstanceUids.size())
     {
@@ -140,7 +151,7 @@ public:
                             const std::vector<StorageCommitmentFailureReason>& failureReasons,
                             const std::string& remoteIp,
                             const std::string& remoteAet,
-                            const std::string& calledAet)
+                            const std::string& calledAet) ORTHANC_OVERRIDE
   {
     if (successSopClassUids.size() != successSopInstanceUids.size() ||
         failedSopClassUids.size() != failedSopInstanceUids.size() ||
@@ -174,14 +185,14 @@ class ModalitiesFromConfiguration : public DicomServer::IRemoteModalities
 {
 public:
   virtual bool IsSameAETitle(const std::string& aet1,
-                             const std::string& aet2) 
+                             const std::string& aet2) ORTHANC_OVERRIDE
   {
     OrthancConfiguration::ReaderLock lock;
     return lock.GetConfiguration().IsSameAETitle(aet1, aet2);
   }
 
   virtual bool LookupAETitle(RemoteModalityParameters& modality,
-                             const std::string& aet) 
+                             const std::string& aet) ORTHANC_OVERRIDE
   {
     OrthancConfiguration::ReaderLock lock;
     return lock.GetConfiguration().LookupDicomModalityUsingAETitle(modality, aet);
@@ -200,16 +211,16 @@ private:
   ServerContext& context_;
 
 public:
-  MyDicomServerFactory(ServerContext& context) : context_(context)
+  explicit MyDicomServerFactory(ServerContext& context) : context_(context)
   {
   }
 
-  virtual IStoreRequestHandler* ConstructStoreRequestHandler()
+  virtual IStoreRequestHandler* ConstructStoreRequestHandler() ORTHANC_OVERRIDE
   {
     return new OrthancStoreRequestHandler(context_);
   }
 
-  virtual IFindRequestHandler* ConstructFindRequestHandler()
+  virtual IFindRequestHandler* ConstructFindRequestHandler() ORTHANC_OVERRIDE
   {
     std::unique_ptr<OrthancFindRequestHandler> result(new OrthancFindRequestHandler(context_));
 
@@ -242,17 +253,17 @@ public:
     return result.release();
   }
 
-  virtual IMoveRequestHandler* ConstructMoveRequestHandler()
+  virtual IMoveRequestHandler* ConstructMoveRequestHandler() ORTHANC_OVERRIDE
   {
     return new OrthancMoveRequestHandler(context_);
   }
   
-  virtual IGetRequestHandler* ConstructGetRequestHandler()
+  virtual IGetRequestHandler* ConstructGetRequestHandler() ORTHANC_OVERRIDE
   {
     return new OrthancGetRequestHandler(context_);
   }
   
-  virtual IStorageCommitmentRequestHandler* ConstructStorageCommitmentRequestHandler()
+  virtual IStorageCommitmentRequestHandler* ConstructStorageCommitmentRequestHandler() ORTHANC_OVERRIDE
   {
     return new OrthancStorageCommitmentRequestHandler(context_);
   }
@@ -269,25 +280,43 @@ class OrthancApplicationEntityFilter : public IApplicationEntityFilter
 private:
   ServerContext&  context_;
   bool            alwaysAllowEcho_;
+  bool            alwaysAllowFind_;  // New in Orthanc 1.9.0
+  bool            alwaysAllowGet_;   // New in Orthanc 1.9.0
   bool            alwaysAllowStore_;
 
 public:
-  OrthancApplicationEntityFilter(ServerContext& context) :
+  explicit OrthancApplicationEntityFilter(ServerContext& context) :
     context_(context)
   {
-    OrthancConfiguration::ReaderLock lock;
-    alwaysAllowEcho_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowEcho", true);
-    alwaysAllowStore_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowStore", true);
+    {
+      OrthancConfiguration::ReaderLock lock;
+      alwaysAllowEcho_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowEcho", true);
+      alwaysAllowFind_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowFind", false);
+      alwaysAllowGet_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowGet", false);
+      alwaysAllowStore_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowStore", true);
+    }
+
+    if (alwaysAllowFind_)
+    {
+      LOG(WARNING) << "Security risk in DICOM SCP: C-FIND requests are always allowed, even from unknown modalities";
+    }
+
+    if (alwaysAllowGet_)
+    {
+      LOG(WARNING) << "Security risk in DICOM SCP: C-GET requests are always allowed, even from unknown modalities";
+    }
   }
 
   virtual bool IsAllowedConnection(const std::string& remoteIp,
                                    const std::string& remoteAet,
-                                   const std::string& calledAet)
+                                   const std::string& calledAet) ORTHANC_OVERRIDE
   {
     LOG(INFO) << "Incoming connection from AET " << remoteAet
               << " on IP " << remoteIp << ", calling AET " << calledAet;
 
     if (alwaysAllowEcho_ ||
+        alwaysAllowFind_ ||
+        alwaysAllowGet_ ||
         alwaysAllowStore_)
     {
       return true;
@@ -302,7 +331,7 @@ public:
   virtual bool IsAllowedRequest(const std::string& remoteIp,
                                 const std::string& remoteAet,
                                 const std::string& calledAet,
-                                DicomRequestType type)
+                                DicomRequestType type) ORTHANC_OVERRIDE
   {
     LOG(INFO) << "Incoming " << EnumerationToString(type) << " request from AET "
               << remoteAet << " on IP " << remoteIp << ", calling AET " << calledAet;
@@ -313,20 +342,50 @@ public:
       // Incoming C-Echo requests are always accepted, even from unknown AET
       return true;
     }
+    else if (type == DicomRequestType_Find &&
+             alwaysAllowFind_)
+    {
+      // Incoming C-Find requests are always accepted, even from unknown AET
+      return true;
+    }
     else if (type == DicomRequestType_Store &&
              alwaysAllowStore_)
     {
       // Incoming C-Store requests are always accepted, even from unknown AET
       return true;
     }
+    else if (type == DicomRequestType_Get &&
+             alwaysAllowGet_)
+    {
+      // Incoming C-Get requests are always accepted, even from unknown AET
+      return true;
+    }
     else
     {
       OrthancConfiguration::ReaderLock lock;
 
-      RemoteModalityParameters modality;
-      if (lock.GetConfiguration().LookupDicomModalityUsingAETitle(modality, remoteAet))
+      std::list<RemoteModalityParameters> modalities;
+      if (lock.GetConfiguration().LookupDicomModalitiesUsingAETitle(modalities, remoteAet))
       {
-        return modality.IsRequestAllowed(type);
+        if (modalities.size() == 1) // don't check the IP if there's only one modality with this AET
+        {
+          return modalities.front().IsRequestAllowed(type);
+        }
+        else // if there are multiple modalities with the same AET, check the one matching this IP
+        {
+          for (std::list<RemoteModalityParameters>::const_iterator it = modalities.begin(); it != modalities.end(); ++it)
+          {
+            if (it->GetHost() == remoteIp)
+            {
+              return it->IsRequestAllowed(type);
+            }
+          }
+
+          LOG(WARNING) << "Unable to check DICOM authorization for AET " << remoteAet
+                       << " on IP " << remoteIp << ", " << modalities.size()
+                       << " modalites found with this AET but none of them matching the IP";
+        }
+        return false;
       }
       else
       {
@@ -335,98 +394,21 @@ public:
     }
   }
 
-  virtual bool IsAllowedTransferSyntax(const std::string& remoteIp,
-                                       const std::string& remoteAet,
-                                       const std::string& calledAet,
-                                       TransferSyntax syntax)
+
+  virtual void GetAcceptedTransferSyntaxes(std::set<DicomTransferSyntax>& target,
+                                           const std::string& remoteIp,
+                                           const std::string& remoteAet,
+                                           const std::string& calledAet) ORTHANC_OVERRIDE
   {
-    std::string configuration;
-
-    switch (syntax)
-    {
-      case TransferSyntax_Deflated:
-        configuration = "DeflatedTransferSyntaxAccepted";
-        break;
-
-      case TransferSyntax_Jpeg:
-        configuration = "JpegTransferSyntaxAccepted";
-        break;
-
-      case TransferSyntax_Jpeg2000:
-        configuration = "Jpeg2000TransferSyntaxAccepted";
-        break;
-
-      case TransferSyntax_JpegLossless:
-        configuration = "JpegLosslessTransferSyntaxAccepted";
-        break;
-
-      case TransferSyntax_Jpip:
-        configuration = "JpipTransferSyntaxAccepted";
-        break;
-
-      case TransferSyntax_Mpeg2:
-        configuration = "Mpeg2TransferSyntaxAccepted";
-        break;
-
-      case TransferSyntax_Mpeg4:
-        configuration = "Mpeg4TransferSyntaxAccepted";
-        break;
-
-      case TransferSyntax_Rle:
-        configuration = "RleTransferSyntaxAccepted";
-        break;
-
-      default: 
-        throw OrthancException(ErrorCode_ParameterOutOfRange);
-    }
-
-    {
-      std::string name = "Is" + configuration;
-
-      LuaScripting::Lock lock(context_.GetLuaScripting());
-      
-      if (lock.GetLua().IsExistingFunction(name.c_str()))
-      {
-        LuaFunctionCall call(lock.GetLua(), name.c_str());
-        call.PushString(remoteAet);
-        call.PushString(remoteIp);
-        call.PushString(calledAet);
-        return call.ExecutePredicate();
-      }
-    }
-
-    {
-      OrthancConfiguration::ReaderLock lock;
-      return lock.GetConfiguration().GetBooleanParameter(configuration, true);
-    }
+    context_.GetAcceptedTransferSyntaxes(target);
   }
 
-
+  
   virtual bool IsUnknownSopClassAccepted(const std::string& remoteIp,
                                          const std::string& remoteAet,
-                                         const std::string& calledAet)
+                                         const std::string& calledAet) ORTHANC_OVERRIDE
   {
-    static const char* configuration = "UnknownSopClassAccepted";
-
-    {
-      std::string lua = "Is" + std::string(configuration);
-
-      LuaScripting::Lock lock(context_.GetLuaScripting());
-      
-      if (lock.GetLua().IsExistingFunction(lua.c_str()))
-      {
-        LuaFunctionCall call(lock.GetLua(), lua.c_str());
-        call.PushString(remoteAet);
-        call.PushString(remoteIp);
-        call.PushString(calledAet);
-        return call.ExecutePredicate();
-      }
-    }
-
-    {
-      OrthancConfiguration::ReaderLock lock;
-      return lock.GetConfiguration().GetBooleanParameter(configuration, false);
-    }
+    return context_.IsUnknownSopClassAccepted();
   }
 };
 
@@ -445,12 +427,22 @@ public:
   {
   }
 
+  virtual bool IsValidBearerToken(const std::string& token) ORTHANC_OVERRIDE
+  {
+#if ORTHANC_ENABLE_PLUGINS == 1
+    return (plugins_ != NULL &&
+            plugins_->IsValidAuthorizationToken(token));
+#else
+    return false;
+#endif    
+  }
+  
   virtual bool IsAllowed(HttpMethod method,
                          const char* uri,
                          const char* ip,
                          const char* username,
-                         const IHttpHandler::Arguments& httpHeaders,
-                         const IHttpHandler::GetArguments& getArguments)
+                         const HttpToolbox::Arguments& httpHeaders,
+                         const HttpToolbox::GetArguments& getArguments) ORTHANC_OVERRIDE
   {
 #if ORTHANC_ENABLE_PLUGINS == 1
     if (plugins_ != NULL &&
@@ -526,7 +518,7 @@ public:
   virtual void Format(HttpOutput& output,
                       const OrthancException& exception,
                       HttpMethod method,
-                      const char* uri)
+                      const char* uri) ORTHANC_OVERRIDE
   {
     {
       bool isPlugin = false;
@@ -592,7 +584,6 @@ public:
 };
 
 
-
 static void PrintHelp(const char* path)
 {
   std::cout 
@@ -602,8 +593,15 @@ static void PrintHelp(const char* path)
     << "The \"CONFIGURATION\" argument can be a single file or a directory. In the " << std::endl
     << "case of a directory, all the JSON files it contains will be merged. " << std::endl
     << "If no configuration path is given on the command line, a set of default " << std::endl
-    << "parameters is used. Please refer to the Orthanc homepage for the full " << std::endl
-    << "instructions about how to use Orthanc <http://www.orthanc-server.com/>." << std::endl
+    << "parameters is used. Please refer to the Orthanc Book for the full " << std::endl
+    << "instructions about how to use Orthanc <http://book.orthanc-server.com/>." << std::endl
+    << std::endl
+    << "Pay attention to the fact that the order of the options is important." << std::endl
+    << "Options are read left to right. In particular, options such as \"--verbose\" can " << std::endl
+    << "reset the value of other log-related options that were read before." << std::endl
+    << std::endl
+    << "The recommended set of options to debug DICOM communications is " << std::endl
+    << "\"--verbose --trace-dicom --logfile=dicom.log\"" << std::endl
     << std::endl
     << "Command-line options:" << std::endl
     << "  --help\t\tdisplay this help and exit" << std::endl
@@ -612,23 +610,40 @@ static void PrintHelp(const char* path)
     << "  --logfile=[file]\tfile where to store the log of Orthanc" << std::endl
     << "\t\t\t(by default, the log is dumped to stderr)" << std::endl
     << "  --config=[file]\tcreate a sample configuration file and exit" << std::endl
-    << "\t\t\t(if file is \"-\", dumps to stdout)" << std::endl
+    << "\t\t\t(if \"file\" is \"-\", dumps to stdout)" << std::endl
     << "  --errors\t\tprint the supported error codes and exit" << std::endl
     << "  --verbose\t\tbe verbose in logs" << std::endl
     << "  --trace\t\thighest verbosity in logs (for debug)" << std::endl
     << "  --upgrade\t\tallow Orthanc to upgrade the version of the" << std::endl
     << "\t\t\tdatabase (beware that the database will become" << std::endl
     << "\t\t\tincompatible with former versions of Orthanc)" << std::endl
-    << "  --no-jobs\t\tDon't restart the jobs that were stored during" << std::endl
+    << "  --no-jobs\t\tdon't restart the jobs that were stored during" << std::endl
     << "\t\t\tthe last execution of Orthanc" << std::endl
+    << "  --openapi=[file]\twrite the OpenAPI documentation and exit" << std::endl
+    << "\t\t\t(if \"file\" is \"-\", dumps to stdout)" << std::endl
+    << "  --cheatsheet=[file]\twrite the cheat sheet of REST API as CSV" << std::endl
+    << "\t\t\tand exit (if \"file\" is \"-\", dumps to stdout)" << std::endl
     << "  --version\t\toutput version information and exit" << std::endl
+    << std::endl
+    << "Fine-tuning of log categories:" << std::endl;
+
+  for (size_t i = 0; i < Logging::GetCategoriesCount(); i++)
+  {
+    const std::string name = Logging::GetCategoryName(i);
+    std::cout << "  --verbose-" << name
+              << "\tbe verbose in logs of category \"" << name << "\"" << std::endl;
+    std::cout << "  --trace-" << name
+              << "\tuse highest verbosity for logs of category \"" << name << "\"" << std::endl;
+  }
+  
+  std::cout
     << std::endl
     << "Exit status:" << std::endl
-    << "   0 if success," << std::endl
+    << "  0\tif success," << std::endl
 #if defined(_WIN32)
-    << "!= 0 if error (use the --errors option to get the list of possible errors)." << std::endl
+    << "  != 0\tif error (use the --errors option to get the list of possible errors)." << std::endl
 #else
-    << "  -1 if error (have a look at the logs)." << std::endl
+    << "  -1\tif error (have a look at the logs)." << std::endl
 #endif
     << std::endl;
 }
@@ -639,7 +654,7 @@ static void PrintVersion(const char* path)
   std::cout
     << path << " " << ORTHANC_VERSION << std::endl
     << "Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics Department, University Hospital of Liege (Belgium)" << std::endl
-    << "Copyright (C) 2017-2020 Osimis S.A. (Belgium)" << std::endl
+    << "Copyright (C) 2017-2021 Osimis S.A. (Belgium)" << std::endl
     << "Licensing GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>, with OpenSSL exception." << std::endl
     << "This is free software: you are free to change and redistribute it." << std::endl
     << "There is NO WARRANTY, to the extent permitted by law." << std::endl
@@ -710,6 +725,8 @@ static void PrintErrors(const char* path)
     PrintErrorCode(ErrorCode_CanceledJob, "This job was canceled");
     PrintErrorCode(ErrorCode_BadGeometry, "Geometry error encountered in Stone");
     PrintErrorCode(ErrorCode_SslInitialization, "Cannot initialize SSL encryption, check out your certificates");
+    PrintErrorCode(ErrorCode_DiscontinuedAbi, "Calling a function that has been removed from the Orthanc Framework");
+    PrintErrorCode(ErrorCode_BadRange, "Incorrect range request");
     PrintErrorCode(ErrorCode_SQLiteNotOpened, "SQLite: The database is not opened");
     PrintErrorCode(ErrorCode_SQLiteAlreadyOpened, "SQLite: Connection is already open");
     PrintErrorCode(ErrorCode_SQLiteCannotOpen, "SQLite: Unable to open the database");
@@ -782,15 +799,15 @@ static void PrintErrors(const char* path)
 #if ORTHANC_ENABLE_PLUGINS == 1
 static void LoadPlugins(OrthancPlugins& plugins)
 {
-  std::list<std::string> path;
+  std::list<std::string> pathList;
 
   {
     OrthancConfiguration::ReaderLock lock;
-    lock.GetConfiguration().GetListOfStringsParameter(path, "Plugins");
+    lock.GetConfiguration().GetListOfStringsParameter(pathList, "Plugins");
   }
 
   for (std::list<std::string>::const_iterator
-         it = path.begin(); it != path.end(); ++it)
+         it = pathList.begin(); it != pathList.end(); ++it)
   {
     std::string path;
 
@@ -809,7 +826,7 @@ static void LoadPlugins(OrthancPlugins& plugins)
 
 // Returns "true" if restart is required
 static bool WaitForExit(ServerContext& context,
-                        OrthancRestApi& restApi)
+                        const OrthancRestApi& restApi)
 {
   LOG(WARNING) << "Orthanc has started";
 
@@ -881,7 +898,7 @@ static bool WaitForExit(ServerContext& context,
 
 
 static bool StartHttpServer(ServerContext& context,
-                            OrthancRestApi& restApi,
+                            const OrthancRestApi& restApi,
                             OrthancPlugins* plugins)
 {
   bool httpServerEnabled;
@@ -993,12 +1010,61 @@ static bool StartHttpServer(ServerContext& context,
           lock.GetConfiguration().GetStringParameter("SslCertificate", "certificate.pem"));
         httpServer.SetSslEnabled(true);
         httpServer.SetSslCertificate(certificate.c_str());
+        
+        // Default to TLS 1.2 as SSL minimum
+        // See https://github.com/civetweb/civetweb/blob/master/docs/UserManual.md "ssl_protocol_version" for mapping
+        static const unsigned int TLS_1_2 = 4;
+        unsigned int minimumVersion = lock.GetConfiguration().GetUnsignedIntegerParameter("SslMinimumProtocolVersion", TLS_1_2);
+        httpServer.SetSslMinimumVersion(minimumVersion);
+
+        static const char* SSL_CIPHERS_ACCEPTED = "SslCiphersAccepted";
+
+        std::list<std::string> ciphers;
+
+        if (lock.GetJson().type() == Json::objectValue &&
+            lock.GetJson().isMember(SSL_CIPHERS_ACCEPTED))
+        {
+          lock.GetConfiguration().GetListOfStringsParameter(ciphers, SSL_CIPHERS_ACCEPTED);
+        }
+        else
+        {
+          // Defaults to FIPS 140-2 ciphers 
+          CLOG(INFO, HTTP) << "No configuration option \"" << SSL_CIPHERS_ACCEPTED
+                           << "\", will accept the FIPS 140-2 ciphers";
+
+          ciphers.push_back("ECDHE-ECDSA-AES256-GCM-SHA384");
+          ciphers.push_back("ECDHE-ECDSA-AES256-SHA384");
+          ciphers.push_back("ECDHE-RSA-AES256-GCM-SHA384");
+          ciphers.push_back("ECDHE-RSA-AES128-GCM-SHA256");
+          ciphers.push_back("ECDHE-RSA-AES256-SHA384");
+          ciphers.push_back("ECDHE-RSA-AES128-SHA256");
+          ciphers.push_back("ECDHE-RSA-AES128-SHA");
+          ciphers.push_back("ECDHE-RSA-AES256-SHA");
+          ciphers.push_back("DHE-RSA-AES256-SHA");
+          ciphers.push_back("DHE-RSA-AES128-SHA");
+          ciphers.push_back("AES256-SHA");
+          ciphers.push_back("AES128-SHA");
+        }
+        
+        httpServer.SetSslCiphers(ciphers);
       }
       else
       {
         httpServer.SetSslEnabled(false);
       }
 
+      if (lock.GetConfiguration().GetBooleanParameter("SslVerifyPeers", false))
+      {
+        std::string trustedClientCertificates = lock.GetConfiguration().InterpretStringParameterAsPath(
+          lock.GetConfiguration().GetStringParameter("SslTrustedClientCertificates", "trustedCertificates.pem"));
+        httpServer.SetSslVerifyPeers(true);
+        httpServer.SetSslTrustedClientCertificates(trustedClientCertificates.c_str());
+      }
+      else
+      {
+        httpServer.SetSslVerifyPeers(false);
+      }
+
       if (lock.GetConfiguration().GetBooleanParameter("ExecuteLuaEnabled", false))
       {
         context.SetExecuteLuaEnabled(true);
@@ -1010,6 +1076,16 @@ static bool StartHttpServer(ServerContext& context,
         context.SetExecuteLuaEnabled(false);
         LOG(WARNING) << "Remote LUA script execution is disabled";
       }
+
+      if (lock.GetConfiguration().GetBooleanParameter("WebDavEnabled", true))
+      {
+        const bool allowDelete = lock.GetConfiguration().GetBooleanParameter("WebDavDeleteAllowed", false);
+        const bool allowUpload = lock.GetConfiguration().GetBooleanParameter("WebDavUploadAllowed", true);
+        
+        UriComponents root;
+        root.push_back("webdav");
+        httpServer.Register(root, new OrthancWebDav(context, allowDelete, allowUpload));
+      }
     }
 
     MyHttpExceptionFormatter exceptionFormatter(httpDescribeErrors, plugins);
@@ -1038,7 +1114,7 @@ static bool StartHttpServer(ServerContext& context,
 
 
 static bool StartDicomServer(ServerContext& context,
-                             OrthancRestApi& restApi,
+                             const OrthancRestApi& restApi,
                              OrthancPlugins* plugins)
 {
   bool dicomServerEnabled;
@@ -1074,6 +1150,19 @@ static bool StartDicomServer(ServerContext& context,
       dicomServer.SetAssociationTimeout(lock.GetConfiguration().GetUnsignedIntegerParameter("DicomScpTimeout", 30));
       dicomServer.SetPortNumber(lock.GetConfiguration().GetUnsignedIntegerParameter("DicomPort", 4242));
       dicomServer.SetApplicationEntityTitle(lock.GetConfiguration().GetOrthancAET());
+
+      // Configuration of DICOM TLS for Orthanc SCP (since Orthanc 1.9.0)
+      dicomServer.SetDicomTlsEnabled(lock.GetConfiguration().GetBooleanParameter(KEY_DICOM_TLS_ENABLED, false));
+      if (dicomServer.IsDicomTlsEnabled())
+      {
+        dicomServer.SetOwnCertificatePath(
+          lock.GetConfiguration().GetStringParameter(KEY_DICOM_TLS_PRIVATE_KEY, ""),
+          lock.GetConfiguration().GetStringParameter(KEY_DICOM_TLS_CERTIFICATE, ""));
+        dicomServer.SetTrustedCertificatesPath(
+          lock.GetConfiguration().GetStringParameter(KEY_DICOM_TLS_TRUSTED_CERTIFICATES, ""));
+      }
+
+      dicomServer.SetMaximumPduLength(lock.GetConfiguration().GetUnsignedIntegerParameter(KEY_MAXIMUM_PDU_LENGTH, 16384));
     }
 
 #if ORTHANC_ENABLE_PLUGINS == 1
@@ -1157,10 +1246,25 @@ static bool ConfigureHttpHandler(ServerContext& context,
   FilesystemHttpHandler staticResources("/app", ORTHANC_PATH "/OrthancExplorer");
 #endif
 
-  context.GetHttpHandler().Register(staticResources, false);
+  // Do not register static resources if orthanc explorer is disabled
+  bool orthancExplorerEnabled = false;
+  {
+    OrthancConfiguration::ReaderLock lock;
+    orthancExplorerEnabled = lock.GetConfiguration().GetBooleanParameter(
+        "OrthancExplorerEnabled", true);
+  }
+
+  if (orthancExplorerEnabled)
+  {
+    context.GetHttpHandler().Register(staticResources, false);
+  }
+  else
+  {
+    LOG(WARNING) << "Orthanc Explorer UI is disabled";
+  }
 
   // Thirdly, consider the built-in REST API of Orthanc
-  OrthancRestApi restApi(context);
+  OrthancRestApi restApi(context, orthancExplorerEnabled);
   context.GetHttpHandler().Register(restApi, true);
 
   context.SetupJobsEngine(false /* not running unit tests */, loadJobsFromDatabase);
@@ -1307,6 +1411,15 @@ static bool ConfigureServerContext(IDatabaseWrapper& database,
     {
       LOG(WARNING) << "Setting option \"JobsHistorySize\" to zero is not recommended";
     }
+
+    // Configuration of DICOM TLS for Orthanc SCU (since Orthanc 1.9.0)
+    DicomAssociationParameters::SetDefaultOwnCertificatePath(
+      lock.GetConfiguration().GetStringParameter(KEY_DICOM_TLS_PRIVATE_KEY, ""),
+      lock.GetConfiguration().GetStringParameter(KEY_DICOM_TLS_CERTIFICATE, ""));
+    DicomAssociationParameters::SetDefaultTrustedCertificatesPath(
+      lock.GetConfiguration().GetStringParameter(KEY_DICOM_TLS_TRUSTED_CERTIFICATES, ""));
+    DicomAssociationParameters::SetDefaultMaximumPduLength(
+      lock.GetConfiguration().GetUnsignedIntegerParameter(KEY_MAXIMUM_PDU_LENGTH, 16384));
   }
   
   ServerContext context(database, storageArea, false /* not running unit tests */, maxCompletedJobs);
@@ -1341,7 +1454,7 @@ static bool ConfigureServerContext(IDatabaseWrapper& database,
   }
 
   {
-    ServerContextConfigurator configurator(context, plugins);
+    ServerContextConfigurator configurator(context, plugins);  // This calls "OrthancConfiguration::SetServerIndex()"
 
     {
       OrthancConfiguration::WriterLock lock;
@@ -1454,6 +1567,22 @@ static bool StartOrthanc(int argc,
 }
 
 
+static bool SetCategoryVerbosity(const Verbosity verbosity,
+                                 const std::string& category)
+{
+  Logging::LogCategory c;
+  if (LookupCategory(c, category))
+  {
+    SetCategoryVerbosity(c, verbosity);
+    return true;
+  }
+  else
+  {
+    return false;
+  }
+}
+
+
 static bool DisplayPerformanceWarning()
 {
   (void) DisplayPerformanceWarning;   // Disable warning about unused function
@@ -1465,6 +1594,7 @@ static bool DisplayPerformanceWarning()
 int main(int argc, char* argv[]) 
 {
   Logging::Initialize();
+  SetGlobalVerbosity(Verbosity_Default);
 
   bool upgradeDatabase = false;
   bool loadJobsFromDatabase = true;
@@ -1516,16 +1646,26 @@ int main(int argc, char* argv[])
     }
     else if (argument == "--verbose")
     {
-      Logging::EnableInfoLevel(true);
+      SetGlobalVerbosity(Verbosity_Verbose);
     }
     else if (argument == "--trace")
     {
-      Logging::EnableTraceLevel(true);
+      SetGlobalVerbosity(Verbosity_Trace);
+    }
+    else if (boost::starts_with(argument, "--verbose-") &&
+             SetCategoryVerbosity(Verbosity_Verbose, argument.substr(10)))
+    {
+      // New in Orthanc 1.8.1
+    }
+    else if (boost::starts_with(argument, "--trace-") &&
+             SetCategoryVerbosity(Verbosity_Trace, argument.substr(8)))
+    {
+      // New in Orthanc 1.8.1
     }
     else if (boost::starts_with(argument, "--logdir="))
     {
       // TODO WHAT IS THE ENCODING?
-      std::string directory = argument.substr(9);
+      const std::string directory = argument.substr(9);
 
       try
       {
@@ -1541,7 +1681,7 @@ int main(int argc, char* argv[])
     else if (boost::starts_with(argument, "--logfile="))
     {
       // TODO WHAT IS THE ENCODING?
-      std::string file = argument.substr(10);
+      const std::string file = argument.substr(10);
 
       try
       {
@@ -1594,6 +1734,91 @@ int main(int argc, char* argv[])
         return -1;
       }
     }
+    else if (boost::starts_with(argument, "--openapi="))
+    {
+      std::string target = argument.substr(10);
+
+      try
+      {
+        Json::Value openapi;
+
+        {
+          SQLiteDatabaseWrapper inMemoryDatabase;
+          inMemoryDatabase.Open();
+          MemoryStorageArea inMemoryStorage;
+          ServerContext context(inMemoryDatabase, inMemoryStorage, true /* unit testing */, 0 /* max completed jobs */);
+          OrthancRestApi restApi(context, false /* no Orthanc Explorer */);
+          restApi.GenerateOpenApiDocumentation(openapi);
+          context.Stop();
+        }
+
+        openapi["info"]["version"] = ORTHANC_VERSION;
+        openapi["info"]["title"] = "Orthanc API";
+        openapi["info"]["description"] =
+          "This is the full documentation of the [REST API](https://book.orthanc-server.com/users/rest.html) "
+          "of Orthanc.<p>This reference is automatically generated from the source code of Orthanc. A "
+          "[shorter cheat sheet](https://book.orthanc-server.com/users/rest-cheatsheet.html) is part of "
+          "the Orthanc Book.<p>An earlier, manually crafted version from August 2019, is [still available]"
+          "(2019-08-orthanc-openapi.html), but is not up-to-date anymore ([source]"
+          "(https://groups.google.com/g/orthanc-users/c/NUiJTEICSl8/m/xKeqMrbqAAAJ)).";
+
+        Json::Value server = Json::objectValue;
+        server["url"] = "https://demo.orthanc-server.com/";
+        openapi["servers"].append(server);
+        
+        std::string s;
+        Toolbox::WriteStyledJson(s, openapi);
+
+        if (target == "-")
+        {
+          std::cout << s;   // Print to stdout
+        }
+        else
+        {
+          SystemToolbox::WriteFile(s, target);
+        }
+        return 0;
+      }
+      catch (OrthancException&)
+      {
+        LOG(ERROR) << "Cannot export OpenAPI documentation as file \"" << target << "\"";
+        return -1;
+      }
+    }
+    else if (boost::starts_with(argument, "--cheatsheet="))
+    {
+      std::string target = argument.substr(13);
+
+      try
+      {
+        std::string cheatsheet;
+
+        {
+          SQLiteDatabaseWrapper inMemoryDatabase;
+          inMemoryDatabase.Open();
+          MemoryStorageArea inMemoryStorage;
+          ServerContext context(inMemoryDatabase, inMemoryStorage, true /* unit testing */, 0 /* max completed jobs */);
+          OrthancRestApi restApi(context, false /* no Orthanc Explorer */);
+          restApi.GenerateReStructuredTextCheatSheet(cheatsheet, "https://api.orthanc-server.com/index.html");
+          context.Stop();
+        }
+
+        if (target == "-")
+        {
+          std::cout << cheatsheet;   // Print to stdout
+        }
+        else
+        {
+          SystemToolbox::WriteFile(cheatsheet, target);
+        }
+        return 0;
+      }
+      catch (OrthancException&)
+      {
+        LOG(ERROR) << "Cannot export REST cheat sheet as file \"" << target << "\"";
+        return -1;
+      }
+    }
     else
     {
       LOG(WARNING) << "Option unsupported by the core of Orthanc: " << argument;
