@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -37,6 +37,7 @@
 #include "../../OrthancFramework/Sources/Compatibility.h"
 #include "../../OrthancFramework/Sources/FileStorage/MemoryStorageArea.h"
 #include "../../OrthancFramework/Sources/JobsEngine/Operations/LogJobOperation.h"
+#include "../../OrthancFramework/Sources/Logging.h"
 #include "../../OrthancFramework/Sources/SerializationToolbox.h"
 
 #include "../Sources/Database/SQLiteDatabaseWrapper.h"
@@ -350,12 +351,12 @@ static bool CheckIdempotentSerialization(IJobUnserializer& unserializer,
 
 
 static bool CheckIdempotentSerialization(IJobUnserializer& unserializer,
-                                         JobOperationValue& value)
+                                         IJobOperationValue& value)
 {
   Json::Value a = 42;
   value.Serialize(a);
   
-  std::unique_ptr<JobOperationValue> unserialized(unserializer.UnserializeValue(a));
+  std::unique_ptr<IJobOperationValue> unserialized(unserializer.UnserializeValue(a));
   
   Json::Value b = 43;
   unserialized->Serialize(b);
@@ -478,6 +479,23 @@ TEST(JobsSerialization, DicomInstanceOrigin)
     ASSERT_FALSE(origin.LookupCalledAet(t));
     ASSERT_FALSE(origin.LookupHttpUsername(t));
   }
+
+  {
+    DicomInstanceOrigin origin(DicomInstanceOrigin::FromWebDav());
+
+    s = 42;
+    origin.Serialize(s);
+  }
+
+  {
+    DicomInstanceOrigin origin(s);
+    ASSERT_EQ(RequestOrigin_WebDav, origin.GetRequestOrigin());
+    ASSERT_EQ("", std::string(origin.GetRemoteAetC()));
+    ASSERT_FALSE(origin.LookupRemoteIp(t));
+    ASSERT_FALSE(origin.LookupRemoteAet(t));
+    ASSERT_FALSE(origin.LookupCalledAet(t));
+    ASSERT_FALSE(origin.LookupHttpUsername(t));
+  }
 }
 
 
@@ -541,9 +559,9 @@ TEST_F(OrthancJobsSerialization, Values)
     instance.Serialize(s);
   }
 
-  std::unique_ptr<JobOperationValue> value;
+  std::unique_ptr<IJobOperationValue> value;
   value.reset(unserializer.UnserializeValue(s));
-  ASSERT_EQ(JobOperationValue::Type_DicomInstance, value->GetType());
+  ASSERT_EQ(IJobOperationValue::Type_DicomInstance, value->GetType());
   ASSERT_EQ(id, dynamic_cast<DicomInstanceOperationValue&>(*value).GetId());
 
   {
@@ -618,7 +636,7 @@ TEST_F(OrthancJobsSerialization, Operations)
       modality.SetApplicationEntityTitle("REMOTE");
       modality.SetHost("192.168.1.1");
       modality.SetPortNumber(1000);
-      modality.SetManufacturer(ModalityManufacturer_StoreScp);
+      modality.SetManufacturer(ModalityManufacturer_GE);
 
       StoreScuOperation operation(GetContext(), luaManager, "TEST", modality);
 
@@ -633,7 +651,7 @@ TEST_F(OrthancJobsSerialization, Operations)
       ASSERT_EQ("REMOTE", tmp.GetRemoteModality().GetApplicationEntityTitle());
       ASSERT_EQ("192.168.1.1", tmp.GetRemoteModality().GetHost());
       ASSERT_EQ(1000, tmp.GetRemoteModality().GetPortNumber());
-      ASSERT_EQ(ModalityManufacturer_StoreScp, tmp.GetRemoteModality().GetManufacturer());
+      ASSERT_EQ(ModalityManufacturer_GE, tmp.GetRemoteModality().GetManufacturer());
       ASSERT_EQ("TEST", tmp.GetLocalAet());
     }
   }
@@ -704,7 +722,7 @@ TEST_F(OrthancJobsSerialization, Jobs)
     modality.SetApplicationEntityTitle("REMOTE");
     modality.SetHost("192.168.1.1");
     modality.SetPortNumber(1000);
-    modality.SetManufacturer(ModalityManufacturer_StoreScp);
+    modality.SetManufacturer(ModalityManufacturer_GE);
 
     DicomModalityStoreJob job(GetContext());
     job.SetLocalAet("LOCAL");
@@ -724,7 +742,7 @@ TEST_F(OrthancJobsSerialization, Jobs)
     ASSERT_EQ("REMOTE", tmp.GetParameters().GetRemoteModality().GetApplicationEntityTitle());
     ASSERT_EQ("192.168.1.1", tmp.GetParameters().GetRemoteModality().GetHost());
     ASSERT_EQ(1000, tmp.GetParameters().GetRemoteModality().GetPortNumber());
-    ASSERT_EQ(ModalityManufacturer_StoreScp, tmp.GetParameters().GetRemoteModality().GetManufacturer());
+    ASSERT_EQ(ModalityManufacturer_GE, tmp.GetParameters().GetRemoteModality().GetManufacturer());
     ASSERT_TRUE(tmp.HasMoveOriginator());
     ASSERT_EQ("MOVESCU", tmp.GetMoveOriginatorAet());
     ASSERT_EQ(42, tmp.GetMoveOriginatorId());
