@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -117,7 +117,6 @@ TEST(EnumerationDictionary, ServerEnumerations)
   ASSERT_STREQ("Generic", EnumerationToString(StringToModalityManufacturer("Generic")));
   ASSERT_STREQ("GenericNoWildcardInDates", EnumerationToString(StringToModalityManufacturer("GenericNoWildcardInDates")));
   ASSERT_STREQ("GenericNoUniversalWildcard", EnumerationToString(StringToModalityManufacturer("GenericNoUniversalWildcard")));
-  ASSERT_STREQ("StoreScp", EnumerationToString(StringToModalityManufacturer("StoreScp")));
   ASSERT_STREQ("Vitrea", EnumerationToString(StringToModalityManufacturer("Vitrea")));
   ASSERT_STREQ("GE", EnumerationToString(StringToModalityManufacturer("GE")));
   // backward compatibility tests (to remove once we make these manufacturer really obsolete)
@@ -127,6 +126,11 @@ TEST(EnumerationDictionary, ServerEnumerations)
   ASSERT_STREQ("Generic", EnumerationToString(StringToModalityManufacturer("Dcm4Chee")));
   ASSERT_STREQ("GenericNoWildcardInDates", EnumerationToString(StringToModalityManufacturer("SyngoVia")));
   ASSERT_STREQ("GenericNoWildcardInDates", EnumerationToString(StringToModalityManufacturer("AgfaImpax")));
+
+  ASSERT_STREQ("default", EnumerationToString(StringToVerbosity("default")));
+  ASSERT_STREQ("verbose", EnumerationToString(StringToVerbosity("verbose")));
+  ASSERT_STREQ("trace", EnumerationToString(StringToVerbosity("trace")));
+  ASSERT_THROW(StringToVerbosity("nope"), OrthancException);
 }
 
 
@@ -144,7 +148,7 @@ TEST(FontRegistry, Basic)
   }
 
   Orthanc::PngWriter w;
-  w.WriteToFile("UnitTestsResults/font.png", s);
+  Orthanc::IImageWriter::WriteToFile(w, "UnitTestsResults/font.png", s);
 }
 
 
@@ -510,7 +514,7 @@ int main(int argc, char **argv)
 {
   Logging::Initialize();
   Toolbox::InitializeGlobalLocale(NULL);
-  Logging::EnableInfoLevel(true);
+  SetGlobalVerbosity(Verbosity_Verbose);
   Toolbox::DetectEndianness();
   SystemToolbox::MakeDirectory("UnitTestsResults");
   OrthancInitialize();
